C     PROGRAM MFLOPS(TAPE6=OUTPUT)
C                             LATEST FILE MODIFICATION DATE:  22 Dec 86     h
C****************************************************************************
C MEASURES CPU PERFORMANCE RANGE OF THE COMPUTER/COMPILER/COMPUTATION COMPLEX
C****************************************************************************
C                                                                           *
C     L. L. N. L.   F O R T R A N   K E R N E L S:   M F L O P S            *
C                                                                           *
C     These kernels measure  Fortran  numerical  computation rates for a    *
C     spectrum of  CPU-limited  computational  structures.  Mathematical    *
C     through-put is measured  in  units  of  millions of floating-point    *
C     operations executed per second, called Megaflops/sec.                 *
C                                                                           *
C     This program  measures  a realistic  CPU performance range for the    *
C     Fortran programming system  on  a  given day.  The CPU performance    *
C     rates depend  strongly  on  the maturity of the Fortran compiler's    *
C     ability to translate Fortran code into efficient machine code.        *
C                                                                           *
C     [ The CPU hardware  capability  apart  from  compiler maturity (or    *
C     availability), could be measured (or simulated) by programming the    *
C     kernels in assembly  or machine code directly.  These measurements    *
C     can also  serve  as a framework for tracking the maturation of the    *
C     Fortran compiler during system development.]                          *
C                                                                           *
C     While this test evaluates  the  performance of a broad sampling of    *
C     Fortran computations,  it  is not an application program and hence    *
C     it is not a benchmark  per  se.  The performance of benchmarks and    *
C     even workloads,  if  CPU  limited,  could  be roughly estimated by    *
C     choosing appropriate weights  and loop limits for each kernel (see    *
C     Block Data).                                                          *
C                                                                           *
C     Use of this program is granted with the request that a copy of the    *
C     results be sent to  the  author  at the address shown below, to be    *
C     added to  our studies of computer performance.  The timing results    *
C     will be held as  proprietary  data  if  so marked.  In return, you    *
C     will receive a copy of our latest report.                             *
C                                                                           *
C                                                                           *
C          F.H. McMahon    L-35                                             *
C          Lawrence Livermore National Laboratory                           *
C          P.0. Box 808                                                     *
C          Livermore, CA                                                    *
C          94550                                                            *
C                                                                           *
C                                                                           *
C                    (C) Copyright 1983 the Regents of the                  *
C                University of California. All Rights Reserved.             *
C                                                                           *
C               This work was produced under the sponsorship of             *
C                the U.S. Department of Energy. The Government              *
C                       retains certain rights therein.                     *
C                                                                           *
C****************************************************************************
C
C
C
C
C
C
C
C                             DIRECTIONS
C
C  1. We REQUIRE one run of the Fortran kernels as is, that is, with
C     no reprogramming.  Standard product compiler directives may be
C     used for optimization as these do not constitute reprogramming.
C
C     In addition, the vendor may, if so desired, reprogram the kernels to
C     demonstrate high performance hardware features.  Kernels 13,14,23
C     are partially vectorisable and kernels 15,16,24 are vectorisable if
C     re-written. Kernels 5,11,17,19,20 are implicit computations that
C     must not be explicitly vectorised using compiler directives to
C     ignore dependencies.  In any case, compiler listings of the codes
C     actually used should be returned along with the timing results.
C
C  2. For vector processors, we REQUIRE an ALL scalar-compilation run
C     to measure the basic scalar performance range of the processor.
C
C  3. On computers where default single precision is REAL*4 we REQUIRE
C     an additional run with all mantissas.ge.47  i.e. declare all real
C     variables REAL*8 using one of the following declarations in each routine:
C
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
c     ( Then there is one redundant declaration in subrs SIGNAL and SUMO.)
C
C  4. Installation includes verifying or changing the following:
C
C      First :  the I/O output device number= IOU assignment in MAIN.
C      Second:  the definition of function SECOND for CPU time only, and
C               the value of TIC:= minimum cpu clock time(sec) in SIZES.
C      Third :  the definition of function MOD2N in KERNEL
C      Fourth:  the system names Komput, Kontrl, and Kompil in REPORT
C      Fifth :  after checkout set Nruns=7 in SIZES for Standard Benchmark Test
C
C  5. Each kernel's computation is check-summed for easy validation.
C     Verify correct processing using the checksums in subroutine REPORT
C     which were computed setting  Loop= 10*Loop  in subroutine SIZES.
C     Your checksums should compare to the precision used, within round-off.
C
C  6. Verify CPU Time measurements from function SECOND by comparing the clock
C     calibration printout of total CPU time with system or real-time measures.
C     The accuracy of SECOND is also tested using the test routine VERIFY.
C
C  7. On computers with Virtual Storage Systems assure a working-set space
C     larger than the entire program so that page faults are negligible,
C     because we must measure the CPU-limited computation rates.
C     IT IS ALSO NECESSARY to run this test stand-alone, i.e. NO timesharing.
C     In VS Systems a series of runs are needed to show stable CPU timings.
C
C  8. On parallel computer systems which compile vectors or Multi-tasking
C     at the Do-loop level (Micro-tasking) parallelisation of the first
C     DO (on L) in each kernel must be prevented by using a compiler directive
C     or by setting Loop= 1. This outermost DO Loop is merely repitition
C     used to increase timing accuracy and could distort the computation
C     sample if parallelisation is based on this artificial iteration level.
C
C  9. On computers with Cache memories and high resolution CPU clocks we
C     may ask for an addtional run setting repitition Loop= 1 in SIZES,
C     to show un-initialized as well as encached execution rates.
C     Increase the size of array CACHE(in subr. VALUES) from 8192 to cache size.
C****************************************************************************
C
C
C
c
C/      PARAMETER( kn= 47, kn2= 95, np= 3, ls= 3*47, krs= 24)
C/      PARAMETER( nk= 47, nl= 3, nr= 8 )
c
      COMMON /ALPHA/ mk,ik,ml,il,Nruns,jr, NPFS(8,3,47)
      DIMENSION  FLOPS(141), TR(141), RATES(141), ID(141)
      DIMENSION  LSPAN(141), WG(141), OSUM (141), TERR(141)
c
      REAL*8     SECOND
c
              t= SECOND(0.0D0)
            iou= 6
C          OPEN  (UNIT=6, FILE='MF.OUT', STATUS='NEW')
cLLNL      call  Q8EBM
cLLNL      call  PFM( 0, iou)
c
c                        Record name in active linkage chain in COMMON /DEBUG/
           CALL  TRACKS(' MAIN.  ')
c
c                        Verify Sufficient Loop Size Versus Cpu Clock Accuracy
           CALL  VERIFY( iou)
c
c                        Define control limits:  Nruns(runs), Loop(time), tic,
           CALL  SIZES(-1)
c
c
c                        Run test Nruns times Cpu-limited; I/O is deferred:
      DO 1    k= 1,Nruns
             jr= k
c                        Run test using one of 3 sets of DO-Loop spans:
c                        Set iou Negative to supress all I/O during Cpu timing.
      DO 1    j= 1,ml
             il= j
           tock= TICK( -iou)
c
           CALL  KERNEL
    1 continue
c
c
c
c                        Report timing errors, Mflops statistics:
      DO 2    j= 1,ml
             il= j
           CALL  RESULT( iou,FLOPS,TR,RATES,LSPAN,WG,OSUM,TERR,ID)
c
           CALL  REPORT( iou,   mk,mk,FLOPS,TR,RATES,LSPAN,WG,OSUM,ID)
    2 continue
c
           CALL  REPORT( iou,3*mk,mk,FLOPS,TR,RATES,LSPAN,WG,OSUM,ID)
c
c
              t= SECOND(0.0D0) - t
          WRITE( iou,9) t
    9    FORMAT( 1H1,//,26H CHECK CLOCK CALIBRATION: ,/,
     .                  18H Total cpu Time = ,e14.5, 5H Sec. )
      STOP
c
c
c
c
c
c
c
c
c
c
c          Subroutine timing of all-scalar execution on CRAY-1:
c
c          Subroutine     Time(%)
c
c          KERNEL         43.46%
c          SUPPLY         21.82%
c          VERIFY         13.12%
c          STATS           8.83%
c          SQRT            1.84%
c          SORDID          1.21%
c          VALUES           .74%
c          SUMO             .47%
c          SIGNAL           .34%
c          IQRANF           .26%
c          STATW            .17%
c
      END
c***********************************************
      BLOCK DATA
C***********************************************
C
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
      DOUBLE PRECISION  SUMS
C
C     l1 :=  param-dimension governs the size of most 1-d arrays
C     l2 :=  param-dimension governs the size of most 2-d arrays
C
C  ISPAN :=  Array of limits for DO loop control in the kernels
C  IPASS :=  Array of limits for multiple pass execution of each kernel
C  FLOPN :=  Array of floating-point operation counts for one pass thru kernel
C     WT :=  Array of weights to average kernel execution rates.
C  SKALE :=  Array of scale factors for SIGNAL data generator.
C   BIAS :=  Array of scale factors for SIGNAL data generator.
C
C    MUL :=  Array of multipliers * FLOPN  for each pass
C    WTP :=  Array of multipliers *    WT  for each pass
C     FR :=  Array of vectorisation fractions in REPORT
C   SUMW :=  Array of quartile weights in REPORT
C     IQ :=  Array of workload weights in REPORT
C   SUMS :=  Array of Verified Checksums of Kernels results: Nruns= 1 and 7.
C
C/      PARAMETER( l1= 1001, l2=  101, l1d= 2*1001 )
C/      PARAMETER( l13=  64, l13h= l13/2, l213= l13+l13h, l813= 8*l13 )
C/      PARAMETER( l14=2048, l16=  75, l416= 4*l16 , l21= 25 )
C
C/      PARAMETER( l1=   27, l2=   15, l1d= 2*1001 )
C/      PARAMETER( l13=   8, l13h= 8/2, l213= 8+4, l813= 8*8 )
C/      PARAMETER( l14=  16, l16= 15, l416= 4*15 , l21= 15)
C
C
C/      PARAMETER( l1=   1001, l2=   101, l1d= 2*1001 )
C/      PARAMETER( l13= 64, l13h= 64/2, l213= 64+32, l813= 8*64 )
C/      PARAMETER( l14= 2048, l16= 75, l416= 4*75 , l21= 25)
C
C/      PARAMETER( kn= 47, kn2= 95, np= 3, ls= 3*47, krs= 24)
C/      PARAMETER( m1= 1001-1, m2= 101-1, m7= 1001-6 )
C
      COMMON /SPACES/ ion,j5,k2,k3,Loop,m,kr,it,n13h,ibuf,
     1                n,n1,n2,n13,n213,n813,n14,n16,n416,n21,nt1,nt2
C
      COMMON /SPACE0/ TIME(47), CSUM(47), WW(47), WT(47), ticks,
     1                FR(9), TERR1(47), SUMW(7), START,
     2              SKALE(47), BIAS(47), WS(95), TOTAL(47), FLOPN(47),
     3                IQ(7), NPF, NPFS1(47)
C
      COMMON /SPACEI/ WTP(3), MUL(3), ISPAN(47,3), IPASS(47,3)
C
      COMMON /ORDER/ index, match, NSTACK(20)
C
      COMMON /PROOF/  SUMS(24,3,2)
C     ****************************************************************
C
      DATA  ( ISPAN(i,1), i= 1,47) /
     : 1001, 101, 1001, 1001, 1001, 64, 995, 100,
     : 101, 101, 1001, 1000, 64, 1001, 101, 75,
     : 101, 100, 101, 1000, 101, 101, 100, 1001, 23*0/
C
C*   : l1, l2, l1, l1, l1, l13, m7, m2,
C*   : l2, l2, l1, m1, l13, l1, l2, l16,
C*   : l2, m2, l2, m1, l21, l2, m2, l1, 23*0/
C
      DATA  ( ISPAN(i,2), i= 1,47) /
     : 101, 101, 101, 101, 101,  32, 101, 100,
     : 101, 101, 101, 100,  32, 101, 101,  40,
     : 101, 100, 101, 100,  50, 101, 100, 101,  23*0/
C
      DATA  ( ISPAN(i,3), i= 1,47) /
     : 27, 15, 27, 27, 27,  8, 21, 14,
     : 15, 15, 27, 26,  8, 27, 15, 15,
     : 15, 14, 15, 26, 20, 15, 14, 27,  23*0/
C
      DATA  ( IPASS(i,1), i= 1,47) /
     :   7, 67,  9, 14, 10,  3,  4, 10, 36, 34, 11, 12,
     :  36,  2,  1, 25, 35,  2, 39,  1,  1, 11,  8,  5,  23*0/
C
      DATA  ( IPASS(i,2), i= 1,47) /
     :   40, 40, 53, 70, 55,  7, 22,  6, 21, 19, 64, 68,
     :   41, 10,  1, 27, 20,  1, 23,  8,  1,  7,  5, 31,  23*0/
C
      DATA  ( IPASS(i,3), i= 1,47) /
     :   28, 46, 37, 38, 40, 21, 20,  9, 26, 25, 46, 48,
     :   31,  8,  1, 14, 26,  2, 28,  7,  1,  8,  7, 23,  23*0/
C
      DATA  (  MUL(i), i= 1,3) / 1, 2, 8 /
      DATA  (  WTP(i), i= 1,3) / 1.0, 2.0, 1.0 /
c
c     The following flop-counts (FLOPN) are required for scalar or serial
c     execution.  The scalar version defines the NECESSARY computation
c     generally, in the absence of proof to the contrary.  The vector
c     or parallel executions are only credited with executing the same
c     necessary computation.  If the parallel methods do more computation
c     than is necessary then the extra flops are not counted as through-put.
c
      DATA  ( FLOPN(i), i= 1,47)
     :     /5., 4., 2., 2., 2., 2., 16., 36., 17., 9., 1., 1.,
     :     7., 11., 33., 7., 9., 44., 6., 26., 2., 17., 11., 1., 23*0.0/
C
      DATA  ( WT(i), i= 1,47) /
     : 1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,
     : 1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,
     : 1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0, 23*0.0/
C
C/   :  .08, .04, .02, .03, .03, .04, .10, .05, .04, .03,                   HLN
C/   :  .01, .02, .03, .02, .03, .05, .03, .20, .02, .02,                   HLN
C/   :  .03, .03, .04, .01, 23*0.0/                                         HLN
C
      DATA  ( SKALE(i), i= 1,47) /
     & 0.100D0, 0.100D0, 0.100D0, 0.100D0, 0.100D0, 0.100D0,
     & 0.100D0, 0.100D0, 0.100D0, 0.100D0, 0.100D0, 0.100D0,
     & 0.100D0, 0.100D0, 0.100D0, 0.100D0, 0.100D0, 0.100D0,
     & 0.100D0, 0.100D0, 0.100D0, 0.100D0, 0.100D0, 0.100D0,
     &       23*0.000D0 /
C
c    : 0.1,  0.1,  0.1,  0.1,  0.1,  0.1,  0.1,  0.1,
c    : 0.1,  0.1,  0.1,  0.1,  0.1,  0.1,  0.1,  0.1,
c    : 0.1,  0.1,  0.1,  0.1,  0.1,  0.1,  0.1,  0.1,  23*0.0/
C
      DATA  ( BIAS(i), i= 1,47) /
     : 0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,
     : 0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,
     : 0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  0.0,  23*0.0/
C
      DATA  ( FR(i), i= 1,9) /
     :  0.0, 0.2, 0.4, 0.6, 0.7, 0.8, 0.9, 0.95, 1.0/
C
      DATA  ( SUMW(i), i= 1,7) /
     : 1.0, 0.95, 0.9, 0.8, 0.7, 0.6, 0.5/
C
      DATA  ( IQ(i), i= 1,7) /
     : 1, 2, 1, 2, 1, 2, 1/
C
      DATA  START /0.0/, NPF/0/, ibuf/0/, match/0/
C
      DATA  ( SUMS(i,1,1), i= 1,24 ) /
     &.5114652693224705D+05,.5150345372943066D+03,.1000742883066623D+02,
     &.5999250595474070D+00,.4548871642388544D+04,.5229095383954675D+13,
     &.6104251075163778D+05,.1501268005627157D+06,.1189443609975085D+06,
     &.7310369784325972D+05,.3342910972650531D+08,.2907141428639174D-04,
     &.4057110454105263D+10,.2982036205992255D+10,.3943816690352311D+05,
     &.2832600000000000D+05,.1114641772903091D+04,.5165625410757306D+05,
     &.5421816960150398D+03,.3040644339317275D+08,.8289464835786202D+07,
     &.2938604376567099D+03,.3549834542446150D+05,.5000000000000000D+03/
c
      DATA  ( SUMS(i,2,1), i= 1,24 ) /
     &.5253344778938000D+03,.5150345372943066D+03,.1009741436579188D+01,
     &.5999250595474070D+00,.4589031939602131D+02,.2693280957416549D+16,
     &.6345586315772524D+03,.1501268005627157D+06,.1189443609975085D+06,
     &.7310369784325972D+05,.3433560407476162D+05,.7127569144561925D-05,
     &.2325318944820836D+10,.3045676741897511D+08,.3943816690352311D+05,
     &.3244100000000000D+05,.1114641772903091D+04,.5165625410757306D+05,
     &.5421816960150398D+03,.3126205178811007D+05,.3986531136462291D+07,
     &.2938604376567099D+03,.3549894609776936D+05,.5000000000000000D+02/
c
      DATA  ( SUMS(i,3,1), i= 1,24 ) /
     &.3855104502494983D+02,.1199847611437483D+02,.2699309089321296D+00,
     &.5999250595474070D+00,.3182615248448271D+01,.8303480073326955D+12,
     &.2845720217638848D+02,.2960543667877649D+04,.2623968460874419D+04,
     &.1651291227698377D+04,.6551161335846537D+03,.1943435981776804D-05,
     &.4755211251524563D+09,.2547733008933910D+07,.1108997288135066D+04,
     &.2577600000000000D+05,.2947368618590713D+02,.9700646212341513D+03,
     &.1268230698051747D+02,.5987713249471801D+03,.2516870081042209D+07,
     &.6109968728264795D+01,.4850340602751675D+03,.1300000000000000D+02/
c
      DATA  ( SUMS(i,1,2), i= 1,7 ) /
     &.2982036205992255D+10,.6118901630090488D+10,.9103526877478772D+10,
     &.1215176334476067D+11,.1519764492169999D+11,.1820312504465359D+11,
     &.2116750694993432D+11/
c
      DATA  ( SUMS(i,2,2), i= 1,7 ) /
     &.3045676741897511D+08,.5718526521576222D+08,.8885029941358330D+08,
     &.1174925822726987D+09,.1501582054695641D+09,.1819691693283694D+09,
     &.2130649341195080D+09/
c
      DATA  ( SUMS(i,3,2), i= 1,7 ) /
     &.2547733008933910D+07,.5131714230651644D+07,.7946120246231201D+07,
     &.1008019578807808D+08,.1269997234773526D+08,.1504905863862026D+08,
     &.1721399839433381D+08/
      END
C
C***********************************************
      SUBROUTINE INDEX
C***********************************************
C
C       MODULE     PURPOSE
C       ------     -----------------------------------------------
C       IQRANF     computes a vector of pseudo-random indices
C
C       KERNEL     executes 24 samples of Fortran computation
C
C       PFM        optional call to system hardware performance monitor
C
C       REPORT     prints timing results
C
C       RESULT     computes execution rates  into pushdown store
C
C       SECOND     cumulative CPU time for task in seconds (MKS units)
C
C       SENSIT     sensitivity analysis of harmonic mean to 49 workloads
C
C       SIGNAL     generates a set of floating-point numbers near 1.0
C
C       SIMD       sensitivity analysis of harmonic mean to SISD/SIMD model
C
C       SIZES      test and set the loop controls before each kernel test
C
C       SORDID     simple sort
C
C       SPACE      sets memory pointers for array variables.  optional.
C
C       STATS      calculates unweighted statistics
C
C       STATW      calculates   weighted statistics
C
C       SUMO       check-sum with ordinal dependency
C
C       SUPPLY     initializes common blocks containing type real arrays.
C
C       TALLY      computes average and minimum Cpu timings and variances.
C
C       TDIGIT     counts lead digits followed by trailing zeroes
C
C       TEST       times, tests, and initializes each kernel test
C
C       TICK       measures timing overhead of subroutine test
C
C       TILE       computes  m-tile value and corresponding index
C
C       TRACKS,TRACKX   push/pop caller's name and serial nr. in /DEBUG/
C
C       TRAP       checks that index-list values are in valid domain
C
C       VALID      compresses valid timing results
C
C       VALUES     initializes special values
C
C       VERIFY     verifies sufficient Loop size versus cpu clock accuracy
C       ------     -----------------------------------------------
c
c  ------------ -------- -------- -------- -------- -------- --------
c  ENTRY LEVELS:   1        2        3        4        5        6
c  ------------ -------- -------- -------- -------- -------- --------
c               MAIN.    SECOND
c                        VERIFY   SECOND
c                                 SIZES
c                                 STATS    SQRT
c                                 TDIGIT   LOG10
c                        SIZES
c
c                        TICK     TEST     SECOND
c                                          SIZES
c                                          SUMO
c                                          VALUES   SUPPLY   SIGNAL
c                                                   IQRANF   MOD
c                                 VALID    TRAP              TRAP
c                                 STATS    SQRT
c                                 IQRANF   MOD
c                                          TRAP
c                        KERNEL   SPACE
c                                 SQRT
c                                 EXP
c                                 TEST     SECOND
c                                          SIZES
c                                          SUMO
c                                          VALUES   SUPPLY   SIGNAL
c                                                   IQRANF   MOD
c                        RESULT   TALLY    SIZES             TRAP
c                                          PAGE
c                                          STATS    SQRT
c                                 LOG10
c
c                        REPORT   VALID    TRAP
c                                 MOD
c                                 STATW    SORDID   TRAP
c                                          TILE
c                                          SQRT
c                                          LOG10
c                                 PAGE
c                                 TRAP
c                                 SENSIT   VALID    TRAP
c                                          SORDID   TRAP
c                                          PAGE
c                                          STATW    SORDID   TRAP
c                                                   TILE
c                                 SIMD     VALID    TRAP
c                                          STATW    SORDID   TRAP
c                                                   TILE
C                        STOP
C
C
C
C
C
C
C
C
C
C
C
C
c    ------ ---- ------     -----   ------------------------------------
c    BASE   TYPE CLASS      NAME    GLOSSARY
c    ------ ---- ------     -----   ------------------------------------
c    SPACE0    R Array      BIAS  - scale factors for SIGNAL data generator
c    SPACE0    R Array      CSUM  - checksums of KERNEL result arrays
c    BETA      R Array      CSUMS - sets of CSUM for all test runs
c    BETA      R Array      DOS   - sets of TOTAL flops for all test runs
c    SPACE0    R Array      FLOPN - flop counts for one execution pass
c    BETA      R Array      FOPN  - sets of FLOPN for all test runs
c    SPACE0    R Array      FR    - vectorisation fractions; abscissa for REPORT
c    SPACES    I scalar     ibuf  - flag enables one call to SIGNAL
c    ALPHA     I scalar     ik    - current number of executing kernel
c    ALPHA     I scalar     il    - selects one of three sets of loop spans
c    SPACES    I scalar     ion   - logical I/O unit number for output
c    SPACEI    I Array      IPASS - Loop control limits for multiple-pass loops
c    SPACE0    I Array      IQ    - set of workload weights for REPORT
c    SPACEI    I Array      ISPAN - loop control limits for each kernel
c    SPACES    I scalar     it    - flags timing call to TEST from TICK
c    SPACES    I scalar     j5    - datum in kernel 16
c    ALPHA     I scalar     jr    - current test run number (1 thru 7)
c    SPACES    I scalar     k2    - counter in kernel 16
c    SPACES    I scalar     k3    - counter in kernel 16
c    SPACES    I scalar     kr    - a copy of mk
c    SPACES    I scalar     Loop  - current multiple-pass loop limit in KERNEL
c    SPACES    I scalar     m     - temp integer datum
c    ALPHA     I scalar     mk    - number of kernels to evaluate .LE.24
c    ALPHA     I scalar     ml    - maximum value of il=  3
c    SPACEI    I Array      MUL   - multipliers * IPASS defines Loop
c    SPACES    I scalar     n     - current DO loop limit in KERNEL
c    SPACES    I scalar     n1    - dimension of most 1-D arrays
c    SPACES    I scalar     n13   - dimension used in kernel 13
c    SPACES    I scalar     n13h  - dimension used in kernel 13
c    SPACES    I scalar     n14   - dimension used in kernel 14
c    SPACES    I scalar     n16   - dimension used in kernel 16
c    SPACES    I scalar     n2    - dimension of most 2-D arrays
c    SPACES    I scalar     n21   - dimension used in kernel 21
c    SPACES    I scalar     n213  - dimension used in kernel 21
c    SPACES    I scalar     n416  - dimension used in kernel 16
c    SPACES    I scalar     n813  - dimension used in kernel 13
c    SPACE0    I scalar     npf   - temp integer datum
c    ALPHA     I Array      NPFS  - sets of NPFS1 for all test runs
c    SPACE0    I Array      NPFS1 - number of page-faults for each kernel
c    ALPHA     I scalar     Nruns - number of complete test runs
c    SPACES    I scalar     nt1   - total size of common -SPACE1- words
c    SPACES    I scalar     nt2   - total size of common -SPACE2- words
c    BETA      R Array      SEE   - (i,1,jr,il) sets of TEST overhead times
c    BETA      R Array      SEE   - (i,2,jr,il) sets of csums of SPACE1
c    BETA      R Array      SEE   - (i,3,jr,il) sets of csums of SPACE2
c    SPACE0    R Array      SKALE - scale factors for SIGNAL data generator
c    SPACE0    R scalar     start - temp start time of each kernel
c    PROOF     R Array      SUMS  - sets of verified checksums for all test runs
c    SPACE0    R Array      SUMW  - set of quartile weights for REPORT
c    SPACE0    R Array      TERR1 - overhead-time errors for each kernel
c    BETA      R Array      TERRS - sets of TERR1 for all runs
c    BETA      R scalar     tic   - minimum cpu clock time= resolution
c    SPACE0    R scalar     ticks - average overhead time in TEST linkage
c    SPACE0    R Array      TIME  - net execution times for all kernels
c    BETA      R Array      TIMES - sets of TIME for all test runs
c    SPACE0    R Array      TOTAL - total flops computed by each kernel
c    SPACE0    R Array      WS    - unused
c    SPACE0    R Array      WT    - weights for each kernel sample
c    SPACEI    R Array      WTP   - weights for the 3 span-varying passes
c    SPACE0    R Array      WW    - unused
C
C
c  --------- -----------------------------------------------------------------
c   COMMON   Usage
c  --------- -----------------------------------------------------------------
C
C   /ALPHA /
C            VERIFY    TICK      TALLY     SIZES     RESULT    REPORT    KERNEL
C            MAIN.
C   /BASE1 /
C            SUPPLY
C   /BASE2 /
C            SUPPLY
C   /BASER /
C            SUPPLY
C   /BETA  /
C            TICK      TALLY     SIZES     RESULT    REPORT    KERNEL
C   /DEBUG /
C            TRACKS    TRACKX    TRAP
C   /ORDER /
C            TRACKS    TRACKX    TRAP
C   /PROOF /
C            RESULT    BLOCKDATA
C   /SPACE0/
C            VALUES    TICK      TEST      TALLY     SUPPLY    SIZES     RESULT
C            REPORT    KERNEL    BLOCKDATA
C   /SPACE1/
C            VERIFY    VALUES    TICK      TEST      SUPPLY    SPACE     KERNEL
C   /SPACE2/
C            VERIFY    VALUES    TICK      TEST      SUPPLY    SPACE     KERNEL
C   /SPACE3/
C            VALUES
C   /SPACEI/
C            VERIFY    VALUES    TICK      TEST      SIZES     RESULT    REPORT
C            KERNEL    BLOCKDATA
C   /SPACER/
C            VALUES    TICK      TEST      SUPPLY    SIZES     KERNEL
C   /SPACES/
C            VERIFY    VALUES    TICK      TEST      SUPPLY    SIZES     KERNEL
C            BLOCKDATA
c  --------- -----------------------------------------------------------------
      RETURN
      END
C
C
C***************************************
      SUBROUTINE IQRANF( M, Mmin,Mmax, n)
C***********************************************************************
C                                                                      *
c     IQRANF  - computes a vector of psuedo-random indices             *
c               in the domain (Mmin,Mmax)                              *
C                                                                      *
C     M     - result array ,  psuedo-random positive integers          *
C     Mmin  - input integer,  lower bound for random integers          *
C     Mmax  - input integer,  upper bound for random integers          *
C     n     - input integer,  number of results in M.                  *
C                                                                      *
C       M(i)= Mmin + INT( (Mmax-Mmin) * RANF(0))                       *
C                                                                      *
c        CALL IQRANF( IX, 1,1001, 30)      should produce in IX:       *
c           3  674  435  415  389   54   44  790  900  282             *
c         177  971  728  851  687  604  815  971  155  112             *
c         877  814  779  192  619  894  544  404  496  505  ...        *
C                                                                      *
C***********************************************************************
C
      IMPLICIT  DOUBLE PRECISION (A-H,K,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,K,O-Z)
      DOUBLE PRECISION  dq, dp, per, dk, spin, span
C
      dimension  M(n)
      save k
      DATA  k /256/
      CALL TRACKS('IQRANF  ')
            IF( n.LE.0 )  GO TO 73
         inset= Mmin
          span= Mmax - Mmin
c         spin= 16807.00D0
c          per= 2147483647.00D0
          spin= 16807
           per= 2147483647
         realn= n
         scale= 1.0000100D0
             q= scale*(span/realn)
C
            dk= k
      DO  1  i= 1,n
            dp= dk*spin
c           dk=    DMOD( dp, per)
            dk= dp -INT( dp/per)*per
            dq= dk*span
          M(i)= inset + ( dq/ per)
            IF( M(i).LT.Mmin .OR. M(i).GT.Mmax )  M(i)= inset + i*q
    1 continue
             k= dk
C
C
ciC     double precision  k, ip, iq, id
ci         inset= Mmin
ci         ispan= Mmax - Mmin
ci         ispin= 16807
ci            id= 2147483647
ci             q= (DBLE(ispan)/DBLE(n))*1.00001
ciC
ci      DO  2  i= 1,n
ci            ip= k*ispin
ci             k=      MOD( ip, id)
ci            iq= k*ispan
ci          M(i)= inset + ( iq/ id)
ci            IF( M(i).LT.Mmin .OR. M(i).GT.Mmax )  M(i)= inset + i*q
ci    2 continue
C
      CALL TRAP( M, 8H IQRANF  , 1, Mmax, n)
C
   73 CONTINUE
      CALL TRACKX
      RETURN
      END
C***********************************************
      SUBROUTINE  PAGE( iou)
C***********************************************
      CALL TRACKS('PAGE    ')
      WRITE(iou,1)
    1 FORMAT(1H1)
c   1 FORMAT(1H)
      CALL TRACKX
      RETURN
      END
C***********************************************
      SUBROUTINE REPORT( iou, ntk,nek,FLOPS,TR,RATES,LSPAN,WG,OSUM,ID)
C***********************************************************************
C                                                                      *
C     REPORT -  Prints Statistical Evaluation Of Fortran Kernel Timings*
C                                                                      *
C     iou    -  Logical Output Device Number                           *
C     ntk    -  Total number of Kernels to Edit in Report              *
C     nek    -  Number of Effective Kernels in each set to Edit        *
C     FLOPS  -  Array:  Number of Flops executed by each kernel        *
C     TR     -  Array:  Time of execution of each kernel(microsecs)    *
C     RATES  -  Array:  Rate of execution of each kernel(megaflops/sec)*
C     LSPAN  -  Array:  Span of inner DO loop in each kernel           *
C     WG     -  Array:  Weight assigned to each kernel for statistics  *
C     OSUM   -  Array:  Checksums of the results of each kernel        *
C                                                                      *
C***********************************************************************
C
C
C                                REFERENCE
C
C          F.H.McMahon, The Livermore Fortran Kernels:  A Computer
C          Test Of The Numerical Performance Range, Lawrence
C          Livermore National Laboratory, Livermore, California,
C          UCRL-53745, October 1986.
C
C
C
C
C                                NOTICE
C
C               "This report was  prepared  as an account
C               of work  sponsored  by  the United States
C               Government.  Neither  the  United  States
C               nor the   United   States  Department  of
C               Energy, nor any  of  their employees, nor
C               any of their contractors, subcontractors,
C               or their employees,  makes  any warranty,
C               express or  implied, or assumes any legal
C               liability or   responsibility   for   the
C               accuracy, completeness  or  usefulness of
C               any information,  apparatus,  product  or
C               process disclosed, or represents that its
C               use would  not  infringe  privateiy-owned
C               rights."
C
C               Reference to  a  company  or product name
C               does not impiy approval or recommendation
C               of the   product  by  the  University  of
C               California or  the   U.S. Department   of
C               Energy to  the  exclusion  of others that
C               may be suitable.
C
C
C               Work performed under  the auspices of the
C               U.S. Department of Energy by the Lawrence
C               Livermore Laboratory    under    contract
C               number W-7405-ENG-48.
C
C***********************************************************************
C
C
C  I: FORTRAN CPU PERFORMANCE ANALYSIS                  F.H.McMahon
C
C
c     These kernels measure  Fortran numerical computation rates for a
c     spectrum of  CPU-limited computational structures or benchmarks.
c     The kernels benchmark  contains  extracts  or  kernels from more
c     than a score CPU-limited scientific application programs.  These
c     kernels are The most  important  CPU  time  components  from The
c     application programs.   This  benchmark  may  be easily extended
c     with important new kernels leaving performance statistics intact.
c
c     The time required  to  convert,  debug,  execute  and time many,
c     entire, large  programs  on  new  machines  each  having  a  new
c     implementation of  Fortran,   or   several   implementations  or
c     dialects rapidly  becomes  excessive.  Almost all The conversion
c     costs are in segments  of  The programs which are irrelevant for
c     evaluation of  The  CPU,  e.g.,  I/O, Fortran variations, memory
c     allocation, overlays,   job  control,   etc.    all   of   these
c     complexities are reduced to a single, small benchmark which uses
c     a minimum of I/O  and  a  single level of storage.  further, the
c     computation in  the  kernels  is  the  most  stable  part of the
c     Fortran language.
c
c     The kernels benchmark  is sufficient to determine a range of CPU
c     performance for  many  different  computational  structures in a
c     single computer run.   Since The range in performance is usually
c     large the  mean  has  a secondary significance.  To estimate the
c     performance of  a particular,  CPU-limited  application  program
c     select the  case(s) which are most similar to the application as
c     most relevent to the  estimate.   The  performance  ratio  of  a
c     kernel on  two  different  machines or compiled by two different
c     compilers on the same  machine  will  approximate  the  ratio of
c     through-puts for   an  application  which  is  very  similar  in
c     structure.
c
c     This set of kernels was chosen to measure lower and upper bounds
c     for scalar Fortran computation rates.  The upper bound on scalar
c     rates serves as a base  to  evaluate the effectiveness of vector
c     computation.  The  kind  of  Fortran  which  has the highest MIP
c     rates is pure arithmetic  in  DO-loops where complete local code
c     optimization by a Fortran compiler is possible.  All other kinds
c     of Fortran  operations  execute  at  much  lower  MIP  rates  on
c     multiple register machines (these ops may not be necessary).
c
c     Through-put is  measured  in  units of floating-point operations
c     executed per micro-second;  called  results  per micro-second or
c     mega-flops.  The  Mflop is a measure of the NECESSARY results in
c     a scientific application  program  regardless  of  the number or
c     kind of  operations  or processing.  The ratio of Mflops for two
c     different machines will approximate  the  ratio  of through-puts
c     for the  majority  of compute-limited scientific applications on
c     the two  machines.    The   kernels  measure  performance  scale
c     factors.
c
C
C
c
c -----------------------------------------------------------------------------
c II:  A CPU Performance Metric For Computational Physics:    Mega-Flops/sec.
c -----------------------------------------------------------------------------
c                                                                 F.H. McMahon
c
c A:  Floating-Point Instructions:  The Necessary Mathematics
c
c Computational physics applies  systems  of  PDEs from Mathematical physics to
c simulate the  evolution of physical systems.  The mathematical methods depend
c on real  valued  functions   and   the   algorithms  are  programmed,  almost
c exclusively, in  Fortran  Floating-point  computer operations (Flops).  These
c floating-point operations  are,  unquestionably,   the   NECESSARY   computer
c operations on  ANY  computer  and  the  total  number  is  INVARIANT.  Thus a
c meaningful computation rate can  always  be  measured  by  counting the total
c number of Flops and dividing by the total execution time of a program.
c
c B:   Procedural Machine  Instructions:   Artifices Of An Archetecture
c
c All of  the non-arithmetic instructions in a machine program are artifices of
c a particular hardware  architecture,  i.e.  machine dependant, as well as the
c result of  a  particular compiler's imperfect coding techniques.  How many of
c these procedural machine  instructions  are  strictly  necessary  can only be
c determined by further, tedious analysis which is ALWAYS machine dependant.  A
c famous example  of software  masking  hardware  capabilities  is  the  PASCAL
c compiler written  by  n.Wirth  which  used  only  50%  of  the command set to
c generate machine programs for the CDC-7600.
c
c Unless the next generation computer design is constrained for some reason, to
c closely resemble  its  obsolete  predecessor,  the  instruction  mix  used in
c current machines is not necessarily  relevent.   Furthermore, the instruction
c mix is  not  a  definitive  characterization  of the intrinsic physics or the
c mathematical algorithms.
c
c  1.  Primary Memory Access Instructions
c
c The number of memory  instructions  that  are necessary for a given algorithm
c depends strongly  on  the  number  and  kind of CPU registers and is a highly
c machine dependent number.   Operating registers, scratch-pad memories, vector
c buffers, short-stop  and  feed-back paths in the cpu are examples of hardware
c artifices which reduce the  number  of  primary memory operations.  Compilers
c and other  coders must make intelligent use of these particular cpu resources
c to minimize memory operations  and this is generally not the case, as is well
c known.
c
c  2.  Branching Instructions
c
c Branching instructions   are   the  slowest  and  most  expensive  procedural
c instructions and are very  often  unecessary.  Here the source programmer has
c primary responsibility  to  minimize  branching in the program by avoiding IF
c statements whenever possible  by  using MAX, MIN, or merge functions like
c CSMG.  Careful  logical  reduction  and  placement of IF tests is required to
c minimize the execution of branching operations.  Compilers can do very little
c to change or optimize the branch graph specified in the source program.
c
c On vector  computers ALL IF tests over mesh or array (state) variables can be
c eliminated.  Conditional computation can be vectorized by direct construction
c using explicit  sub-set mappings.  Vector relationals replace the IF clauses.
c Then sparse,  one-to-one   mappings  called  vector  Compress/Decompress  and
c one-to-many mappings   called   vector   Gather/Scatter   are  necessary  and
c sufficient to compose sub-vector operands for simple vector operations.
c
c
C
CIII: FORTRAN PROGRAMMING SYSTEM MATURITY
c
c     Hardware performance   gains   depend   criticaly   on  compiler
c     maturity.  These  kernels   measure  the  joint  performance  of
c     hardware and  Fortran  compiler  software and may easily be used
c     for a comparative  analysis  of  all  the available compilers or
c     options on a given machine.  For a new or proposed machine where
c     no compiler is available  the  performance  may  be estimated by
c     simulating a  reasonable  compilation.  An example of simulation
c     rationale is given below.
c
c     Fortran compilers for new  types  of  machines require a lengthy
c     development cycle  to  achive  an  effective  level  of  machine
c     utilization.  A fully  mature  compiler  may not be completed in
c     the first  years  of  a  new machine.  Indeed, maturity is not a
c     stationary state   but  evolves   with   advances   in   program
c     optimization techniques.   Some  of  these  techniques depend on
c     special facilities in the  new  machines and serious development
c     and implementation cannot start much earlier than development of
c     the new machine.   Assumptions  on  the  maturity  of  available
c     Fortran compilers  are  crucial  to  the  evaluation  of Fortran
c     performance and  thus,   compiler   characteristics   should  be
c     explicit parameters of the performance analysis.
c
C
C  A: OPTIMIZATION CRITERIA FOR SIMULATING A MODERATELY MATURE COMPILER
c
c     The coding is deliberately simplified to the level of complexity
c     of a moderately  mature  compiler,  i.e.   a proper compiler for
c     processors with  segmented cpu design circa 1975.  The coding is
c     deliberately not lossless,  ideal  assembly  language since this
c     technique may  outperform  Fortran  by more than a factor of two
c     and may over estimate typical Fortran performance.
c
c       0. reduction of redundant operations in the source text.
c
c       1. all scalar values are defined in registers over the
c          scope of loops.
c
c       2. all scripted input variables are loaded into registers
c          at the begining of loops.
c
c       3. code optimization techniques include:
c
c            3.1 effective address computation for subscripted
c                variables which are a function of the loop variable
c                is optimized by:
c            3.1.1   reduction to recursive address functions.
c            3.1.2   indicial address and increment as well as
c                    current index are register values.
c            3.2 moderate scheduling of instructions.
c                not pert scheduling level, hence not best ordering.
c            3.3 loop controls are defined in registers.
c                specific, minimum loop branch tests.
c            3.4 pre-loading operands for the next loop iteration
c                was not allowed.  all operations in a loop have
c                the same phase, i.e. are the same iteration.
c
c       4. memory bank conflicts were ignored.
c
c       5. branch ratios on if conditionals are equal, then
c          averaged for two cases.
c
C
C  B: OPTIMIZATION CRITERIA FOR A MATURE COMPILER INCLUDE:
c
c       0. minimization of effective address computation
c          especially in loops.
c
c       1. optimal register allocation; ranked by usage.
c
c       2. critical path(pert) time schedule for each block of
c          instructions on machines with segmented function units.
c
c       3. global program optimization, viz IBM Fortran-H. implies
c          reduction of redundant operations in object code string.
c          important reductions for programs using dynamic addressing
c          and dynamically dimensioned arrays.
c
c       4. decomposition of iterative scalar Fortran into parallel
c          vector or array operations where these exist as parallel
c          machine operations.
c
c       5. recomposition of scalar Fortran loops that cannot be
c          decomposed as in (4.) into multi-phase loops.  the goal
c          is to eliminate serial fetch delays by forward-fetching
c          of scalar operands, i.e. memory references in a loop are
c          for operands in the next iteration.
c
c
c
C  C: EXTENSIONS TO THE SET OF KERNELS
C
c     The kernels  benchmarks  were  extended from 14 to 20 kernels in
c     1976 , largely as a result of our experiences in vectorizing old
c     applications.  They   are   intended   to   compensate  for  the
c     oversimplicity of some of  the original kernels which are mostly
c     elementary matrix  operations  and  thus,  to help represent the
c     Fortran workload  more  accurately.    The   new   kernels  have
c     significant improvements:
c
c          1. more complete excerpts of real  applications.
c
c          2. high complexity computation to test the claims
c             made for Fortran compiler code optimization.
c
c          3. original, amateur Fortran coding to test the claims
c             made for automatic vectorizers of olde Fortran.
c
c     As a  matter  of policy kernels should be updated, without prior
c     notice or further  justification,  whenever  new  algorithms are
c     percieved in  important  applications.   The  primitive  kernels
c     should be  retained   for  comparisons  and  scaling  over  long
c     periods.
c
C
C
c
c
c
c
c
c
C
C IV: PERFORMANCE MEASUREMENTS
c
c
c
c
c     Through-put is measured in units of millions of floating-point
c     operations executed per second, called mflops.
c
c
c     Artificially long computer  runs do not have to be contrived for
c     timing on  machines  where  a cpu clock may be read in job mode.
c     Statistics on  the accuracy  of  the  timing  method  should  be
c     measured.
c
c     Net mflops is meaningful only if real run time of each kernel
c     is adjusted such that it weights the total time in proportion
c     to the actual usage of that catagory of computation in the
c     total workload.
c
C
C
c
c
c   1. Assignment Of Weights To Floating-Point Operations
c
c     Weights are assigned to different kinds of floating-point
c     operations to normalize their hardware execution time to
c     addition time so that the flop rates computed for various
c     Fortran Kernels will be commensurable.
c
C                           +,-,*   1
C                          /,SQRT   4
C                     EXP,SIN,ETC.  8
C                     IF(X.REL.Y)   1
c
c
c     Each Kernel flop-count is the weighted number of flops required for
c     serial execution.  The scalar version defines the NECESSARY computation
c     generally, in the absence of proof to the contrary.  The vector
c     or parallel executions are only credited with executing the same
c     necessary computation.  If the parallel methods do more computation
c     than is necessary then the extra flops are not counted as through-put.
c
c
c
c
c
C
C
c
c
c
c
c
c
c
c
C
C
C
C       2. SAMPLE OUTPUT:               CDC-7600/FTN-4.4
C
C
C                 KERNEL  FLOPS   TIME   MFLOPS
C                      1    500    94.4    5.30
C                      2    300    45.3    6.62
C                      3    100    21.9    4.57
C                      4    300   109.3    2.75
C                      5    100    25.6    3.91
C                      6    100    27.8    3.60
C                      7    640    88.2    7.25
C                      8   1440   249.0    5.78
C                      9    680   123.2    5.52
C                     10    360   102.8    3.50
C                     11     49    34.8    1.41
C                     12     49    18.3    2.68
C                     13    224   107.7    2.08
C                     14   3300   809.3    4.08
C                     15   3960  1769.5    2.24
C                     16    530   320.3    1.65
C                     17    405    92.2    4.39
C                     18   6600  1121.5    5.88
C                     19    540   105.8    5.11
C                     20   1300   266.0    4.89
C                     21   1250   370.9    3.37
c                     22   1700   601.9    2.82
c                     23   1650   362.4    4.55
c                     24    200   171.7    1.16
C
C
C                      AVERAGE  RATE =     3.96 MEGA-FLOPS/SEC.
C
C                      MEDIAN   RATE =     4.08 MEGA-FLOPS/SEC.
C
C                      HARMONIC MEAN =     3.15 MEGA-FLOPS/SEC.
C
C                      STANDARD DEV. =     1.61 MEGA-FLOPS/SEC.
C
C
C                                                    F.H.MCMAHON  1972
c
c
c
c
c
c
c
c
c
c
c
c
c
c
C   SAMPLE OUTPUT FROM SUBROUTINe REPORT:   (CRAY-XMP4/CFT Compiler)
C
C
c                                                                       aus
c
c
c
c
c
c
C
C         The following output was produced on CRAY-XMP4 in a
C         fully loaded, multi-processing, multi-programming system:
c
c
c
c
c
c
c          VERIFY ADEQUATE Loop SIZE VERSUS CPU CLOCK ACCURACY
c          -----     -------     -------    -------   --------
c          EXTRA     MAXIMUM     DIGITAL    DYNAMIC   RELATIVE
c          Loop      CPUTIME     CLOCK      CLOCK     TIMING
c          SIZE      SECONDS     ERROR      ERROR     ERROR
c          -----     -------     -------    -------   --------
c              1  4.0000e-06      12.50%     10.58%     38.32%
c              2  6.0000e-06       8.33%      9.01%     18.96%
c              4  1.1000e-05       4.55%      6.07%      9.28%
c              8  1.9000e-05       2.63%      2.60%      3.05%
c             16  3.6000e-05       1.39%      0.00%      0.41%
c             32  7.4000e-05       0.68%      1.77%      0.58%
c             64  1.4700e-04       0.34%      1.13%      0.11%
c            128  2.9300e-04       0.17%      1.16%      0.07%
c            256  5.8300e-04       0.09%      0.89%      0.21%
c            512  1.1580e-03       0.04%      0.40%      0.61%
c           1024  2.3090e-03       0.02%      0.21%      0.54%
c           1360              Current Run:    Loop=   10.000*Loop
c           2048  4.6830e-03       0.01%      0.61%      0.31%
c           4096  9.3590e-03       0.01%      0.78%      0.55%
c          -----     -------     -------    -------   --------
c
c           Approximate Serial Job Time=   2.2e+01 Sec.    ( Nruns= 7 RUNS)
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c   CPU CLOCK OVERHEAD (t err):
c
c        RUN        AVERAGE        STANDEV        MINIMUM        MAXIMUM
c   TICK   1   1.711250e-04   5.994789e-07   1.700000e-04   1.720000e-04
c   TICK   2   1.705000e-04   5.000000e-07   1.700000e-04   1.710000e-04
c   TICK   3   1.701250e-04   3.307189e-07   1.700000e-04   1.710000e-04
c   TICK   4   1.706250e-04   4.841229e-07   1.700000e-04   1.710000e-04
c   TICK   5   1.698750e-04   3.307189e-07   1.690000e-04   1.700000e-04
c   TICK   6   1.708750e-04   3.307189e-07   1.700000e-04   1.710000e-04
c   TICK   7   1.702500e-04   4.330127e-07   1.700000e-04   1.710000e-04
c   DATA   7   9.998664e-02   5.433200e-07   9.998564e-02   9.998765e-02
c   DATA   7   9.998599e-02   9.174233e-07   9.998434e-02   9.998765e-02
c   TICK   7   1.704821e-04   4.032711e-07   1.698750e-04   1.711250e-04
c
c
c   THE EXPERIMENTAL TIMING ERRORS FOR ALL  7 RUNS
c   --  ---------  ---------  --------- -----  -----   ---
c    k   T min      T avg      T max    T err   tick   P-F
c   --  ---------  ---------  --------- -----  -----   ---
c    1 2.0711e-03 2.0768e-03 2.0889e-03  0.28%  7.59%     0
c    2 9.0371e-03 9.1581e-03 9.3587e-03  1.24%  1.83%     0
c    3 1.2985e-03 1.3268e-03 1.4117e-03  2.72% 11.39%     0
c    4 3.5151e-03 3.6089e-03 3.8508e-03  3.13%  4.51%     0
c    5 3.0219e-02 3.0433e-02 3.0790e-02  0.59%  0.56%     0
c    6 9.0741e-03 9.1724e-03 9.3707e-03  1.19%  1.82%     0
c    7 3.3459e-03 3.3745e-03 3.4269e-03  0.91%  4.81%     0
c    8 6.3415e-03 6.4552e-03 6.6848e-03  1.93%  2.57%     0
c    9 3.9629e-03 4.0299e-03 4.1938e-03  2.04%  4.06%     0
c   10 4.5211e-03 4.6594e-03 4.9197e-03  3.21%  3.53%     0
c   11 2.6588e-02 2.6797e-02 2.7038e-02  0.56%  0.63%     0
c   12 1.7391e-03 1.7824e-03 1.8417e-03  2.00%  8.73%     0
c   13 3.3066e-02 3.3292e-02 3.3462e-02  0.38%  0.51%     0
c   14 1.6947e-02 1.7032e-02 1.7144e-02  0.35%  0.99%     0
c   15 3.2127e-02 3.2212e-02 3.2355e-02  0.27%  0.53%     0
c   16 3.4654e-02 3.4858e-02 3.5057e-02  0.37%  0.49%     0
c   17 3.0707e-02 3.1048e-02 3.1258e-02  0.57%  0.55%     0
c   18 3.5849e-03 3.6278e-03 3.6808e-03  0.78%  4.49%     0
c   19 3.0307e-02 3.0433e-02 3.0639e-02  0.35%  0.56%     0
c   20 1.9727e-02 1.9762e-02 1.9795e-02  0.11%  0.86%     0
c   21 1.7039e-02 1.7663e-02 1.8165e-02  2.21%  0.96%     0
c   22 2.6061e-03 2.6168e-03 2.6289e-03  0.29%  6.12%     0
c   23 2.9290e-02 2.9506e-02 2.9640e-02  0.42%  0.57%     0
c   24 2.7286e-02 2.7356e-02 2.7429e-02  0.22%  0.62%     0
c   --  ---------  ---------  --------- -----  -----   ---
c
c
c   NET CPU TIMING VARIANCE (T err);  A few % is ok:
c
c                   AVERAGE        STANDEV        MINIMUM        MAXIMUM
c       Terr          1.09%          0.96%          0.11%          3.21%
c
c
c
c
c
c   ********************************************
c   THE LIVERMORE  FORTRAN KERNELS:  M F L O P S
c   ********************************************
c
c                Computer : CRAY-XMP/1   (8.5 ns.)
c                System   : CTSS 10e  fully loaded
c                Compiler : CFT 1.14
c
c           Mean  DO Span =   471
c
c           When the computer performance range is very large
c           the net Mflops rate of many Fortran programs and
c           workloads will be in the sub-range between the equi-
c           weighted harmonic and arithmetic means depending
c           on the degree of code parallelism and optimization.
c           More accurate estimates of cpu workload rates depend
c           on assigning appropriate weights for each kernel.
c
c
c
c   KERNEL  FLOPS   MICROSEC   MFLOP/SEC SPAN WEIGHT  CHECK-SUMS         OK
c   ------  -----   --------   --------- ---- ------  ------------------ --
c    1 3.5035e+05 2.0768e+03    168.6967 1001   1.00  3.580256885257e+05 15
c    2 2.5996e+05 9.1581e+03     28.3858  101   1.00  3.605241761060e+03 15
c    3 1.8018e+05 1.3268e+03    135.8001 1001   1.00  7.005200181466e+01 14
c    4 1.6800e+05 3.6089e+03     46.5510 1001   1.00  4.199475416832e+00 14
c    5 2.0000e+05 3.0433e+04      6.5718 1001   1.00  3.184210149672e+04 15
c    6 1.1904e+05 9.1724e+03     12.9781   64   1.00  3.660366768766e+13 13
c    7 6.3680e+05 3.3745e+03    188.7084  995   1.00  4.272975752615e+05 14
c    8 7.1280e+05 6.4552e+03    110.4221  100   1.00  1.050887603939e+06 15
c    9 6.1812e+05 4.0299e+03    153.3817  101   1.00  8.326105269826e+05 15
c   10 3.0906e+05 4.6594e+03     66.3308  101   1.00  5.117258849028e+05 15
c   11 1.1000e+05 2.6797e+04      4.1049 1001   1.00  2.340037680853e+08 13
c   12 1.2000e+05 1.7824e+03     67.3259 1000   1.00  2.034999012772e-04  9
c   13 1.6128e+05 3.3292e+04      4.8444   64   1.00  2.839977317870e+10 12
c   14 2.2022e+05 1.7032e+04     12.9301 1001   1.00  2.116750694993e+10 14
c   15 1.6500e+05 3.2212e+04      5.1224  101   1.00  2.760671683247e+05 14
c   16 1.3250e+05 3.4858e+04      3.8011   75   1.00  1.982820000000e+05 16
c   17 3.1815e+05 3.1048e+04     10.2471  101   1.00  7.802492410321e+03 14
c   18 4.3560e+05 3.6278e+03    120.0727  100   1.00  3.615937787530e+05 14
c   19 2.3634e+05 3.0433e+04      7.7659  101   1.00  3.795271872105e+03 14
c   20 2.6000e+05 1.9762e+04     13.1567 1000   1.00  2.128451037520e+08 12
c   21 1.2625e+06 1.7663e+04     71.4762  101   1.00  5.802625385048e+07 13
c   22 1.8887e+05 2.6168e+03     72.1758  101   1.00  2.057023063597e+03 15
c   23 4.3560e+05 2.9506e+04     14.7633  100   1.00  2.484884179712e+05 14
c   24 5.0000e+04 2.7356e+04      1.8278 1001   1.00  3.500000000000e+03 16
c   ------  -----   --------   --------- ---- ------  ------------------ --
c           MFLOPS   Range :                                            335
c
c           Maximum   Rate =    188.7084 Mega-Flops/Sec.
c           Average   Rate =     55.3100 Mega-Flops/Sec.
c           Geometric Mean =     24.9006 Mega-Flops/Sec.
c           Median    Rate =     21.5746 Mega-Flops/Sec.
c           Harmonic  Mean =     10.4846 Mega-Flops/Sec.
c           Minimum   Rate =      1.8278 Mega-Flops/Sec.
c
c
c           Standard  Dev. =     58.6194 Mega-Flops/Sec.
c           Median    Dev. =     50.1115 Mega-Flops/Sec.
c
c
c
c
c
c                      SENSITIVITY ANALYSIS
c
c
c           The sensitivity of the harmonic mean rate (Mflops)
c           to various weightings is shown in the table below.
c           Seven work distributions are generated by assigning
c           two distinct weights to ranked kernels by quartiles.
c           Forty nine possible cpu workloads are then evaluated
c           using seven sets of values for the total weights:
c
c
c               ------ ------ ------ ------ ------ ------ ------
c     1st QT:       O      O      O      O      O      X      X
c     2nd QT:       O      O      O      X      X      X      O
c     3rd QT:       O      X      X      X      O      O      O
c     4th QT:       X      X      O      O      O      O      O
c               ------ ------ ------ ------ ------ ------ ------
c     Total
c     Weights                    Net Mflops:
c      X    O
c    ---- ----
c
c    1.00 0.00    3.73   5.63  11.44  18.78  52.41  76.44 141.19
c
c    0.95 0.05    3.90   5.90  11.37  17.40  41.38  46.92  77.11
c
c    0.90 0.10    4.08   6.20  11.30  16.21  34.18  33.85  53.04
c
c    0.80 0.20    4.51   6.91  11.17  14.27  25.36  21.74  32.65
c
c    0.70 0.30    5.03   7.79  11.04  12.73  20.16  16.01  23.58
c
c    0.60 0.40    5.68   8.94  10.91  11.50  16.73  12.67  18.46
c
c    0.50 0.50    6.54  10.48  10.78  10.48  14.30  10.48  15.16
c    ---- ----
c               ------ ------ ------ ------ ------ ------ ------
c
c
c
c
c
c
c SENSITIVITY OF NET MFLOPS RATE TO USE OF OPTIMAL FORTRAN CODE(SISD/SIMD MODEL)
c
c      5.63   6.97   9.14  13.28  17.16   24.27   41.42   64.05  141.19
c
c      0.00   0.20   0.40   0.60   0.70    0.80    0.90    0.95    1.00
c      Fraction Of Operations Run At Optimal Fortran Rates
c
c
c
c
c
c
c
c
c
c
c   CPU CLOCK OVERHEAD (t err):
c
c        RUN        AVERAGE        STANDEV        MINIMUM        MAXIMUM
c   TICK   1   1.702500e-04   4.330127e-07   1.700000e-04   1.710000e-04
c   TICK   2   1.708750e-04   5.994789e-07   1.700000e-04   1.720000e-04
c   TICK   3   1.705000e-04   5.000000e-07   1.700000e-04   1.710000e-04
c   TICK   4   1.707500e-04   4.330127e-07   1.700000e-04   1.710000e-04
c   TICK   5   1.703750e-04   4.841229e-07   1.700000e-04   1.710000e-04
c   TICK   6   1.713750e-04   4.841229e-07   1.710000e-04   1.720000e-04
c   TICK   7   1.713750e-04   6.959705e-07   1.700000e-04   1.720000e-04
c   DATA   7   9.998664e-02   5.433200e-07   9.998564e-02   9.998765e-02
c   DATA   7   9.998599e-02   9.174233e-07   9.998434e-02   9.998765e-02
c   TICK   7   1.707857e-04   4.210652e-07   1.702500e-04   1.713750e-04
c
c
c   THE EXPERIMENTAL TIMING ERRORS FOR ALL  7 RUNS
c   --  ---------  ---------  --------- -----  -----   ---
c    k   T min      T avg      T max    T err   tick   P-F
c   --  ---------  ---------  --------- -----  -----   ---
c    1 3.2316e-03 3.2864e-03 3.3716e-03  1.24%  4.94%     0
c    2 1.0752e-02 1.0895e-02 1.1205e-02  1.37%  1.54%     0
c    3 5.3096e-03 5.4232e-03 5.6036e-03  1.54%  3.05%     0
c    4 1.7793e-02 1.7997e-02 1.8296e-02  0.83%  0.94%     0
c    5 3.3821e-02 3.4055e-02 3.4313e-02  0.45%  0.50%     0
c    6 1.7833e-02 1.8017e-02 1.8207e-02  0.86%  0.94%     0
c    7 4.2955e-03 4.3318e-03 4.3788e-03  0.63%  3.79%     0
c    8 7.5466e-03 7.7076e-03 7.8566e-03  1.31%  2.17%     0
c    9 4.5706e-03 4.6989e-03 4.9246e-03  2.38%  3.51%     0
c   10 5.0116e-03 5.2598e-03 5.6236e-03  3.99%  3.15%     0
c   11 3.1628e-02 3.1935e-02 3.2428e-02  0.77%  0.53%     0
c   12 3.0023e-03 3.0851e-03 3.2086e-03  2.45%  5.25%     0
c   13 3.7940e-02 3.8124e-02 3.8796e-02  0.73%  0.45%     0
c   14 1.7428e-02 1.7617e-02 1.7950e-02  0.88%  0.96%     0
c   15 6.4200e-02 6.4414e-02 6.4627e-02  0.19%  0.26%     0
c   16 3.9948e-02 4.0099e-02 4.0321e-02  0.36%  0.42%     0
c   17 3.5173e-02 3.5518e-02 3.5798e-02  0.62%  0.48%     0
c   18 3.5961e-03 3.6291e-03 3.6636e-03  0.64%  4.49%     0
c   19 3.5764e-02 3.5864e-02 3.5952e-02  0.16%  0.47%     0
c   20 3.1651e-02 3.1711e-02 3.1767e-02  0.12%  0.54%     0
c   21 2.4059e-02 2.4559e-02 2.5165e-02  1.76%  0.69%     0
c   22 3.3117e-03 3.3241e-03 3.3406e-03  0.27%  4.89%     0
c   23 3.6698e-02 3.6950e-02 3.7378e-02  0.58%  0.46%     0
c   24 3.4283e-02 3.4332e-02 3.4365e-02  0.08%  0.49%     0
c   --  ---------  ---------  --------- -----  -----   ---
c
c
c   NET CPU TIMING VARIANCE (T err);  A few % is ok:
c
c                   AVERAGE        STANDEV        MINIMUM        MAXIMUM
c       Terr          1.01%          0.89%          0.08%          3.99%
c
c
c
c
c
c
c
c   ********************************************
c   THE LIVERMORE  FORTRAN KERNELS:  M F L O P S
c   ********************************************
c
c                Computer : CRAY-XMP/1   (8.5 ns.)
c                System   : CTSS 10e  fully loaded
c                Compiler : CFT 1.14
c
c           Mean  DO Span =    90
c
c           When the computer performance range is very large
c           the net Mflops rate of many Fortran programs and
c           workloads will be in the sub-range between the equi-
c           weighted harmonic and arithmetic means depending
c           on the degree of code parallelism and optimization.
c           More accurate estimates of cpu workload rates depend
c           on assigning appropriate weights for each kernel.
c
c
c
c   KERNEL  FLOPS   MICROSEC   MFLOP/SEC SPAN WEIGHT  CHECK-SUMS         OK
c   ------  -----   --------   --------- ---- ------  ------------------ --
c    1 4.0400e+05 3.2864e+03    122.9325  101   2.00  3.677341345257e+03 15
c    2 3.1040e+05 1.0895e+04     28.4896  101   2.00  3.605241761060e+03 15
c    3 2.1412e+05 5.4232e+03     39.4821  101   2.00  7.068190056054e+00 14
c    4 1.6800e+05 1.7997e+04      9.3349  101   2.00  4.199475416832e+00 14
c    5 2.2000e+05 3.4055e+04      6.4602  101   2.00  3.212322357721e+02 15
c    6 1.3440e+05 1.8017e+04      7.4597   32   2.00  1.885296670188e+16 12
c    7 7.1104e+05 4.3318e+03    164.1448  101   2.00  4.441910421041e+03 14
c    8 8.5536e+05 7.7076e+03    110.9756  100   2.00  1.050887603939e+06 15
c    9 7.2114e+05 4.6989e+03    153.4690  101   2.00  8.326105269826e+05 15
c   10 3.4542e+05 5.2598e+03     65.6719  101   2.00  5.117258849028e+05 15
c   11 1.2800e+05 3.1935e+04      4.0082  101   2.00  2.403492285233e+05 14
c   12 1.3600e+05 3.0851e+03     44.0833  100   2.00  4.989298413705e-05  9
c   13 1.8368e+05 3.8124e+04      4.8180   32   2.00  1.627723261369e+10 12
c   14 2.2220e+05 1.7617e+04     12.6126  101   2.00  2.130649341195e+08 14
c   15 3.3000e+05 6.4414e+04      5.1231  101   2.00  2.760671683247e+05 14
c   16 1.5120e+05 4.0099e+04      3.7707   40   2.00  2.270870000000e+05 16
c   17 3.6360e+05 3.5518e+04     10.2370  101   2.00  7.802492410321e+03 14
c   18 4.3560e+05 3.6291e+03    120.0307  100   2.00  3.615937787530e+05 14
c   19 2.7876e+05 3.5864e+04      7.7727  101   2.00  3.795271872105e+03 14
c   20 4.1600e+05 3.1711e+04     13.1186  100   2.00  2.188343625167e+05 13
c   21 1.2500e+06 2.4559e+04     50.8986   50   2.00  2.790571795522e+07 13
c   22 2.4038e+05 3.3241e+03     72.3149  101   2.00  2.057023063597e+03 15
c   23 5.4450e+05 3.6950e+04     14.7363  100   2.00  2.484926226844e+05 14
c   24 6.2000e+04 3.4332e+04      1.8059  101   2.00  3.500000000000e+02 16
c   ------  -----   --------   --------- ---- ------  ------------------ --
c           MFLOPS   Range :                                            336
c
c           Maximum   Rate =    164.1448 Mega-Flops/Sec.
c           Average   Rate =     44.7396 Mega-Flops/Sec.
c           Geometric Mean =     20.4634 Mega-Flops/Sec.
c           Median    Rate =     13.9274 Mega-Flops/Sec.
c           Harmonic  Mean =      9.6511 Mega-Flops/Sec.
c           Minimum   Rate =      1.8059 Mega-Flops/Sec.
c
c
c           Standard  Dev. =     50.7117 Mega-Flops/Sec.
c           Median    Dev. =     41.4031 Mega-Flops/Sec.
c
c
c
c
c
c                      SENSITIVITY ANALYSIS
c
c
c           The sensitivity of the harmonic mean rate (Mflops)
c           to various weightings is shown in the table below.
c           Seven work distributions are generated by assigning
c           two distinct weights to ranked kernels by quartiles.
c           Forty nine possible cpu workloads are then evaluated
c           using seven sets of values for the total weights:
c
c
c               ------ ------ ------ ------ ------ ------ ------
c     1st QT:       O      O      O      O      O      X      X
c     2nd QT:       O      O      O      X      X      X      O
c     3rd QT:       O      X      X      X      O      O      O
c     4th QT:       X      X      O      O      O      O      O
c               ------ ------ ------ ------ ------ ------ ------
c     Total
c     Weights                    Net Mflops:
c      X    O
c    ---- ----
c
c    1.00 0.00    3.69   5.34   9.63  14.84  32.28  50.47 115.58
c
c    0.95 0.05    3.85   5.59   9.63  14.08  27.92  35.47  66.74
c
c    0.90 0.10    4.02   5.86   9.63  13.40  24.59  27.34  46.92
c
c    0.80 0.20    4.42   6.50   9.64  12.21  19.86  18.75  29.43
c
c    0.70 0.30    4.90   7.29   9.64  11.22  16.66  14.27  21.44
c
c    0.60 0.40    5.50   8.31   9.64  10.38  14.34  11.51  16.86
c
c    0.50 0.50    6.27   9.65   9.64   9.65  12.59   9.65  13.90
c    ---- ----
c               ------ ------ ------ ------ ------ ------ ------
c
c
c
c
c
c
c SENSITIVITY OF NET MFLOPS RATE TO USE OF OPTIMAL FORTRAN CODE(SISD/SIMD MODEL)
c
c      5.34   6.59   8.63  12.48  16.06   22.52   37.69   56.85  115.58
c
c      0.00   0.20   0.40   0.60   0.70    0.80    0.90    0.95    1.00
c      Fraction Of Operations Run At Optimal Fortran Rates
c
c
c
c
c
c
c
c
c
c
c
c   CPU CLOCK OVERHEAD (t err):
c
c        RUN        AVERAGE        STANDEV        MINIMUM        MAXIMUM
c   TICK   1   1.705000e-04   5.000000e-07   1.700000e-04   1.710000e-04
c   TICK   2   1.703750e-04   8.569568e-07   1.690000e-04   1.720000e-04
c   TICK   3   1.705000e-04   7.071068e-07   1.700000e-04   1.720000e-04
c   TICK   4   1.703750e-04   4.841229e-07   1.700000e-04   1.710000e-04
c   TICK   5   1.703750e-04   4.841229e-07   1.700000e-04   1.710000e-04
c   TICK   6   1.711250e-04   5.994790e-07   1.700000e-04   1.720000e-04
c   TICK   7   1.703750e-04   4.841229e-07   1.700000e-04   1.710000e-04
c   DATA   7   9.998664e-02   5.433200e-07   9.998564e-02   9.998765e-02
c   DATA   7   9.998599e-02   9.174233e-07   9.998434e-02   9.998765e-02
c   TICK   7   1.705179e-04   2.537977e-07   1.703750e-04   1.711250e-04
c
c
c   THE EXPERIMENTAL TIMING ERRORS FOR ALL  7 RUNS
c   --  ---------  ---------  --------- -----  -----   ---
c    k   T min      T avg      T max    T err   tick   P-F
c   --  ---------  ---------  --------- -----  -----   ---
c    1 4.4326e-03 4.4753e-03 4.5326e-03  0.66%  3.67%     0
c    2 2.2472e-02 2.2538e-02 2.2624e-02  0.21%  0.75%     0
c    3 1.0765e-02 1.0820e-02 1.0869e-02  0.29%  1.55%     0
c    4 3.6484e-02 3.6894e-02 3.7248e-02  0.71%  0.46%     0
c    5 2.7303e-02 2.7436e-02 2.7813e-02  0.65%  0.62%     0
c    6 4.3140e-02 4.3455e-02 4.4160e-02  0.73%  0.39%     0
c    7 4.8685e-03 4.9351e-03 5.0386e-03  1.17%  3.34%     0
c    8 9.0905e-03 9.2139e-03 9.3176e-03  0.78%  1.82%     0
c    9 6.1915e-03 6.2526e-03 6.3236e-03  0.77%  2.65%     0
c   10 6.9315e-03 7.0521e-03 7.2076e-03  1.16%  2.36%     0
c   11 2.5618e-02 2.5701e-02 2.5775e-02  0.17%  0.66%     0
c   12 4.8386e-03 4.8696e-03 4.8996e-03  0.43%  3.38%     0
c   13 2.9929e-02 3.0063e-02 3.0225e-02  0.30%  0.56%     0
c   14 1.6473e-02 1.6583e-02 1.6991e-02  1.03%  1.02%     0
c   15 3.5486e-02 3.5613e-02 3.5729e-02  0.24%  0.48%     0
c   16 3.0663e-02 3.0825e-02 3.1203e-02  0.58%  0.55%     0
c   17 2.7777e-02 2.8010e-02 2.8539e-02  0.87%  0.61%     0
c   18 7.7105e-03 7.7742e-03 7.8675e-03  0.63%  2.15%     0
c   19 2.8446e-02 2.8505e-02 2.8572e-02  0.17%  0.59%     0
c   20 2.9133e-02 2.9171e-02 2.9209e-02  0.09%  0.58%     0
c   21 7.7980e-02 7.8881e-02 8.0355e-02  0.93%  0.22%     0
c   22 3.6236e-03 3.6338e-03 3.6426e-03  0.15%  4.48%     0
c   23 2.9334e-02 2.9478e-02 2.9736e-02  0.45%  0.58%     0
c   24 2.7287e-02 2.7360e-02 2.7406e-02  0.13%  0.62%     0
c   --  ---------  ---------  --------- -----  -----   ---
c
c
c   NET CPU TIMING VARIANCE (T err);  A few % is ok:
c
c                   AVERAGE        STANDEV        MINIMUM        MAXIMUM
c       Terr          0.55%          0.33%          0.09%          1.17%
c
c
c
c
c
c
c
c   ********************************************
c   THE LIVERMORE  FORTRAN KERNELS:  M F L O P S
c   ********************************************
c
c                Computer : CRAY-XMP/1   (8.5 ns.)
c                System   : CTSS 10e  fully loaded
c                Compiler : CFT 1.14
c
c           Mean  DO Span =    19
c
c           When the computer performance range is very large
c           the net Mflops rate of many Fortran programs and
c           workloads will be in the sub-range between the equi-
c           weighted harmonic and arithmetic means depending
c           on the degree of code parallelism and optimization.
c           More accurate estimates of cpu workload rates depend
c           on assigning appropriate weights for each kernel.
c
c
c
c   KERNEL  FLOPS   MICROSEC   MFLOP/SEC SPAN WEIGHT  CHECK-SUMS         OK
c   ------  -----   --------   --------- ---- ------  ------------------ --
c    1 3.0240e+05 4.4753e+03     67.5703   27   1.00  2.698573151746e+02 14
c    2 1.6192e+05 2.2538e+04      7.1844   15   1.00  8.398933280062e+01 15
c    3 1.5984e+05 1.0820e+04     14.7728   27   1.00  1.889516362525e+00 14
c    4 9.1200e+04 3.6894e+04      2.4719   27   1.00  4.199475416832e+00 14
c    5 1.6640e+05 2.7436e+04      6.0651   27   1.00  2.227830673914e+01 15
c    6 8.0640e+04 4.3455e+04      1.8557    8   1.00  5.812436051264e+12 11
c    7 5.3760e+05 4.9351e+03    108.9350   21   1.00  1.992004152347e+02 14
c    8 6.7392e+05 9.2139e+03     73.1416   14   1.00  2.072380567514e+04 15
c    9 5.3040e+05 6.2526e+03     84.8284   15   1.00  1.836777922612e+04 15
c   10 2.7000e+05 7.0521e+03     38.2867   15   1.00  1.155903859389e+04 15
c   11 9.5680e+04 2.5701e+04      3.7227   27   1.00  4.585812935092e+03 14
c   12 9.9840e+04 4.8696e+03     20.5026   26   1.00  1.360405188278e-05 10
c   13 1.3888e+05 3.0063e+04      4.6196    8   1.00  3.328647876037e+09 12
c   14 1.9008e+05 1.6583e+04     11.4622   27   1.00  1.721399839433e+07 14
c   15 1.8480e+05 3.5613e+04      5.1891   15   1.00  7.762981016945e+03 15
c   16 1.2320e+05 3.0825e+04      3.9967   15   1.00  1.804320000000e+05 16
c   17 2.8080e+05 2.8010e+04     10.0251   15   1.00  2.063158033013e+02 14
c   18 4.5760e+05 7.7742e+03     58.8614   14   1.00  6.790452348639e+03 15
c   19 2.0160e+05 2.8505e+04      7.0725   15   1.00  8.877614886362e+01 14
c   20 3.7856e+05 2.9171e+04     12.9774   26   1.00  4.191399274630e+03 14
c   21 2.0000e+06 7.8881e+04     25.3547   20   1.00  1.761809056725e+07 12
c   22 1.6320e+05 3.6338e+03     44.9121   15   1.00  4.276978109785e+01 15
c   23 4.0040e+05 2.9478e+04     13.5830   14   1.00  3.395238421926e+03 15
c   24 4.7840e+04 2.7360e+04      1.7485   27   1.00  9.100000000000e+01 16
c   ------  -----   --------   --------- ---- ------  ------------------ --
c           MFLOPS   Range :                                            338
c
c           Maximum   Rate =    108.9350 Mega-Flops/Sec.
c           Average   Rate =     26.2141 Mega-Flops/Sec.
c           Geometric Mean =     13.0843 Mega-Flops/Sec.
c           Median    Rate =     12.2198 Mega-Flops/Sec.
c           Harmonic  Mean =      6.8627 Mega-Flops/Sec.
c           Minimum   Rate =      1.7485 Mega-Flops/Sec.
c
c           Standard  Dev. =     29.9505 Mega-Flops/Sec.
c           Median    Dev. =     17.0145 Mega-Flops/Sec.
c
c
c
c
c
c
c                      SENSITIVITY ANALYSIS
c
c
c           The sensitivity of the harmonic mean rate (Mflops)
c           to various weightings is shown in the table below.
c           Seven work distributions are generated by assigning
c           two distinct weights to ranked kernels by quartiles.
c           Forty nine possible cpu workloads are then evaluated
c           using seven sets of values for the total weights:
c
c
c               ------ ------ ------ ------ ------ ------ ------
c     1st QT:       O      O      O      O      O      X      X
c     2nd QT:       O      O      O      X      X      X      O
c     3rd QT:       O      X      X      X      O      O      O
c     4th QT:       X      X      O      O      O      O      O
c               ------ ------ ------ ------ ------ ------ ------
c     Total
c     Weights                    Net Mflops:
c      X    O
c    ---- ----
c
c    1.00 0.00    2.67   3.90   7.27  10.36  18.03  28.48  67.65
c
c    0.95 0.05    2.78   4.08   7.24   9.86  16.27  21.66  42.53
c
c    0.90 0.10    2.90   4.27   7.21   9.40  14.82  17.47  31.02
c
c    0.80 0.20    3.19   4.72   7.16   8.61  12.58  12.60  20.12
c
c    0.70 0.30    3.53   5.26   7.10   7.93  10.92   9.85  14.89
c
c    0.60 0.40    3.96   5.96   7.05   7.36   9.65   8.09  11.82
c
c    0.50 0.50    4.50   6.86   6.99   6.86   8.65   6.86   9.80
c    ---- ----
c               ------ ------ ------ ------ ------ ------ ------
c
c
c
c
c
c
c SENSITIVITY OF NET MFLOPS RATE TO USE OF OPTIMAL FORTRAN CODE(SISD/SIMD MODEL)
c
c      3.90   4.81   6.26   8.98  11.46   15.85   25.68   37.23   67.65
c
c      0.00   0.20   0.40   0.60   0.70    0.80    0.90    0.95    1.00
c      Fraction Of Operations Run At Optimal Fortran Rates
c
c
c
c
c
c
c
c   ********************************************
c   THE LIVERMORE  FORTRAN KERNELS:  M F L O P S
c   ********************************************
c
c                Computer : CRAY-XMP/1   (8.5 ns.)
c                System   : CTSS 10e  fully loaded
c                Compiler : CFT 1.14
c
c           Mean  DO Span =   167
c
c           When the computer performance range is very large
c           the net Mflops rate of many Fortran programs and
c           workloads will be in the sub-range between the equi-
c           weighted harmonic and arithmetic means depending
c           on the degree of code parallelism and optimization.
c           More accurate estimates of cpu workload rates depend
c           on assigning appropriate weights for each kernel.
c
c
c
c   KERNEL  FLOPS   MICROSEC   MFLOP/SEC SPAN WEIGHT  CHECK-SUMS         OK
c   ------  -----   --------   --------- ---- ------  ------------------ --
c    1 3.0240e+05 4.4753e+03     67.5703   27   1.00  2.698573151746e+02 14
c    2 1.6192e+05 2.2538e+04      7.1844   15   1.00  8.398933280062e+01 15
c    3 1.5984e+05 1.0820e+04     14.7728   27   1.00  1.889516362525e+00 14
c    4 9.1200e+04 3.6894e+04      2.4719   27   1.00  4.199475416832e+00 14
c    5 1.6640e+05 2.7436e+04      6.0651   27   1.00  2.227830673914e+01 15
c    6 8.0640e+04 4.3455e+04      1.8557    8   1.00  5.812436051264e+12 11
c    7 5.3760e+05 4.9351e+03    108.9350   21   1.00  1.992004152347e+02 14
c    8 6.7392e+05 9.2139e+03     73.1416   14   1.00  2.072380567514e+04 15
c    9 5.3040e+05 6.2526e+03     84.8284   15   1.00  1.836777922612e+04 15
c   10 2.7000e+05 7.0521e+03     38.2867   15   1.00  1.155903859389e+04 15
c   11 9.5680e+04 2.5701e+04      3.7227   27   1.00  4.585812935092e+03 14
c   12 9.9840e+04 4.8696e+03     20.5026   26   1.00  1.360405188278e-05 10
c   13 1.3888e+05 3.0063e+04      4.6196    8   1.00  3.328647876037e+09 12
c   14 1.9008e+05 1.6583e+04     11.4622   27   1.00  1.721399839433e+07 14
c   15 1.8480e+05 3.5613e+04      5.1891   15   1.00  7.762981016945e+03 15
c   16 1.2320e+05 3.0825e+04      3.9967   15   1.00  1.804320000000e+05 16
c   17 2.8080e+05 2.8010e+04     10.0251   15   1.00  2.063158033013e+02 14
c   18 4.5760e+05 7.7742e+03     58.8614   14   1.00  6.790452348639e+03 15
c   19 2.0160e+05 2.8505e+04      7.0725   15   1.00  8.877614886362e+01 14
c   20 3.7856e+05 2.9171e+04     12.9774   26   1.00  4.191399274630e+03 14
c   21 2.0000e+06 7.8881e+04     25.3547   20   1.00  1.761809056725e+07 12
c   22 1.6320e+05 3.6338e+03     44.9121   15   1.00  4.276978109785e+01 15
c   23 4.0040e+05 2.9478e+04     13.5830   14   1.00  3.395238421926e+03 15
c   24 4.7840e+04 2.7360e+04      1.7485   27   1.00  9.100000000000e+01 16
c    1 4.0400e+05 3.2864e+03    122.9325  101   2.00  3.677341345257e+03 15
c    2 3.1040e+05 1.0895e+04     28.4896  101   2.00  3.605241761060e+03 15
c    3 2.1412e+05 5.4232e+03     39.4821  101   2.00  7.068190056054e+00 14
c    4 1.6800e+05 1.7997e+04      9.3349  101   2.00  4.199475416832e+00 14
c    5 2.2000e+05 3.4055e+04      6.4602  101   2.00  3.212322357721e+02 15
c    6 1.3440e+05 1.8017e+04      7.4597   32   2.00  1.885296670188e+16 12
c    7 7.1104e+05 4.3318e+03    164.1448  101   2.00  4.441910421041e+03 14
c    8 8.5536e+05 7.7076e+03    110.9756  100   2.00  1.050887603939e+06 15
c    9 7.2114e+05 4.6989e+03    153.4690  101   2.00  8.326105269826e+05 15
c   10 3.4542e+05 5.2598e+03     65.6719  101   2.00  5.117258849028e+05 15
c   11 1.2800e+05 3.1935e+04      4.0082  101   2.00  2.403492285233e+05 14
c   12 1.3600e+05 3.0851e+03     44.0833  100   2.00  4.989298413705e-05  9
c   13 1.8368e+05 3.8124e+04      4.8180   32   2.00  1.627723261369e+10 12
c   14 2.2220e+05 1.7617e+04     12.6126  101   2.00  2.130649341195e+08 14
c   15 3.3000e+05 6.4414e+04      5.1231  101   2.00  2.760671683247e+05 14
c   16 1.5120e+05 4.0099e+04      3.7707   40   2.00  2.270870000000e+05 16
c   17 3.6360e+05 3.5518e+04     10.2370  101   2.00  7.802492410321e+03 14
c   18 4.3560e+05 3.6291e+03    120.0307  100   2.00  3.615937787530e+05 14
c   19 2.7876e+05 3.5864e+04      7.7727  101   2.00  3.795271872105e+03 14
c   20 4.1600e+05 3.1711e+04     13.1186  100   2.00  2.188343625167e+05 13
c   21 1.2500e+06 2.4559e+04     50.8986   50   2.00  2.790571795522e+07 13
c   22 2.4038e+05 3.3241e+03     72.3149  101   2.00  2.057023063597e+03 15
c   23 5.4450e+05 3.6950e+04     14.7363  100   2.00  2.484926226844e+05 14
c   24 6.2000e+04 3.4332e+04      1.8059  101   2.00  3.500000000000e+02 16
c    1 3.5035e+05 2.0768e+03    168.6967 1001   1.00  3.580256885257e+05 15
c    2 2.5996e+05 9.1581e+03     28.3858  101   1.00  3.605241761060e+03 15
c    3 1.8018e+05 1.3268e+03    135.8001 1001   1.00  7.005200181466e+01 14
c    4 1.6800e+05 3.6089e+03     46.5510 1001   1.00  4.199475416832e+00 14
c    5 2.0000e+05 3.0433e+04      6.5718 1001   1.00  3.184210149672e+04 15
c    6 1.1904e+05 9.1724e+03     12.9781   64   1.00  3.660366768766e+13 13
c    7 6.3680e+05 3.3745e+03    188.7084  995   1.00  4.272975752615e+05 14
c    8 7.1280e+05 6.4552e+03    110.4221  100   1.00  1.050887603939e+06 15
c    9 6.1812e+05 4.0299e+03    153.3817  101   1.00  8.326105269826e+05 15
c   10 3.0906e+05 4.6594e+03     66.3308  101   1.00  5.117258849028e+05 15
c   11 1.1000e+05 2.6797e+04      4.1049 1001   1.00  2.340037680853e+08 13
c   12 1.2000e+05 1.7824e+03     67.3259 1000   1.00  2.034999012772e-04  9
c   13 1.6128e+05 3.3292e+04      4.8444   64   1.00  2.839977317870e+10 12
c   14 2.2022e+05 1.7032e+04     12.9301 1001   1.00  2.116750694993e+10 14
c   15 1.6500e+05 3.2212e+04      5.1224  101   1.00  2.760671683247e+05 14
c   16 1.3250e+05 3.4858e+04      3.8011   75   1.00  1.982820000000e+05 16
c   17 3.1815e+05 3.1048e+04     10.2471  101   1.00  7.802492410321e+03 14
c   18 4.3560e+05 3.6278e+03    120.0727  100   1.00  3.615937787530e+05 14
c   19 2.3634e+05 3.0433e+04      7.7659  101   1.00  3.795271872105e+03 14
c   20 2.6000e+05 1.9762e+04     13.1567 1000   1.00  2.128451037520e+08 12
c   21 1.2625e+06 1.7663e+04     71.4762  101   1.00  5.802625385048e+07 13
c   22 1.8887e+05 2.6168e+03     72.1758  101   1.00  2.057023063597e+03 15
c   23 4.3560e+05 2.9506e+04     14.7633  100   1.00  2.484884179712e+05 14
c   24 5.0000e+04 2.7356e+04      1.8278 1001   1.00  3.500000000000e+03 16
c   ------  -----   --------   --------- ---- ------  ------------------ --
c           MFLOPS   Range :                                           1009
c
c           Maximum   Rate =    188.7084 Mega-Flops/Sec.
c           Average   Rate =     42.7509 Mega-Flops/Sec.
c           Geometric Mean =     19.2189 Mega-Flops/Sec.
c           Median    Rate =     13.1567 Mega-Flops/Sec.
c           Harmonic  Mean =      8.9221 Mega-Flops/Sec.
c           Minimum   Rate =      1.7485 Mega-Flops/Sec.
c
c
c           Standard  Dev. =     49.7889 Mega-Flops/Sec.
c           Median    Dev. =     39.3344 Mega-Flops/Sec.
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c   TOP QUARTILE: BEST ARCHITECTURE/APPLICATION MATCH
c
c
c
c   KERNEL  FLOPS   MICROSEC   MFLOP/SEC SPAN WEIGHT
c   ------  -----   --------   --------- ---- ------
c    7 6.3680e+05 3.3745e+03    188.7084  995   1.00
c    1 3.5035e+05 2.0768e+03    168.6967 1001   1.00
c    7 7.1104e+05 4.3318e+03    164.1448  101   2.00
c    9 7.2114e+05 4.6989e+03    153.4690  101   2.00
c    9 6.1812e+05 4.0299e+03    153.3817  101   1.00
c    3 1.8018e+05 1.3268e+03    135.8001 1001   1.00
c    1 4.0400e+05 3.2864e+03    122.9325  101   2.00
c   18 4.3560e+05 3.6278e+03    120.0727  100   1.00
c   18 4.3560e+05 3.6291e+03    120.0307  100   2.00
c    8 8.5536e+05 7.7076e+03    110.9756  100   2.00
c    8 7.1280e+05 6.4552e+03    110.4221  100   1.00
c    7 5.3760e+05 4.9351e+03    108.9350   21   1.00
c    9 5.3040e+05 6.2526e+03     84.8284   15   1.00
c    8 6.7392e+05 9.2139e+03     73.1416   14   1.00
c   22 2.4038e+05 3.3241e+03     72.3149  101   2.00
c   22 1.8887e+05 2.6168e+03     72.1758  101   1.00
c   21 1.2625e+06 1.7663e+04     71.4762  101   1.00
c    1 3.0240e+05 4.4753e+03     67.5703   27   1.00
c   ------  -----   --------   --------- ---- ------
c
c           Frac.  Weights =      0.2500
c           Average   Rate =    118.4560 Mega-Flops/Sec.
c           Harmonic  Mean =    107.3312 Mega-Flops/Sec.
c           Standard  Dev. =     35.5330 Mega-Flops/Sec.
c
c
c
c   KERNEL  FLOPS   MICROSEC   MFLOP/SEC SPAN WEIGHT
c   ------  -----   --------   --------- ---- ------
c   12 1.2000e+05 1.7824e+03     67.3259 1000   1.00
c   10 3.0906e+05 4.6594e+03     66.3308  101   1.00
c   10 3.4542e+05 5.2598e+03     65.6719  101   2.00
c   18 4.5760e+05 7.7742e+03     58.8614   14   1.00
c   21 1.2500e+06 2.4559e+04     50.8986   50   2.00
c    4 1.6800e+05 3.6089e+03     46.5510 1001   1.00
c   22 1.6320e+05 3.6338e+03     44.9121   15   1.00
c   12 1.3600e+05 3.0851e+03     44.0833  100   2.00
c    3 2.1412e+05 5.4232e+03     39.4821  101   2.00
c   10 2.7000e+05 7.0521e+03     38.2867   15   1.00
c    2 3.1040e+05 1.0895e+04     28.4896  101   2.00
c    2 2.5996e+05 9.1581e+03     28.3858  101   1.00
c   21 2.0000e+06 7.8881e+04     25.3547   20   1.00
c   12 9.9840e+04 4.8696e+03     20.5026   26   1.00
c    3 1.5984e+05 1.0820e+04     14.7728   27   1.00
c   23 4.3560e+05 2.9506e+04     14.7633  100   1.00
c   23 5.4450e+05 3.6950e+04     14.7363  100   2.00
c   23 4.0040e+05 2.9478e+04     13.5830   14   1.00
c   20 2.6000e+05 1.9762e+04     13.1567 1000   1.00
c   20 4.1600e+05 3.1711e+04     13.1186  100   2.00
c    6 1.1904e+05 9.1724e+03     12.9781   64   1.00
c   20 3.7856e+05 2.9171e+04     12.9774   26   1.00
c   14 2.2022e+05 1.7032e+04     12.9301 1001   1.00
c   14 2.2220e+05 1.7617e+04     12.6126  101   2.00
c   14 1.9008e+05 1.6583e+04     11.4622   27   1.00
c   17 3.1815e+05 3.1048e+04     10.2471  101   1.00
c   17 3.6360e+05 3.5518e+04     10.2370  101   2.00
c   17 2.8080e+05 2.8010e+04     10.0251   15   1.00
c    4 1.6800e+05 1.7997e+04      9.3349  101   2.00
c   19 2.7876e+05 3.5864e+04      7.7727  101   2.00
c   19 2.3634e+05 3.0433e+04      7.7659  101   1.00
c    6 1.3440e+05 1.8017e+04      7.4597   32   2.00
c    2 1.6192e+05 2.2538e+04      7.1844   15   1.00
c   19 2.0160e+05 2.8505e+04      7.0725   15   1.00
c    5 2.0000e+05 3.0433e+04      6.5718 1001   1.00
c    5 2.2000e+05 3.4055e+04      6.4602  101   2.00
c   ------  -----   --------   --------- ---- ------
c
c           Frac.  Weights =      0.5104
c           Average   Rate =     23.9330 Mega-Flops/Sec.
c           Harmonic  Mean =     13.8803 Mega-Flops/Sec.
c           Standard  Dev. =     19.1237 Mega-Flops/Sec.
c
c
c
c   KERNEL  FLOPS   MICROSEC   MFLOP/SEC SPAN WEIGHT
c   ------  -----   --------   --------- ---- ------
c    5 1.6640e+05 2.7436e+04      6.0651   27   1.00
c   15 1.8480e+05 3.5613e+04      5.1891   15   1.00
c   15 3.3000e+05 6.4414e+04      5.1231  101   2.00
c   15 1.6500e+05 3.2212e+04      5.1224  101   1.00
c   13 1.6128e+05 3.3292e+04      4.8444   64   1.00
c   13 1.8368e+05 3.8124e+04      4.8180   32   2.00
c   13 1.3888e+05 3.0063e+04      4.6196    8   1.00
c   11 1.1000e+05 2.6797e+04      4.1049 1001   1.00
c   11 1.2800e+05 3.1935e+04      4.0082  101   2.00
c   16 1.2320e+05 3.0825e+04      3.9967   15   1.00
c   16 1.3250e+05 3.4858e+04      3.8011   75   1.00
c   16 1.5120e+05 4.0099e+04      3.7707   40   2.00
c   11 9.5680e+04 2.5701e+04      3.7227   27   1.00
c    4 9.1200e+04 3.6894e+04      2.4719   27   1.00
c    6 8.0640e+04 4.3455e+04      1.8557    8   1.00
c   24 5.0000e+04 2.7356e+04      1.8278 1001   1.00
c   24 6.2000e+04 3.4332e+04      1.8059  101   2.00
c   24 4.7840e+04 2.7360e+04      1.7485   27   1.00
c   ------  -----   --------   --------- ---- ------
c
c           Frac.  Weights =      0.2396
c           Average   Rate =      3.8444 Mega-Flops/Sec.
c           Harmonic  Mean =      3.2829 Mega-Flops/Sec.
c           Standard  Dev. =      1.2840 Mega-Flops/Sec.
c
c
c
c
c
c
c
c
c
c
c
c
c
c
c                      SENSITIVITY ANALYSIS
c
c
c           The sensitivity of the harmonic mean rate (Mflops)
c           to various weightings is shown in the table below.
c           Seven work distributions are generated by assigning
c           two distinct weights to ranked kernels by quartiles.
c           Forty nine possible cpu workloads are then evaluated
c           using seven sets of values for the total weights:
c
c
c               ------ ------ ------ ------ ------ ------ ------
c     1st QT:       O      O      O      O      O      X      X
c     2nd QT:       O      O      O      X      X      X      O
c     3rd QT:       O      X      X      X      O      O      O
c     4th QT:       X      X      O      O      O      O      O
c               ------ ------ ------ ------ ------ ------ ------
c     Total
c     Weights                    Net Mflops:
c      X    O
c    ---- ----
c
c    1.00 0.00    3.25   4.82   9.32  14.03  28.34  44.62 104.84
c
c    0.95 0.05    3.39   5.05   9.28  13.22  24.64  31.58  60.38
c
c    0.90 0.10    3.55   5.29   9.23  12.50  21.79  24.44  42.40
c
c    0.80 0.20    3.90   5.87   9.15  11.27  17.69  16.83  26.57
c
c    0.70 0.30    4.34   6.58   9.06  10.26  14.90  12.84  19.35
c
c    0.60 0.40    4.88   7.50   8.98   9.42  12.86  10.37  15.21
c
c    0.50 0.50    5.58   8.70   8.90   8.70  11.32   8.70  12.53
c    ---- ----
c               ------ ------ ------ ------ ------ ------ ------
c
c
c
c
c
c
c SENSITIVITY OF NET MFLOPS RATE TO USE OF OPTIMAL FORTRAN CODE(SISD/SIMD MODEL)
c
c      4.94   6.11   7.99  11.55  14.87   20.86   34.93   52.71  107.33
c
c      0.00   0.20   0.40   0.60   0.70    0.80    0.90    0.95    1.00
c      Fraction Of Operations Run At Optimal Fortran Rates
c
c
c
c
c
c
c
c
c
c  Quadruple precision checksums computed by Dr. D.S. Lindsay, NAS:
c
c  CUMULATIVE CHECKSUMS:  RUN=    1
c
c  K             VL=  471                 90                 19
c  1 .5114652693224705102Q+05 .5253344778938000681Q+03 .3855104502494983491Q+02
c  2 .5150345372943066022Q+03 .5150345372943066022Q+03 .1199847611437483513Q+02
c  3 .1000742883066623145Q+02 .1009741436579188086Q+01 .2699309089321296439Q+00
c  4 .5999250595474070357Q+00 .5999250595474070357Q+00 .5999250595474070357Q+00
c  5 .4548871642388544199Q+04 .4589031939602131581Q+02 .3182615248448271678Q+01
c  6 .5229095383954675635Q+13 .2693280957416549457Q+16 .8303480073326955433Q+12
c  7 .6104251075163778121Q+05 .6345586315772524401Q+03 .2845720217638848365Q+02
c  8 .1501268005627157186Q+06 .1501268005627157186Q+06 .2960543667877649943Q+04
c  9 .1189443609975085966Q+06 .1189443609975085966Q+06 .2623968460874419268Q+04
c 10 .7310369784325972183Q+05 .7310369784325972183Q+05 .1651291227698377392Q+04
c 11 .3342910972650530561Q+08 .3433560407476162072Q+05 .6551161335846536530Q+03
c 12 .2907141428639174056Q-04 .7127569144561925151Q-05 .1943435981776804808Q-05
c 13 .4057110454105263471Q+10 .2325318944820836005Q+10 .4755211251524563699Q+09
c 14 .2982036205992255154Q+10 .3045676741897511424Q+08 .2547733008933910800Q+07
c 15 .3943816690352311804Q+05 .3943816690352311804Q+05 .1108997288135066584Q+04
c 16 .2832600000000000000Q+05 .3244100000000000000Q+05 .2577600000000000000Q+05
c 17 .1114641772903091760Q+04 .1114641772903091760Q+04 .2947368618590713935Q+02
c 18 .5165625410757306606Q+05 .5165625410757306606Q+05 .9700646212341513210Q+03
c 19 .5421816960150398899Q+03 .5421816960150398899Q+03 .1268230698051747067Q+02
c 20 .3040644339317275409Q+08 .3126205178811007613Q+05 .5987713249471801461Q+03
c 21 .8289464835786202431Q+07 .3986531136462291709Q+07 .2516870081042209239Q+07
c 22 .2938604376567099667Q+03 .2938604376567099667Q+03 .6109968728264795136Q+01
c 23 .3549834542446150511Q+05 .3549894609776936556Q+05 .4850340602751675804Q+03
c 24 .5000000000000000000Q+03 .5000000000000000000Q+02 .1300000000000000000Q+02
c
c
c  CUMULATIVE CHECKSUMS:  RUN=    2
c
c  K             VL=  471                 90                 19
c  1 .1022930538644941020Q+06 .1050668955787600136Q+04 .7710209004989966983Q+02
c  2 .1030069074588613204Q+04 .1030069074588613204Q+04 .2399695222874967026Q+02
c  3 .2001485766133246290Q+02 .2019482873158376173Q+01 .5398618178642592878Q+00
c  4 .1199850119094814071Q+01 .1199850119094814071Q+01 .1199850119094814071Q+01
c  5 .9097743284777088398Q+04 .9178063879204263162Q+02 .6365230496896543357Q+01
c  6 .1045819076790935127Q+14 .5386561914833098914Q+16 .1660696014665391086Q+13
c  7 .1220850215032755624Q+06 .1269117263154504880Q+04 .5691440435277696731Q+02
c  8 .3002536011254314373Q+06 .3002536011254314373Q+06 .5921087335755299887Q+04
c  9 .2378887219950171932Q+06 .2378887219950171932Q+06 .5247936921748838536Q+04
c 10 .1462073956865194436Q+06 .1462073956865194436Q+06 .3302582455396754785Q+04
c 11 .6685821945301061123Q+08 .6867120814952324144Q+05 .1310232267169307306Q+04
c 12 .5814282857278348113Q-04 .1425513828912385030Q-04 .3886871963553609616Q-05
c 13 .8114220908210526943Q+10 .4650637889641672010Q+10 .9510422503049127399Q+09
c 14 .6118901630090488734Q+10 .5718526521576222224Q+08 .5131714230651644153Q+07
c 15 .7887633380704623608Q+05 .7887633380704623608Q+05 .2217994576270133168Q+04
c 16 .5665200000000000000Q+05 .6488200000000000000Q+05 .5155200000000000000Q+05
c 17 .2229283545806183520Q+04 .2229283545806183520Q+04 .5894737237181427870Q+02
c 18 .1033125082151461321Q+06 .1033125082151461321Q+06 .1940129242468302642Q+04
c 19 .1084363392030079779Q+04 .1084363392030079779Q+04 .2536461396103494135Q+02
c 20 .6081288678634550819Q+08 .6252410357622015226Q+05 .1197542649894360292Q+04
c 21 .1657892967157240486Q+08 .7973062272924583418Q+07 .5033740162084418479Q+07
c 22 .5877208753134199335Q+03 .5877208753134199335Q+03 .1221993745652959027Q+02
c 23 .7099669084892301023Q+05 .7099789219553873113Q+05 .9700681205503351609Q+03
c 24 .1000000000000000000Q+04 .1000000000000000000Q+03 .2600000000000000000Q+02
c
c
c  CUMULATIVE CHECKSUMS:  RUN=    3
c
c  K             VL=  471                 90                 19
c  1 .1534395807967411530Q+06 .1576003433681400204Q+04 .1156531350748495047Q+03
c  2 .1545103611882919806Q+04 .1545103611882919806Q+04 .3599542834312450539Q+02
c  3 .3002228649199869435Q+02 .3029224309737564260Q+01 .8097927267963889317Q+00
c  4 .1799775178642221107Q+01 .1799775178642221107Q+01 .1799775178642221107Q+01
c  5 .1364661492716563259Q+05 .1376709581880639474Q+03 .9547845745344815036Q+01
c  6 .1568728615186402690Q+14 .8079842872249648371Q+16 .2491044021998086629Q+13
c  7 .1831275322549133436Q+06 .1903675894731757320Q+04 .8537160652916545097Q+02
c  8 .4503804016881471560Q+06 .4503804016881471560Q+06 .8881631003632949831Q+04
c  9 .3568330829925257898Q+06 .3568330829925257898Q+06 .7871905382623257805Q+04
c 10 .2193110935297791654Q+06 .2193110935297791654Q+06 .4953873683095132178Q+04
c 11 .1002873291795159168Q+09 .1030068122242848621Q+06 .1965348400753960959Q+04
c 12 .8721424285917522169Q-04 .2138270743368577545Q-04 .5830307945330414425Q-05
c 13 .1217133136231579041Q+11 .6975956834462508016Q+10 .1426563375457369109Q+10
c 14 .9103526877478772509Q+10 .8885029941358330359Q+08 .7946120246231201651Q+07
c 15 .1183145007105693541Q+06 .1183145007105693541Q+06 .3326991864405199752Q+04
c 16 .8497800000000000000Q+05 .9732300000000000000Q+05 .7732800000000000000Q+05
c 17 .3343925318709275281Q+04 .3343925318709275281Q+04 .8842105855772141805Q+02
c 18 .1549687623227191981Q+06 .1549687623227191981Q+06 .2910193863702453963Q+04
c 19 .1626545088045119669Q+04 .1626545088045119669Q+04 .3804692094155241203Q+02
c 20 .9121933017951826228Q+08 .9378615536433022839Q+05 .1796313974841540438Q+04
c 21 .2486839450735860729Q+08 .1195959340938687512Q+08 .7550610243126627718Q+07
c 22 .8815813129701299003Q+03 .8815813129701299003Q+03 .1832990618479438540Q+02
c 23 .1064950362733845153Q+06 .1064968382933080966Q+06 .1455102180825502741Q+04
c 24 .1500000000000000000Q+04 .1500000000000000000Q+03 .3900000000000000000Q+02
c
c
c  CUMULATIVE CHECKSUMS:  RUN=    4
c
c  K             VL=  471                 90                 19
c  1 .2045861077289882040Q+06 .2101337911575200272Q+04 .1542041800997993396Q+03
c  2 .2060138149177226409Q+04 .2060138149177226409Q+04 .4799390445749934052Q+02
c  3 .4002971532266492580Q+02 .4038965746316752347Q+01 .1079723635728518575Q+01
c  4 .2399700238189628143Q+01 .2399700238189628143Q+01 .2399700238189628143Q+01
c  5 .1819548656955417679Q+05 .1835612775840852632Q+03 .1273046099379308671Q+02
c  6 .2091638153581870254Q+14 .1077312382966619782Q+17 .3321392029330782173Q+13
c  7 .2441700430065511248Q+06 .2538234526309009760Q+04 .1138288087055539346Q+03
c  8 .6005072022508628747Q+06 .6005072022508628747Q+06 .1184217467151059977Q+05
c  9 .4757774439900343865Q+06 .4757774439900343865Q+06 .1049587384349767707Q+05
c 10 .2924147913730388873Q+06 .2924147913730388873Q+06 .6605164910793509571Q+04
c 11 .1337164389060212224Q+09 .1373424162990464828Q+06 .2620464534338614612Q+04
c 12 .1162856571455669622Q-03 .2851027657824770060Q-04 .7773743927107219233Q-05
c 13 .1622844181642105388Q+11 .9301275779283344021Q+10 .1902084500609825479Q+10
c 14 .1215176334476067437Q+11 .1174925822726987451Q+09 .1008019578807808401Q+08
c 15 .1577526676140924721Q+06 .1577526676140924721Q+06 .4435989152540266336Q+04
c 16 .1133040000000000000Q+06 .1297640000000000000Q+06 .1031040000000000000Q+06
c 17 .4458567091612367041Q+04 .4458567091612367041Q+04 .1178947447436285574Q+03
c 18 .2066250164302922642Q+06 .2066250164302922642Q+06 .3880258484936605284Q+04
c 19 .2168726784060159559Q+04 .2168726784060159559Q+04 .5072922792206988271Q+02
c 20 .1216257735726910163Q+09 .1250482071524403045Q+06 .2395085299788720584Q+04
c 21 .3315785934314480972Q+08 .1594612454584916683Q+08 .1006748032416883695Q+08
c 22 .1175441750626839867Q+04 .1175441750626839867Q+04 .2443987491305918054Q+02
c 23 .1419933816978460204Q+06 .1419957843910774622Q+06 .1940136241100670321Q+04
c 24 .2000000000000000000Q+04 .2000000000000000000Q+03 .5200000000000000000Q+02
c
c
c
c
c  CUMULATIVE CHECKSUMS:  RUN=    5
c
c  K             VL=  471                 90                 19
c  1 .2557326346612352551Q+06 .2626672389469000340Q+04 .1927552251247491745Q+03
c  2 .2575172686471533011Q+04 .2575172686471533011Q+04 .5999238057187417565Q+02
c  3 .5003714415333115725Q+02 .5048707182895940434Q+01 .1349654544660648219Q+01
c  4 .2999625297737035178Q+01 .2999625297737035178Q+01 .2999625297737035178Q+01
c  5 .2274435821194272099Q+05 .2294515969801065790Q+03 .1591307624224135839Q+02
c  6 .2614547691977337817Q+14 .1346640478708274728Q+17 .4151740036663477716Q+13
c  7 .3052125537581889060Q+06 .3172793157886262200Q+04 .1422860108819424182Q+03
c  8 .7506340028135785934Q+06 .7506340028135785934Q+06 .1480271833938824971Q+05
c  9 .5947218049875429831Q+06 .5947218049875429831Q+06 .1311984230437209634Q+05
c 10 .3655184892162986091Q+06 .3655184892162986091Q+06 .8256456138491886963Q+04
c 11 .1671455486325265280Q+09 .1716780203738081036Q+06 .3275580667923268265Q+04
c 12 .1453570714319587028Q-03 .3563784572280962575Q-04 .9717179908884024042Q-05
c 13 .2028555227052631735Q+11 .1162659472410418002Q+11 .2377605625762281849Q+10
c 14 .1519764492169999253Q+11 .1501582054695641385Q+09 .1269997234773526700Q+08
c 15 .1971908345176155902Q+06 .1971908345176155902Q+06 .5544986440675332920Q+04
c 16 .1416300000000000000Q+06 .1622050000000000000Q+06 .1288800000000000000Q+06
c 17 .5573208864515458802Q+04 .5573208864515458802Q+04 .1473684309295356967Q+03
c 18 .2582812705378653303Q+06 .2582812705378653303Q+06 .4850323106170756605Q+04
c 19 .2710908480075199449Q+04 .2710908480075199449Q+04 .6341153490258735339Q+02
c 20 .1520322169658637704Q+09 .1563102589405503806Q+06 .2993856624735900730Q+04
c 21 .4144732417893101215Q+08 .1993265568231145854Q+08 .1258435040521104619Q+08
c 22 .1469302188283549833Q+04 .1469302188283549833Q+04 .3054984364132397568Q+02
c 23 .1774917271223075255Q+06 .1774947304888468278Q+06 .2425170301375837902Q+04
c 24 .2500000000000000000Q+04 .2500000000000000000Q+03 .6500000000000000000Q+02
c
c
c  CUMULATIVE CHECKSUMS:  RUN=    6
c
c  K             VL=  471                 90                 19
c  1 .3068791615934823061Q+06 .3152006867362800409Q+04 .2313062701496990095Q+03
c  2 .3090207223765839613Q+04 .3090207223765839613Q+04 .7199085668624901078Q+02
c  3 .6004457298399738870Q+02 .6058448619475128521Q+01 .1619585453592777863Q+01
c  4 .3599550357284442214Q+01 .3599550357284442214Q+01 .3599550357284442214Q+01
c  5 .2729322985433126519Q+05 .2753419163761278948Q+03 .1909569149068963007Q+02
c  6 .3137457230372805381Q+14 .1615968574449929674Q+17 .4982088043996173259Q+13
c  7 .3662550645098266873Q+06 .3807351789463514640Q+04 .1707432130583309019Q+03
c  8 .9007608033762943120Q+06 .9007608033762943120Q+06 .1776326200726589966Q+05
c  9 .7136661659850515797Q+06 .7136661659850515797Q+06 .1574381076524651561Q+05
c 10 .4386221870595583309Q+06 .4386221870595583309Q+06 .9907747366190264356Q+04
c 11 .2005746583590318337Q+09 .2060136244485697243Q+06 .3930696801507921918Q+04
c 12 .1744284857183504433Q-03 .4276541486737155090Q-04 .1166061589066082885Q-04
c 13 .2434266272463158082Q+11 .1395191366892501603Q+11 .2853126750914738219Q+10
c 14 .1820312504465359131Q+11 .1819691693283694618Q+09 .1504905863862026021Q+08
c 15 .2366290014211387082Q+06 .2366290014211387082Q+06 .6653983728810399504Q+04
c 16 .1699560000000000000Q+06 .1946460000000000000Q+06 .1546560000000000000Q+06
c 17 .6687850637418550562Q+04 .6687850637418550562Q+04 .1768421171154428361Q+03
c 18 .3099375246454383963Q+06 .3099375246454383963Q+06 .5820387727404907926Q+04
c 19 .3253090176090239339Q+04 .3253090176090239339Q+04 .7609384188310482407Q+02
c 20 .1824386603590365245Q+09 .1875723107286604567Q+06 .3592627949683080876Q+04
c 21 .4973678901471721458Q+08 .2391918681877375025Q+08 .1510122048625325543Q+08
c 22 .1763162625940259800Q+04 .1763162625940259800Q+04 .3665981236958877081Q+02
c 23 .2129900725467690306Q+06 .2129936765866161933Q+06 .2910204361651005482Q+04
c 24 .3000000000000000000Q+04 .3000000000000000000Q+03 .7800000000000000000Q+02
c
c
c
c  CUMULATIVE CHECKSUMS:  RUN=    7
c
c  K             VL=  471                 90                 19
c  1 .3580256885257293571Q+06 .3677341345256600477Q+04 .2698573151746488444Q+03
c  2 .3605241761060146215Q+04 .3605241761060146215Q+04 .8398933280062384591Q+02
c  3 .7005200181466362015Q+02 .7068190056054316608Q+01 .1889516362524907507Q+01
c  4 .4199475416831849250Q+01 .4199475416831849250Q+01 .4199475416831849250Q+01
c  5 .3184210149671980939Q+05 .3212322357721492106Q+03 .2227830673913790175Q+02
c  6 .3660366768768272944Q+14 .1885296670191584620Q+17 .5812436051328868803Q+13
c  7 .4272975752614644685Q+06 .4441910421040767080Q+04 .1992004152347193856Q+03
c  8 .1050887603939010030Q+07 .1050887603939010030Q+07 .2072380567514354960Q+05
c  9 .8326105269825601763Q+06 .8326105269825601763Q+06 .1836777922612093487Q+05
c 10 .5117258849028180528Q+06 .5117258849028180528Q+06 .1155903859388864174Q+05
c 11 .2340037680855371393Q+09 .2403492285233313450Q+06 .4585812935092575571Q+04
c 12 .2034999000047421839Q-03 .4989298401193347605Q-04 .1360405187243763365Q-04
c 13 .2839977317873684430Q+11 .1627723261374585203Q+11 .3328647876067194589Q+10
c 14 .2116750694993432503Q+11 .2130649341195080691Q+09 .1721399839433381833Q+08
c 15 .2760671683246618263Q+06 .2760671683246618263Q+06 .7762981016945466088Q+04
c 16 .1982820000000000000Q+06 .2270870000000000000Q+06 .1804320000000000000Q+06
c 17 .7802492410321642323Q+04 .7802492410321642323Q+04 .2063158033013499754Q+03
c 18 .3615937787530114624Q+06 .3615937787530114624Q+06 .6790452348639059247Q+04
c 19 .3795271872105279229Q+04 .3795271872105279229Q+04 .8877614886362229475Q+02
c 20 .2128451037522092786Q+09 .2188343625167705329Q+06 .4191399274630261022Q+04
c 21 .5802625385050341702Q+08 .2790571795523604196Q+08 .1761809056729546467Q+08
c 22 .2057023063596969767Q+04 .2057023063596969767Q+04 .4276978109785356595Q+02
c 23 .2484884179712305358Q+06 .2484926226843855589Q+06 .3395238421926173063Q+04
c 24 .3500000000000000000Q+04 .3500000000000000000Q+03 .9100000000000000000Q+02
c
c
c
c
c
c
c   CHECK CLOCK CALIBRATION:
c   Total cpu Time =    1.62806e+01 Sec.
c
C                                                    F.H.MCMAHON  1986
C**********************************************************************
c
c
C
C
C
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
      DOUBLE PRECISION  sum
C
C/      PARAMETER( kn= 47, kn2= 95, np= 3, ls= 3*47, krs= 24)
C/      PARAMETER( nk= 47, nl= 3, nr= 8 )
C
      COMMON /ALPHA/ mk,ik,ml,il,Nruns,jr, NPFS(8,3,47)
      COMMON /BETA / tic, TIMES(8,3,47), SEE(5,3,8,3),
     1              TERRS(8,3,47), CSUMS(8,3,47),
     2              FOPN(8,3,47), DOS(8,3,47)
C
      COMMON /SPACE0/ TIME(47), CSUM(47), WW(47), WT(47), ticks,
     1                FR(9), TERR1(47), SUMW(7), START,
     2              SKALE(47), BIAS(47), WS(95), TOTAL(47), FLOPN(47),
     3                IQ(7), NPF, NPFS1(47)
C
      COMMON /SPACEI/ WTP(3), MUL(3), ISPAN(47,3), IPASS(47,3)
C
      DIMENSION  FLOPS(141), TR(141), RATES(141)
      DIMENSION  LSPAN(141), WG(141), OSUM (141), ID(141)
      DIMENSION  HM(12), N1(10), N2(10), LVL(10)
      DIMENSION  LQ(5), STAT1(20), STAT2(20)
      DIMENSION  IN(141), CSUM1(141)
      DIMENSION  MAP1(141), MAP2(141), MAP3(141), IN2(141), VL1(141)
      DIMENSION  MAP(141), VL(141), WL(141), TV(141), TV1(141), TV2(141)
      DIMENSION  FLOPS1(141), RT1(141), ISPAN1(141), WT1(141)
      DIMENSION  FLOPS2(141), RT2(141), ISPAN2(141), WT2(141)
      SAVE       Komput, Kontrl, Kompil
C
C     The Fortran-77 Character variables Komput, Kontrl, Kompil, Tag  are
C     used in Format statements  7007, 7057, 7008, 7341, 7342, 7343, 7344
C
      CHARACTER  Komput*24, Kontrl*24, Kompil*24                         f77
C
       MODI(i,M)= (MOD( ABS(i), M) + M*( 1 - MIN( 1, MOD( ABS(i), M))))
C
c     DATA       Komput   /'VAX-11/780  w/FPA'/
c     DATA       Komput   /'VAX-11/780 wo/FPA'/
c     DATA       Komput   /'CRAY-1                  '/                   f77
c     DATA       Komput   /'CRAY-XMP                '/                   f77
c     DATA       Komput   / 8HCRAY-XMP/                                  f66
c     DATA       Komput   / 8HCRAY-1  /                                  f66
      DATA       Komput   /'MIPS M/2000'/
C
c     DATA       Kontrl   /'VMS  4.1BSD'/
c     DATA       Kontrl   /'UNIX 4.2BSD'/
c     DATA       Kontrl   /'UNIX 4.3BSD'/
c     DATA       Kontrl   /'CTSS 10e  fully loaded  '/                   f77
c     DATA       Kontrl   / 8HCTSS 10e/                                  f66
      DATA       Kontrl   /'RISC/os 3.1'/
C
c     DATA       Kompil   /'VAX/Fortran-77 V4.0     '/
c     DATA       Kompil   /'4.2BSD UNIX f77'/
c     DATA       Kompil   /'4.2BSD UNIX f77'/
c     DATA       Kompil   /'fortrel 0.4'/
c     DATA       Kompil   /'LCC CC 1.05             '/                   f77
c     DATA       Kompil   /'CFT 1.14                '/                   f77
c     DATA       Kompil   /'CIVIC 30i               '/                   f77
c     DATA       Kompil   / 8HCFT-1.14/                                  f66
c     DATA       Kompil   / 8HCIVIC30i/                                  f66
      DATA       Kompil   /'f771.31 -O'/

C
      CALL TRACKS('REPORT  ')
C
          IF( iou.LT.0) GO TO 73
C
            fuzz= 1.0e-9
       DO 1000 k= 1,ntk
           VL(k)= LSPAN(k)
 1000  CONTINUE
C
              bl= 1.0D-5
              bu= 1.0D+4
            CALL  VALID( TV,MAP,neff,  bl, RATES, bu, ntk)
C
C      Compress valid data sets mapping on MAP.
C
              nd= 0
        DO  1  k= 1,neff
         MAP1(k)=  MODI( MAP(k),nek)
       FLOPS1(k)= FLOPS( MAP(k))
          RT1(k)=    TR( MAP(k))
          VL1(k)=    VL( MAP(k))
       ISPAN1(k)= LSPAN( MAP(k))
          WT1(k)=    WG( MAP(k))
          TV1(k)= RATES( MAP(k))
        CSUM1(k)=  OSUM( MAP(k))
              nd=    ID( MAP(k)) + nd
    1  continue
C
            CALL  STATW( STAT1,TV,IN, VL1,WT1,neff)
              LV= STAT1(1)
C
            CALL  STATW( STAT1,TV,IN, TV1,WT1,neff)
             twt= STAT1(6)
C
          WRITE ( iou,7001)
          WRITE ( iou,7001)
          WRITE ( iou,7001)
          WRITE ( iou,7001)
          WRITE ( iou,7001)
          WRITE ( iou,7001)
       CALL PAGE( iou)
          WRITE ( iou,7002)
          WRITE ( iou,7003)
          WRITE ( iou,7002)
          WRITE ( iou,7007)  Komput
          WRITE ( iou,7057)  Kontrl
          WRITE ( iou,7008)  Kompil
          WRITE ( iou,7009)  LV
          WRITE ( iou,7061)
          WRITE ( iou,7062)
          WRITE ( iou,7063)
          WRITE ( iou,7064)
          WRITE ( iou,7065)
          WRITE ( iou,7066)
          WRITE ( iou,7067)
          WRITE ( iou,7001)
          WRITE ( iou,7004)
          WRITE ( iou,7005)
          WRITE ( iou,7011) (MAP1(k),  FLOPS1(k), RT1(k), TV1(k),
     .                    ISPAN1(k), WT1(k), CSUM1(k), ID(k), k=1,neff)
          WRITE ( iou,7005)
C
          WRITE ( iou,7022)  nd
          WRITE ( iou,7041)  STAT1( 4)
          WRITE ( iou,7033)  STAT1( 1)
          WRITE ( iou,7043)  STAT1(10)
          WRITE ( iou,7030)  STAT1( 7)
          WRITE ( iou,7055)  STAT1( 5)
          WRITE ( iou,7042)  STAT1( 3)
          WRITE ( iou,7001)
          WRITE ( iou,7044)  STAT1( 2)
          WRITE ( iou,7031)  STAT1( 9)
C
 7001 FORMAT(/)
 7002 FORMAT(  45H ******************************************** )
 7003 FORMAT(  45H THE LIVERMORE  FORTRAN KERNELS:  M F L O P S  )
 7004 FORMAT(/,53H KERNEL  FLOPS   MICROSEC   MFLOP/SEC SPAN WEIGHT  CH,
     X19HECK-SUMS         OK )
 7005 FORMAT(  53H ------  -----   --------   --------- ---- ------  --,
     X19H---------------- -- )
 7007 FORMAT(/,9X,16H     Computer :  ,A )                               f77
 7057 FORMAT(  9X,16H     System   :  ,A )                               f77
 7008 FORMAT(  9X,16H     Compiler :  ,A )                               f77
c7007 FORMAT(/,9X,16H     Computer :  ,A8)                               f66
c7057 FORMAT(  9X,16H     System   :  ,A8)                               f66
c7008 FORMAT(  9X,16H     Compiler :  ,A8)                               f66
 7009 FORMAT(/,9X,16HMean  DO Span =  ,I5,/)
 7011 FORMAT(1X,i2,E11.4,E11.4,F12.4,1X,I4,1X,F6.2,E20.12,1X,I2)
 7012 FORMAT(1X,i2,E11.4,E11.4,F12.4,1X,I4,1X,F6.2)
 7022 FORMAT(  9X,16HMFLOPS   Range :  ,43X,I4)
 7041 FORMAT(/,9X,16HMaximum   Rate =  ,F12.4,16H Mega-Flops/Sec. )
 7033 FORMAT(  9X,16HAverage   Rate =  ,F12.4,16H Mega-Flops/Sec. )
 7043 FORMAT(  9X,16HGeometric Mean =  ,F12.4,16H Mega-Flops/Sec. )
 7030 FORMAT(  9X,16HMedian    Rate =  ,F12.4,16H Mega-Flops/Sec. )
 7055 FORMAT(  9X,16HHarmonic  Mean =  ,F12.4,16H Mega-Flops/Sec. )
 7042 FORMAT(  9X,16HMinimum   Rate =  ,F12.4,16H Mega-Flops/Sec. )
 7044 FORMAT(  9X,16HStandard  Dev. =  ,F12.4,16H Mega-Flops/Sec. )
 7031 FORMAT(  9X,16HMedian    Dev. =  ,F12.4,16H Mega-Flops/Sec. )
 7053 FORMAT(/,9X,16HFrac.  Weights =  ,F12.4)
 7104 FORMAT(/,50H KERNEL  FLOPS   MICROSEC   MFLOP/SEC SPAN WEIGHT   )
 7105 FORMAT(  50H ------  -----   --------   --------- ---- ------   )
C
 7061 FORMAT(9X,52HWhen the computer performance range is very large   )
 7062 FORMAT(9X,52Hthe net Mflops rate of many Fortran programs and    )
 7063 FORMAT(9X,52Hworkloads will be in the sub-range between the equi-)
 7064 FORMAT(9X,52Hweighted harmonic and arithmetic means depending    )
 7065 FORMAT(9X,52Hon the degree of code parallelism and optimization. )
 7066 FORMAT(9X,52HMore accurate estimates of cpu workload rates depend)
 7067 FORMAT(9X,52Hon assigning appropriate weights for each kernel.   )
C
      IF( ntk .NE. nek )  THEN
C
       CALL PAGE( iou)
          WRITE ( iou,7070)
 7070 FORMAT(//,50H TOP QUARTILE: BEST ARCHITECTURE/APPLICATION MATCH )
C
C      Compute compression index-list MAP1:  Non-zero weights.
C
              bl= 1.0D-6
              bu= 1.0D+6
            CALL  VALID( TV,MAP1,meff,  bl, WT1, bu, neff)
C
C      Re-order data sets mapping on IN (descending order of MFlops).
C
        DO  2  k= 1,meff
         MAP3(k)=     IN( MAP1(k))
    2  continue
C
      IF( meff.GT.0 )  THEN
          CALL TRAP( MAP3, 8H REPORT  , 1, neff,meff)
      ENDIF
C
        DO  3  k= 1,meff
               i=   MAP3(k)
        FLOPS2(k)=  FLOPS1(i)
          RT2(k)=    RT1(i)
       ISPAN2(k)= ISPAN1(i)
          WT2(k)=    WT1(i)
          TV2(k)=    TV1(i)
         MAP2(k)=   MODI( MAP(i),nek)
    3  continue
C
              nq= meff/4
              lo= meff -4*nq
           LQ(1)= nq
           LQ(2)= nq + nq + lo
           LQ(3)= nq
              i2= 0
C
         DO 5  j= 1,3
              i1= i2 + 1
              i2= i2 + LQ(j)
              ll= i2 - i1 + 1
            CALL  STATW( STAT2,TV,IN2, TV2(i1),WT2(i1),ll)
            frac= STAT2(6)/( twt +fuzz)
           WL(j)= STAT2(5)
C
          WRITE ( iou,7001)
          WRITE ( iou,7104)
          WRITE ( iou,7105)
          WRITE ( iou,7012) ( MAP2(k),  FLOPS2(k), RT2(k), TV2(k),
     .                         ISPAN2(k), WT2(k),  k=i1,i2 )
          WRITE ( iou,7105)
C
          WRITE ( iou,7053)  frac
          WRITE ( iou,7033)  STAT2(1)
          WRITE ( iou,7055)  STAT2(5)
          WRITE ( iou,7044)  STAT2(2)
    5 continue
      ENDIF
C
C
C           Sensitivity analysis of harmonic mean rate to 49 workloads
C
      CALL  SENSIT(   iou,RATES,WG,IQ,SUMW, MAP,TV,TV1,TV2,RT1, ntk)
C
C
C           Sensitivity analysis of harmonic mean rate to SISD/SIMD model
C
      CALL  SIMD( HM, iou,RATES,WG,FR,9,    MAP,TV,TV1,TV2, ntk)
C
C
      IF( ntk .NE. nek )  THEN
               CALL  PAGE( iou)
               mrl= Nruns
                IF( Nruns.gt.8) mrl= 8
C
      DO  8      k= 1,mk
      DO  8      j= 1,ml
               sum= 0.
      DO  8      i= 1,mrl
               sum= sum + CSUMS(i,j,k)
      CSUMS(i,j,k)= sum
    8 continue
C
      DO  10     i= 1,mrl
                IF( (i.NE.1).AND.(i.NE.mrl))  GO TO 10
             WRITE( iou,76) i
             WRITE( iou,77)  ( LVL(j), j= 1,3 )
   76       FORMAT( //,29h  Cumulative Checksums:  RUN=,i5)
   77       FORMAT( /,10h  k    VL=,i5,3i19)
C
      DO  9      k= 1,mk
             WRITE( iou,78)  k, ( CSUMS(i,j,k), j= 1,3)
   78       FORMAT( 1x,i2,4e19.12)
    9 continue
   10 continue
C
      ENDIF
          LVL(il)= LV
   73 CONTINUE
      CALL TRACKX
      RETURN
C
      END
C**********************************************
      SUBROUTINE RESULT( iou,FLOPS,TR,RATES,LSPAN,WG,OSUM,TERR,ID)
C***********************************************************************
C                                                                      *
C     RESULT -  Computes timing Results into pushdown store.           *
C                                                                      *
C      iou   -  Input   IO unit number for print output                *
C     FLOPS  - Out.Ary  Number of Flops executed by each kernel        *
C     TR     - Out.Ary  Time of execution of each kernel(microsecs)    *
C     RATES  - Out.Ary  Rate of execution of each kernel(megaflops/sec)*
C     LSPAN  - Out.Ary  Span of inner DO loop in each kernel           *
C     WG     - Out.Ary  Weight assigned to each kernel for statistics  *
C     OSUM   - Out.Ary  Checksums of the results of each kernel        *
C     TERR   - Out.Ary  Experimental timing errors per kernel          *
C     ID     - Out.Ary  Number of valid digits in checksum.            *
C                                                                      *
C***********************************************************************
C
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
      DOUBLE PRECISION  SUMS, cs
C
C/      PARAMETER( kn= 47, kn2= 95, np= 3, ls= 3*47, krs= 24)
C/      PARAMETER( nk= 47, nl= 3, nr= 8 )
C
      COMMON /ALPHA/ mk,ik,ml,il,Nruns,jr, NPFS(8,3,47)
      COMMON /BETA / tic, TIMES(8,3,47), SEE(5,3,8,3),
     1              TERRS(8,3,47), CSUMS(8,3,47),
     2              FOPN(8,3,47), DOS(8,3,47)
C
      DIMENSION  FLOPS(141), TR(141), RATES(141), ID(141)
      DIMENSION  LSPAN(141), WG(141), OSUM (141), TERR(141)
C
      COMMON /SPACE0/ TIME(47), CSUM(47), WW(47), WT(47), ticks,
     1                FR(9), TERR1(47), SUMW(7), START,
     2              SKALE(47), BIAS(47), WS(95), TOTAL(47), FLOPN(47),
     3                IQ(7), NPF, NPFS1(47)
C
      COMMON /SPACEI/ WTP(3), MUL(3), ISPAN(47,3), IPASS(47,3)
C
      COMMON /PROOF/  SUMS(24,3,2)
C
C
      CALL TRACKS('RESULT  ')
C
           CALL  TALLY( iou, 1 )
C
C                             Push Result Arrays Down before entering new result
           limit= 141 - mk
              j = 141
      DO 1001 k = limit,1,-1
        FLOPS(j)= FLOPS(k)
           TR(j)=    TR(k)
        RATES(j)= RATES(k)
        LSPAN(j)= LSPAN(k)
           WG(j)=    WG(k)
         OSUM(j)=  OSUM(k)
         TERR(j)=  TERR(k)
           ID(j)=    ID(k)
              j = j - 1
 1001 CONTINUE
C
C                             CALCULATE MFLOPS FOR EACH KERNEL
            tmin= 5.0*tic
      DO 1010 k = 1,mk
        FLOPS(k)= FLOPN(k)*TOTAL(k)
           TR(k)=  TIME(k) * 1.0e+6
        RATES(k)= 0.0
              IF( TR(k).NE. 0.0)   RATES(k)= FLOPS(k)/TR(k)
              IF( WT(k).LE. 0.0)   RATES(k)= 0.0
              IF( TIME(k).LE.tmin) RATES(k)= 0.0
        LSPAN(k)= ISPAN(k,il)
           WG(k)= WT(k)*WTP(il)
         OSUM(k)= CSUM(k)
         TERR(k)= TERR1(k)
c
c                 compute relative error and digits of precision in CSUM
c
              cs= DBLE( Nruns) * SUMS(k,il,1)
              IF( k .EQ. 14  )  cs= SUMS( Nruns,il,2)
                                re= ABS(  CSUM(k))
              IF( cs.NE. 0.0 )  re= ABS( (CSUM(k) - cs)/cs)
              IF((re.GT. 0.0 ) .AND. (re.LT. 1.0))  THEN
                  ID(k)= INT( ABS( LOG10( re)) + 0.9999999 )
              ELSE
                  ID(k)= 16
              ENDIF
              IF(    re.GE. 1.0 )  ID(k)= 0
              IF( ID(k).GT. 16  )  ID(k)= 16
 1010 CONTINUE
C
      CALL TRACKX
      RETURN
C
      END
C
C
C Use external SECOND function on 4.3 systems for maximum resolution
C**********************************************
C     function   SECOND( OLDSEC)
C***********************************************
CANSI IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C     REAL*8 SECOND
C
C     SECOND= Cumulative CPU time for job in seconds.  MKS unit is seconds.
C             Clock resolution should be less than 2% of Kernel 12 run-time.
C             ONLY CPU time should be measured, NO system or I/O time included.
C             In VM systems, page-fault time must be avoided (Direction 4).
C             SECOND accuracy may be tested by calling the test program VERIFY.
C
C     If your system provides a timing routine that satisfies
C     the definition above; then simply delete this function.
C
C     Else this function must be programmed using some
C     timing routine available in your system.
C     Timing routines with CPU clock resolution are always sufficient.
C     Timing routines with microsec. resolution are usually sufficient.
C
C     Timing routines with much less resolution may require the use
C     of multiple-pass loops around each kernel to make the run time
C     at least 50 times the tick-period of the timing routine.
C     To reduce timing errors you can increase the value of Loop
C     in subroutine SIZES.
C
C     If no CPU timer is available, then you can time each kernel by
C     the wall clock using the PAUSE statement at the end of subr. VALUES.
C
C     Function TICK measures the overhead time for a call to SECOND.
C     An independent calibration of the running time of at least
C     one kernel may be wise.  See Subroutine VERIFY.
C
C
C     The following statement is deliberately incomplete:
C
C     SECOND=                                                            sdef
C
C*******************************************************************************
C
C     The following statements were used on the DEC  VAX/780  VMS 3.0 .
C     Enable page-fault tallys in TEST by un-commenting LIB$STAT_TIMER calls.
C     Also set:   Loop= 10*Loop in Subroutine SIZES to run kernels long enough.
C
C       DATA  INITIA   /123/
C       IF(   INITIA.EQ.123 )  THEN
C             INITIA= 1
C             NSTAT = LIB$INIT_TIMER()
C       ELSE
C             NSTAT = LIB$STAT_TIMER(2,ISEC)
C             SECOND= DBLE(ISEC)*0.01 - OLDSEC
C       ENDIF
C
C* OR less accurately:
C*        REAL    SECNDS
C*        SECOND= SECNDS( OLDSEC)
C
C*******************************************************************************
C
C     The following statements were used on  UNIX 4.2bsd systems, e.g.  SUN
C     Also set:   Loop= 10*Loop in Subroutine SIZES to run kernels long enough.
C
C*     REAL OLDSEC
C*     DIMENSION XTIME(2)                                               unix
C*     REAL*4 XTIME                                                     unix
C*     XT= ETIME( XTIME)                                                unix
C*     SECOND=  XTIME(1) - OLDSEC                                       unix
C
C*******************************************************************************
C
C     The following statements were used on the DEC PDP-11/23 RT-11 system.
C     Also set:   Loop=100*Loop in Subroutine SIZES to run kernels long enough.
C
C*       DIMENSION JT(2)
C*       CALL GTIM(JT)
C*       TIME1 = JT(1)
C*       TIME2 = JT(2)
C*       TIME = TIME1 * 65768. + TIME2
C*       SECOND=TIME/60. - OLDSEC
C
C*******************************************************************************
C
C     The following statements were used on the Hewlett-Packard HP 9000
C     Also set:   Loop= 10*Loop in Subroutine SIZES to run kernels long enough.
C
C*       INTEGER*4 ITIME(4)
C*       CALL TIMES( ITIME(4))
C*       TIMEX= ITIME(1) + ITIME(2) + ITIME(3) + ITIME(4)
C*       SECOND= TIMEX/60. - OLDSEC
C
C*******************************************************************************
C
C     The following statement was used on the IBM 3090  MVS
C
c     CALL TOD( itime)
c     xtime = ( DFLOAT( itime)/64.0) * 1.0D-6
c     SECOND= xtime - oldsec
C
C
C*******************************************************************************
C
C     FOR THE GOULD 32/87 WITH MPX 3.2  (et seq. gratis D.Lindsay)
C
C     INTEGER*4 NSEC, NCLICK
C     REAL*8 CPUTIM
C
C      CALL M:CLOCK (NSEC, NCLICK)
C      CPUTIM = FLOAT(NSEC)
C      SECOND = CPUTIM + FLOAT(NCLICK)/60.
C
C*******************************************************************************
C
C  FOR THE HP 1000 RUNNING FORTRAN 77.
C  note that since the hp operating system has no facility for
C  returning cpu time, this routine only measures elapsed time.
C  therefore, the tests must be run stand-alone.
C
C     REAL*8 TOTIME
C     INTEGER*2 TIMEA(5)
C
C     CALL EXEC (11, TIMEA)
C     TOTIME = DBLE (TIMEA(1))/100.
C     TOTIME = TOTIME + DBLE (TIMEA(2))
C     TOTIME = TOTIME + DBLE (TIMEA(3)) * 60.
C     SECOND = TOTIME + DBLE (TIMEA(4)) * 3600.
C
C*******************************************************************************
C
C  FOR THE IBM PC.
C  note that the pc's operating system has no facility for
C  returning cpu time; this routine only measures elapsed time.
C  also, the pc does not have real*8.  Remove all references to real*8
C
C      IMPLICIT INTEGER*4 (I-N)
C      LOGICAL FIRST
C      DATA FIRST /.TRUE./
C
C      CALL GETTIM (IYEAR, IMONTH, IDAY, IHOUR, IMIN, ISEC, IFRACT)
C
C  ifract is integer fractions of a second
C  in units of 1/32,768 seconds
C
C      IF (.NOT. FIRST) GO TO 10
C        FIRST = .FALSE.
C
C        LASTHR = IHOUR
C        BASETM = 0.
C10    CONTINUE
C
C  because of limited precision, do not include the time of day
C  in hours in the total time.  but correct for an hour change.
C
C      IF (LASTHR .EQ. IHOUR) GO TO 20
C        BASETM = BASETM + 3600.
C        LASTHR = IHOUR
C
C20    TOTIME = FLOAT(IMIN) * 60
C    . + FLOAT(ISEC)
C    . + FLOAT(IFRACT)/32768.
C      SECOND = TOTIM + BASETM
C
C*******************************************************************************
C
C     FOR THE PR1ME SYSTEM UNDER PRIMOS
C
C     REAL*8 CPUTIM
C     INTEGER*2 TIMERS (28)
C
C     CALL TMDAT (TIMERS)
C     SECOND = DBLE (TIMERS(7))
C    .+ DBLE(TIMERS(8)) / DBLE(TIMERS(11))
C
C
C
C
C
C
C*******************************************************************************
C
C     FOR IBM M V S    (Calls following MVSTIM assembly code)
C
C   the microsecond clock accessed by the routine MVSTIM counts down
C   from about 2 billion (2**31).  so when it gets below 1 minute,
C   reset it to prevent underflow.
C
C   a call to MVSTIM with an argument of 0 resets the clock;
C   any other argument causes the counted-down clock to be returned.
C
C     IMPLICIT REAL*8 (T)
C     LOGICAL FIRST
C     DATA FIRST /.TRUE./
C
C     IF (.NOT. FIRST) GO TO 5
C       L = 0
C       CALL MVSTIM (L)
C       FIRST = .FALSE.
C       TBASE = L
C       TBASE = TBASE/1.D6
C5     CONTINUE
C
C   always get current time (even if first entry)
C
C      CALL MVSTIM (L)
C      TEMP = L
C      TEMP = TEMP/1.D6
C      SECOND = TBASE - TEMP
C      IF (L .GT. 60000000) RETURN
C
C   here we reset the counter if less than one minute left.
C
C      L = 0
C      CALL MVSTIM (L)
C      TNEW = L
C     TNEW = TNEW/1.D6
C     TBASE = TBASE + TNEW - TEMP
C     RETURN
C     END
C
C for MVS systems, remove these 2 cards, move all the following
C assembler code 1 space left (to remove column 1 stuff), and assembl
CMVSTIM   CSECT
C         PRINT GEN
C         USING *,12
C********************************************************************
C* STANDARD SUBROUTINE LINKAGE FROM FORTRAN PROGRAM
C* GPR  1: ARGUMENT LIST ADDRESS FROM CALLER
C* GPR 13: CALLER'S SAVEAREA ADDRESS
C* GPR 14: ADDRESS IN CALLER FOR RETURN
C* GPR 15: ENTRY ADDRESS OF THIS SUBPROGRAM
C********************************************************************
C* REGISTER USAGE IN THIS PROGRAM...
C* GPR  0: RETURN OF ELAPSED MICROSECONDS TO FORTRAN
C* GPR  1: ARGUMENT LIST ADDRESS FROM CALLER
C/C* GPR  2: ADDRESS OF PARAMETER PASSED FROM FORTRAN
C* GPR  3: WORK REGISTER
C* GPR  4: WORK REGISTER
C* GPR  5: WORK REGISTER
C* GPR  6: NOT USED
C* GPR  7: NOT USED
C* GPR  8: NOT USED
C* GPR  9: NOT USED
C* GPR 10: NOT USED
C* GPR 11: NOT USED
C* GPR 12: BASE REGISTER FOR THIS PROGRAM
C* GPR 13: NOT USED
C* GPR 14: NOT USED
C* GPR 15: RETURN CODE TO FORTRAN IN CASE OF ERROR IN PROCESSING
C********************************************************************
C********************************************************************
C*                      REGISTER EQUATES
C********************************************************************
CR0       EQU   0
CR1       EQU   1
CR2       EQU   2
CR3       EQU   3
CR4       EQU   4
CR5       EQU   5
CR6       EQU   6
CR7       EQU   7
CR8       EQU   8
CR9       EQU   9
CR10      EQU   10
CR11      EQU   11
CR12      EQU   12
CR13      EQU   13
CR14      EQU   14
CR15      EQU   15
C***********************************************************************
C*             END REGISTER EQUATES                                    *
C***********************************************************************
C*             BEGIN BODY OF PROGRAM CODE                              *
C***********************************************************************
C         STM   R14,R12,12(R13)         STORE CALLING PGM'S GPRS
C         LR    R12,R15                 OUR ENTRY POINT INTO BASE REG
C         LA    R15,SAVEAREA            THIS PGM'S SAVEAREA ADDR IN 15
C         ST    R13,4(R15)              SAVE CALLER'S SAVEAREA ADDR
C         ST    R15,8(R13)              SAVE OUR SAVEAREA ADDR IN CALLER
C         LR    R13,R15                 CURRENT SAVEAREA ADDR IN 13
C         L     R2,0(R1)                ADDRESS OF PASSED VARIABLE
C         SR    R3,R3                   ZERO OUT R3
C         C     R3,0(R2)                IS SUBROUTINE INVOKED WITH 0?
C         BE    SETIT                   YES, SET INTERVAL TIMER
CGETIT    DS    0H                      NO, GET INTERVAL TIMER VALUE
C         TTIMER ,MIC,TVAL2             GET TIME INTO TVAL2
C         L     R4,TVAL2                LOAD, GET READY FOR SHIFT
C         L     R5,TVAL2+4              LOAD, GET READY FOR SHIFT
C         LA    R3,X'0C'                LOAD 12 TO REG 3
C         SRDL  R4,0(R3)                SHIFT RIGHT 12 BITS
C         ST    R5,0(R2)                SAVE ELAPSED TIME FOR CALLER
C         B     GETOUT                  OUR WORK HERE IS DONE KEMOSABE
CSETIT    DS    0H                      HERE IS WHERE WE SET THE I.T.
C         L     R4,HOURS1               PART 1 OF TIME VAL TO R3
C         ST    R4,TVAL                 SET TVAL PART 1
C         L     R5,HOURS2               PART 2 OF TIME VAL TO R3
C         ST    R5,TVAL+4               SET TVAL PART 2
C         LA    R3,X'0C'                LOAD 12 TO REG 3
C         SRDL  R4,0(R3)                SHIFT RIGHT 12 BITS
C         ST    R5,0(R2)                SAVE START TIME FOR CALLER
C         STIMER TASK,MICVL=TVAL        SET TIMER BASED IN MICROSECS
C         SR    R0,R0                   RETURN DUMMY VALUE ON SET
C***********************************************************************
C*                       END OF PROGRAM                                *
C***********************************************************************
CGETOUT   DS    0H
C         L     R13,4(R13)             LOAD CALLER'S SAVEAREA ADDR TO 13
C         L     R14,12(R13)            LOAD RETURN ADDR OF CALLER TO 14
C         LM    R1,R12,24(R13)         RESTORE CALLER'S REGISTERS
C*                                      (BUT DON'T RELOAD R0)
C         SR    R15,R15                SET RETURN CODE TO FORTRAN
C         BR    R14                    RETURN TO CALLER
C***********************************************************************
C*              DATA AREAS REQUIRED FOR OPERATION                      *
C***********************************************************************
CALGN0    DS    0D            DOUBLEWORD ALIGNMENT REQUIRED
CTVAL     DC    D'0'          TIMER VALUE
CTVAL2    DC    D'0'          TIMER VALUE 2 - ("NEW" VALUE)
CHOURS1   DC    X'000007FF'
CHOURS2   DC    X'FFFFF000'
CSAVEAREA DS    18F           18 WORD REGISTER SAVE AREA
C         END
C
C
C
C
C*******************************************************************************
C
C     FOR IBM VM SYSTEMS    (Calls following VMTIME assembly code)
C
C     IMPLICIT INTEGER*4 (I-N)
C     REAL*8 CPUTIM
C     INTEGER*4 VSEC, VUSEC
C
C     CALL VMTIME (VSEC, VUSEC)
C
C     SECOND = VSEC + VUSEC/1.D6
C
C     RETURN
C     END
C
C**********************************************************************
C*                                                                    *
C*   this program is a fortran callable routine that returns a        *
C*   result of the diagnose x'c' instruction.                         *
C*                                                                    *
C*   FORTRAN CALL:  CALL VMTIME(VSEC, VUSEC)                          *
C*                                                                    *
C*   where:                                                           *
C*           vsec   the virtual cpu time, the seconds portion (i*4)   *
C*           vusec  the virtual cpu time, the micro-second portion    *
C*                                                                    *
C**********************************************************************
C
CVMTIME   START 0
CBEGIN    SAVE  (14,12)
C         BALR  3,0
C         USING *,3
C         ST    13,SAVE+4
C         LA    13,SAVE
C
C         LM    4,5,0(1)           LOAD THE FORTRAN PLIST INTO REGS
C
C         LA    2,PTIMER
C
C         DIAG  2,0,X'C'
C
C         L     6,VCPU
C         L     7,VCPU+4
C         D     6,=F'1000000'      SEPARATE SEC AND USEC
C         ST    7,0(4)             STORE THE SECOND PORTION
C         ST    6,0(5)             STORE THE MICROSECOND PORTION
C
C
C         L     13,SAVE+4
C         RETURN (14,12),RC=0
C
C
CPTIMER   DS    0D
CDATE     DS    D                  CURRENT DATE
CTIME     DS    D                  CURRENT TIME
CVCPU     DS    D                  VIRTUAL CPU TIME
CTCPU     DS    D                  TOTAL CPUTIME
CSAVE     DS    18F
C
C         LTORG
C         END
C     RETURN
C     END
c
c
c
C
C***********************************************************************
      SUBROUTINE  SENSIT( iou, RATES,WG,IQ,SUMW,  MAP,TV,TV1,TV2,TV3,n)
C***********************************************************************
C                                                                      *
C     SENSIT  - Sensitivity Of Harmonic Mean Rate(Mflops) 49 Workloads *
C                                                                      *
C     iou     - input scalar,  i/o unit number                         *
C     RATES   - input array ,  execution rates (Mflops)                *
C     WG      - input array ,  weights paired with RATES               *
C     IQ      - input array ,  1 or 2 quartiles specifier              *
C     SUMW    - input array ,  workload fractions.                     *
C                                                                      *
C     MAP,TV,TV1,TV2,TV3    -  output temporary arrays                 *
C     n       - input scalar,  number of rates, etc.                   *
C                                                                      *
C***********************************************************************
C
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
c In
      DIMENSION  RATES(n), WG(n), IQ(7), SUMW(7)
c Temp
      DIMENSION  MAP(n), TV(n), TV1(n), TV2(n), TV3(n)
      DIMENSION  NR1(10), NR2(10), STAT2(20)
 
c     DIMENSION  TAG(4)                                                  f66
      CHARACTER*8  TAG(4)                                                f77
C
      DATA  ( TAG(i), i= 1,4)
     . /'1st QT: ',  '2nd QT: ',  '3rd QT: ',  '4th QT: '/               f77
c    ./8H1st QT:  , 8H2nd QT:  , 8H3rd QT:  , 8H4th QT:  /               f66
C
      CALL TRACKS('SENSIT  ')
C
C                 Compress valid data sets RATES,  mapping on MAP.
 
              bl= 1.0D-5
              bu= 1.0D+5
            CALL  VALID( TV1,MAP,neff,  bl, RATES, bu, n)
 
        DO  1  k= 1,neff
          TV3(k)=    WG( MAP(k))
    1  continue
 
 
C                 Compress valid data sets WG,  mapping on MAP.
 
            CALL  VALID( TV3,MAP,meff,  bl, TV3, bu, neff)
 
        DO  3  k= 1,meff
           TV(k)=TV1( MAP(k))
    3  continue
C
C                 Sort selected rates into descending order
 
            CALL  SORDID( MAP,TV2,   TV,meff,2)
 
C
C
       CALL PAGE( iou)
          WRITE ( iou,7001)
C
 7001 FORMAT(/)
 7301 FORMAT(9X,31H           SENSITIVITY ANALYSIS )
 7302 FORMAT(9X,52HThe sensitivity of the harmonic mean rate (Mflops)  )
 7303 FORMAT(9X,52Hto various weightings is shown in the table below.  )
 7304 FORMAT(9X,52HSeven work distributions are generated by assigning )
 7305 FORMAT(9X,52Htwo distinct weights to ranked kernels by quartiles.)
 7306 FORMAT(9X,52HForty nine possible cpu workloads are then evaluated)
 7307 FORMAT(9X,52Husing seven sets of values for the total weights:   )
 7341 FORMAT(3X,A ,6X,43HO      O      O      O      O      X      X)    f77
 7342 FORMAT(3X,A ,6X,43HO      O      O      X      X      X      O)    f77
 7343 FORMAT(3X,A ,6X,43HO      X      X      X      O      O      O)    f77
 7344 FORMAT(3X,A ,6X,43HX      X      O      O      O      O      O)    f77
c7341 FORMAT(3X,A7,6X,43HO      O      O      O      O      X      X)    f66
c7342 FORMAT(3X,A7,6X,43HO      O      O      X      X      X      O)    f66
c7343 FORMAT(3X,A7,6X,43HO      X      X      X      O      O      O)    f66
c7344 FORMAT(3X,A7,6X,43HX      X      O      O      O      O      O)    f66
 7346 FORMAT(13X,  48H------ ------ ------ ------ ------ ------ ------)
 7348 FORMAT(3X,5HTotal,/,3X,7HWeights,20X,11HNet Mflops:,/,4X,6HX    O)
 7349 FORMAT(2X,9H---- ---- )
 7220 FORMAT(/,1X,2F5.2,1X,7F7.2)
C
          WRITE ( iou,7001)
          WRITE ( iou,7001)
          WRITE ( iou,7301)
          WRITE ( iou,7001)
          WRITE ( iou,7302)
          WRITE ( iou,7303)
          WRITE ( iou,7304)
          WRITE ( iou,7305)
          WRITE ( iou,7306)
          WRITE ( iou,7307)
          WRITE ( iou,7001)
          WRITE ( iou,7346)
          WRITE ( iou,7341)   TAG(1)
          WRITE ( iou,7342)   TAG(2)
          WRITE ( iou,7343)   TAG(3)
          WRITE ( iou,7344)   TAG(4)
          WRITE ( iou,7346)
          WRITE ( iou,7348)
          WRITE ( iou,7349)
C
            IF( meff .LE. 0 )  GO TO 73
          fuzz= 1.0e-9
             r= meff
            mq= (meff+3)/4
             q= mq
             j= 1
      DO 21  i= 8,2,-2
      NR1(i  )= j
      NR1(i+1)= j
      NR2(i  )= j + mq + mq - 1
      NR2(i+1)= j + mq - 1
             j= j + mq
   21  continue
C
       DO 29 j= 1,7
          sumo= 1.0 - SUMW(j)
       DO 27 i= 1,7
             p= IQ(i)*q
            xt= SUMW(j)/(p + fuzz)
            ot= sumo   /(r - p + fuzz)
       DO 23 k= 1,meff
        TV3(k)= ot
   23  continue
            k1= NR1(i+2)
            k2= NR2(i+2)
       DO 25 k= k1,k2
        TV3(k)= xt
   25  continue
          CALL  STATW( STAT2,TV,MAP, TV2,TV3,meff)
        TV1(i)= STAT2(5)
   27  continue
        WRITE ( iou,7220) SUMW(j), sumo, ( TV1(k), k=1,7)
   29  continue
C
           WRITE ( iou,7349)
           WRITE ( iou,7346)
C
C
   73 CONTINUE
      CALL TRACKX
      RETURN
C
      END
C***********************************************
      SUBROUTINE  SIGNAL( V, SCALE,BIAS, n)
C***********************************************
C
C    SIGNAL GENERATES VERY FRIENDLY FLOATING-POINT NUMBERS NEAR 1.0
C                     WHEN SCALE= 1.0 AND BIAS= 0.
C
C     V      - result array,  floating-point test data
C     SCALE  - input scalar,  scales magnitude of results
C     BIAS   - input scalar,  offsets magnitude of results
C     n      - input integer, number of results in V.
C
C***********************************************
      DOUBLE PRECISION  V, SCALE, BIAS
cIBM  REAL*8            V, SCALE, BIAS
c
c     Use the following Double Precision declaration to improve Real*4 tests.
c     Use the following Real*16          declaration to improve Real*8 tests.
c
      DOUBLE PRECISION  SCALED,BIASED,FUZZ,BUZZ,FIZZ,ONE
cIBM  REAL*16           SCALED,BIASED,FUZZ,BUZZ,FIZZ,ONE
      DIMENSION  V(n)
C
      CALL TRACKS('SIGNAL  ')
C
        SCALED= SCALE
        BIASED= BIAS
C
        SCALED= 10.00D0
        SCALED=  1.00D0/SCALED
        BIASED=  0.00D0
C
C         FUZZ= 1.234500D-9
          FUZZ= 1.234500D-3
          BUZZ= 1.000D0  + FUZZ
          FIZZ= 1.100D0  * FUZZ
           ONE= 1.000D0
C
        DO 1 k= 1,n
          BUZZ= (ONE - FUZZ)*BUZZ +FUZZ
          FUZZ= -FUZZ
c         V(k)=((BUZZ- FIZZ) -BIASED)*SCALED
          V(k)= (BUZZ- FIZZ)*SCALED
    1 CONTINUE
C
      CALL TRACKX
      RETURN
      END
C
C
C***********************************************************************
      SUBROUTINE SIMD( HM,  iou,RATES,WG,FR,m,  MAP,TV1,TV2,TV3,n)
C***********************************************************************
C                                                                      *
C     SIMD  - Sensitivity Of Harmonic Mean Rate(Mflops) SISD/SIMD Model*
C                                                                      *
C     iou     - input scalar,  i/o unit number                         *
C     RATES   - input array ,  execution rates (Mflops)                *
C     WG      - input array ,  weights paired with RATES               *
C     FR      - input array ,  fractions of flops executed SIMD        *
C     m       - input scalar,  number of fractions                     *
C                                                                      *
C     MAP,TV,TV1,TV2,TV3    -  output temporary arrays                 *
C     n       - input scalar,  number of rates, etc.                   *
C                                                                      *
C***********************************************************************
C
C
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
c SENSITIVITY OF NET MFLOPS RATE TO USE OF OPTIMAL FORTRAN CODE(SISD/SIMD MODEL)
c Out
      DIMENSION  HM(m)
c In
      DIMENSION  FR(m), RATES(n), WG(n)
c Temp
      DIMENSION  MAP(n), TV1(n), TV2(n), TV3(n), STAT2(20)
C
      CALL TRACKS('SIMD    ')
 
C                 Compress valid data sets RATES,  mapping on MAP.
 
              bl= 1.0D-5
              bu= 1.0D+5
            CALL  VALID( TV1,MAP,neff,  bl, RATES, bu, n)
 
        DO  1  k= 1,neff
          TV3(k)=    WG( MAP(k))
    1  continue
 
 
C                 Compress valid data sets WG,  mapping on MAP.
 
            CALL  VALID( TV3,MAP,meff,  bl, TV3, bu, neff)
 
        DO  3  k= 1,meff
          TV2(k)= TV1( MAP(k))
    3  continue
 
C                 Sort RATES,WT into descending order.
 
            CALL  STATW( STAT2,TV1,MAP, TV2, TV3, meff)
             med= meff + 1 - INT(STAT2(8))
              lh= meff + 1 - med
 
        DO  5  k= 1,meff
          TV2(k)= TV3( MAP(k))
    5  continue
 
 
C                 Estimate vector rate= HMean of top LFK quartile.
 
              nq= meff/4
            CALL  STATW( STAT2,TV3,MAP, TV1,TV2,nq)
             vmf= STAT2(5)
 
C                 Estimate scalar rate= HMean of lowest two LFK quartiles.
 
            CALL  STATW( STAT2,TV3,MAP, TV1(med),TV2(med),lh)
             smf= STAT2(5)
            fuzz= 1.0e-9
 
               g= 1.0 -   smf/( vmf + fuzz)
           HM(1)= smf
 
          DO 7 k= 2,m
           HM(k)=   smf/( 1.0 - FR(k)*g + fuzz)
   7      continue
C
      IF( iou .GT. 0)  THEN
C
          WRITE ( iou,7001)
          WRITE ( iou,7001)
          WRITE ( iou,7001)
          WRITE ( iou,7101)
          WRITE ( iou,7102) ( HM(k), k= 1,9)
          WRITE ( iou,7102) ( FR(k), k= 1,9)
          WRITE ( iou,7103)
          WRITE ( iou,7001)
 7001 FORMAT(/)
 7101 FORMAT(' SENSITIVITY OF NET MFLOPS RATE TO USE OF OPTIMAL FORTRAN
     1CODE(SISD/SIMD MODEL)' )
 7102 FORMAT(/,1X,5F7.2,4F8.2)
 7103 FORMAT(3x,52H Fraction Of Operations Run At Optimal Fortran Rates)
C
      ENDIF
C
      CALL TRACKX
      RETURN
C
      END
C
C
C***********************************************
      SUBROUTINE SIZES(i)
C***********************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
C            SIZES      test and set the loop controls before each kernel test
C
C     i    :=  kernel number
C
C     mk   :=  number of kernels to test
C     Nruns:=  number of timed runs of complete test.
C     tic  :=  cpu clock resolution or minimum time in seconds.
C     Loop :=  multiple pass control to execute kernel long enough to time.
C     n    :=  DO loop control for each kernel.
C     ******************************************************************
C
C
C/      PARAMETER( l1= 1001, l2=  101, l1d= 2*1001 )
C/      PARAMETER( l13=  64, l13h= l13/2, l213= l13+l13h, l813= 8*l13 )
C/      PARAMETER( l14=2048, l16=  75, l416= 4*l16 , l21= 25 )
C
C/      PARAMETER( l1=   27, l2=   15, l1d= 2*1001 )
C/      PARAMETER( l13= 8, l13h= 8/2, l213= 8+4, l813= 8*8 )
C/      PARAMETER( l14=  16, l16= 15, l416= 4*15 , l21= 15)
C
C/      PARAMETER( l1=   1001, l2=   101, l1d= 2*1001 )
C/      PARAMETER( l13= 64, l13h= 64/2, l213= 64+32, l813= 8*64 )
C/      PARAMETER( l14= 2048, l16= 75, l416= 4*75 , l21= 25)
C
C/      PARAMETER( kn= 47, kn2= 95, np= 3, ls= 3*47, krs= 24)
C
C/      PARAMETER( NNI=  2*l1 +2*l213 +l416 )
C/      PARAMETER( NN1= 16*l1 +13*l2 +2*l416 + l14 )
C/      PARAMETER( NN2= 4*l813 + 3*l21*l2 +121*l2 +3*l13*l13 )
C/      PARAMETER( Nl1= 19*l1, Nl2= 131*l2 +3*l21*l2 )
C/      PARAMETER( Nl13= 3*l13*l13 +34*l13 +32)
C
C/      PARAMETER( nk= 47, nl= 3, nr= 8 )
C
      COMMON /ALPHA/ mk,ik,ml,il,Nruns,jr, NPFS(8,3,47)
      COMMON /BETA / tic, TIMES(8,3,47), SEE(5,3,8,3),
     1              TERRS(8,3,47), CSUMS(8,3,47),
     2              FOPN(8,3,47), DOS(8,3,47)
C
C
      COMMON /SPACES/ ion,j5,k2,k3,Loop,m,kr,it,n13h,ibuf,
     1                n,n1,n2,n13,n213,n813,n14,n16,n416,n21,nt1,nt2
C
      COMMON /SPACER/ A11,A12,A13,A21,A22,A23,A31,A32,A33,
     2                AR,BR,C0,CR,DI,DK,
     3  DM22,DM23,DM24,DM25,DM26,DM27,DM28,DN,E3,E6,EXPMAX,FLX,
     4  Q,QA,R,RI,S,SCALE,SIG,STB5,T,XNC,XNEI,XNM
C
      COMMON /SPACE0/ TIME(47), CSUM(47), WW(47), WT(47), ticks,
     1                FR(9), TERR1(47), SUMW(7), START,
     2              SKALE(47), BIAS(47), WS(95), TOTAL(47), FLOPN(47),
     3                IQ(7), NPF, NPFS1(47)
C
      COMMON /SPACEI/ WTP(3), MUL(3), ISPAN(47,3), IPASS(47,3)
C
C     ******************************************************************
C
      CALL TRACKS('SIZES   ')
C
      nif= 0
C
c                        Set  mk .LE. 47  number of kernels to test.
c            mk= 14
             mk= 24
             ml= 3
c                        Set  Nruns .LT. 8  number of timed runs of KERNEL test
c                        Set  Nruns= 1   to REDUCE RUN TIME for debug runs.
          Nruns= 1
c                        Set  Nruns= 7   for Standard BENCHMARK Test.
          Nruns= 7
             IF( Nruns.GT. 7) Nruns= 7
c                        Set tic to the minimum Cpu clock resolution time(Sec.)
c           tic= 1./60.
            tic= 1.0e-2
C
      IF( i.EQ.-1)  GO TO 73
C
C
C     Domain tests follow to detect overstoring of controls for array opns.
C
      nif= 1
      iup= 999000
      IF( iup.LT.65000 ) iup= 65000
      IF( i.LT.0 .OR.  (i-1).GT.  24)      GO TO 911
      IF( n.LT.0 .OR.  n.GT.   1001)          GO TO 911
      IF(Loop.LT.0 .OR. Loop.GT.iup)        GO TO 911
C
      nif= 2
      IF(  il.LT.1 .OR. il.GT.3 )  GO TO 911
                 n= ISPAN(i,il)
      Loop        = IPASS(i,il) * MUL(il)
C
c
C     If clock resolution test fails, you must increase Loop= 40*Loop etc.
C     Standard Checksums in REPORT were obtained with   Loop= 10*Loop.
C     Verify correct execution using these checksums then, if timing
C     errors are too large, increase the value of Loop:
c
c     Loop= 100*Loop
      Loop=  40*Loop
c     Loop=  10*Loop
c
Computers with high resolution clocks tic= O(microsec.) should use Loop= 1
C
C     Loop= 1
C
      IF( Loop.LT. 1)   Loop= 1
      nif= 3
      IF( n.LT.0 .OR.  n.GT.   1001)  GO TO 911
      IF(Loop.LT.0 .OR. Loop.GT.iup)  GO TO 911
      n1  = 1001
      n2  = 101
      n13 = 64
      n13h= 32
      n213= 96
      n813= 512
      n14 = 2048
      n16 = 75
      n416= 300
      n21 = 25
C
      nt1= 16*1001 +13*101 +2*300 + 2048
      nt2= 4*512 + 3*25*101 +121*101 +3*64*64
C
   73 CONTINUE
      CALL TRACKX
      RETURN
C
C
  911 io= ABS( ion)
      IF( io.LE.0 .OR. io.GT.100 ) io=6
      WRITE( io,913) i, nif, n, Loop, il
  913 FORMAT(1H1,///,37H FATAL OVERSTORE/ DATA LOSS.  TEST=  ,6I6)
      STOP
C
      END
C***********************************************
      SUBROUTINE SORDID( I,W, V,n,KIND)
C***********************************************
C                    QUICK AND DIRTY PORTABLE SORT.
C
C                I - RESULT INDEX-LIST. MAPS V TO SORTED W.
C                W - RESULT ARRAY, SORTED V.
C
C                V - INPUT  ARRAY SORTED IN PLACE.
C                n - INPUT  NUMBER OF ELEMENTS IN V
C             KIND - SORT ORDER:   = 1  ASCENDING MAGNITUDE
C                                  = 2 DESCENDING MAGNITUDE
C
C***********************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
      DIMENSION  I(n), W(n), V(n)
C
      CALL TRACKS('SORDID  ')
C
      DO  1  k= 1,n
          W(k)= V(k)
    1     I(k)= k
C
      IF( KIND.EQ.1)  THEN
C
          DO  3  j= 1,n-1
                 m= j
          DO  2  k= j+1,n
                IF( W(k).LT.W(m)) m= k
    2     CONTINUE
                 X= W(j)
                 k= I(j)
              W(j)= W(m)
              I(j)= I(m)
              W(m)= X
              I(m)= k
    3     CONTINUE
C
C
      ELSE
C
    4     DO  6  j= 1,n-1
                 m= j
          DO  5  k= j+1,n
                IF( W(k).GT.W(m)) m= k
    5     CONTINUE
                 X= W(j)
                 k= I(j)
              W(j)= W(m)
              I(j)= I(m)
              W(m)= X
              I(m)= k
    6     CONTINUE
      ENDIF
C
      IF( n.GT.0 )  THEN
          CALL TRAP( I, 8H SORDID  , 1, n,n)
      ENDIF
C
      CALL TRACKX
      RETURN
      END
C
C***********************************************
      SUBROUTINE  SPACE
C***********************************************
C
C            SPACE      sets memory pointers for array variables.  optional.
C
C     Subroutine Space dynamically allocates physical memory space
C     for the array variables in KERNEL by setting pointer values.
C     The POINTER declaration has been defined in the IBM PL1 language
C     and defined as a Fortran extension in Livermore and CRAY compilers.
C
C     In general, large FORTRAN simulation programs use a memory
C     manager to dynamically allocate arrays to conserve high speed
C     physical memory and thus avoid slow disk references (page faults).
C
C     It is sufficient for our purposes to trivially set the values
C     of pointers to the location of static arrays used in common.
C     The efficiency of pointered (indirect) computation should be measured
C     if available.
C
C***********************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
C
C/      PARAMETER( l1=   1001, l2=   101, l1d= 2*1001 )
C/      PARAMETER( l13= 64, l13h= 64/2, l213= 64+32, l813= 8*64 )
C/      PARAMETER( l14= 2048, l16= 75, l416= 4*75 , l21= 25)
C
      INTEGER    E,F,ZONE
      COMMON /ISPACE/ E(96), F(96),
     1  IX(1001), IR(1001), ZONE(300)
C
      COMMON /SPACE1/ U(1001), V(1001), W(1001),
     1  X(1001), Y(1001), Z(1001), G(1001),
     2  DU1(101), DU2(101), DU3(101), GRD(1001), DEX(1001),
     3  XI(1001), EX(1001), EX1(1001), DEX1(1001),
     4  VX(1001), XX(1001), RX(1001), RH(2048),
     5  VSP(101), VSTP(101), VXNE(101), VXND(101),
     6  VE3(101), VLR(101), VLIN(101), B5(101),
     7  PLAN(300), D(300), SA(101), SB(101)
C
      COMMON /SPACE2/ P(4,512), PX(25,101), CX(25,101),
     1  VY(101,25), VH(101,7), VF(101,7), VG(101,7), VS(101,7),
     2  ZA(101,7)  , ZP(101,7), ZQ(101,7), ZR(101,7), ZM(101,7),
     3  ZB(101,7)  , ZU(101,7), ZV(101,7), ZZ(101,7),
     4  B(64,64), C(64,64), H(64,64),
     5  U1(5,101,2),  U2(5,101,2),  U3(5,101,2)
C
C     ******************************************************************
C
C//      COMMON /POINT/ ME,MF,MU,MV,MW,MX,MY,MZ,MG,MDU1,MDU2,MDU3,MGRD,
C//     1  MDEX,MIX,MXI,MEX,MEX1,MDEX1,MVX,MXX,MIR,MRX,MRH,MVSP,MVSTP,
C//     2  MVXNE,MVXND,MVE3,MVLR,MVLIN,MB5,MPLAN,MZONE,MD,MSA,MSB,
C//     3  MP,MPX,MCX,MVY,MVH,MVF,MVG,MVS,MZA,MZP,MZQ,MZR,MZM,MZB,MZU,
C//     4  MZV,MZZ,MB,MC,MH,MU1,MU2,MU3
C//C
C//CLLL. LOC(X) =.LOC.X
C//C
           CALL TRACKS('SPACE   ')
C//      ME     = LOC( E )
C//      MF     = LOC( F )
C//      MU     = LOC( U )
C//      MV     = LOC( V )
C//      MW     = LOC( W )
C//      MX     = LOC( X )
C//      MY     = LOC( Y )
C//      MZ     = LOC( Z )
C//      MG     = LOC( G )
C//      MDU1   = LOC( DU1 )
C//      MDU2   = LOC( DU2 )
C//      MDU3   = LOC( DU3 )
C//      MGRD   = LOC( GRD )
C//      MDEX   = LOC( DEX )
C//      MIX    = LOC( IX )
C//      MXI    = LOC( XI )
C//      MEX    = LOC( EX )
C//      MEX1   = LOC( EX1 )
C//      MDEX1  = LOC( DEX1 )
C//      MVX    = LOC( VX )
C//      MXX    = LOC( XX )
C//      MIR    = LOC( IR )
C//      MRX    = LOC( RX )
C//      MRH    = LOC( RH )
C//      MVSP   = LOC( VSP )
C//      MVSTP  = LOC( VSTP )
C//      MVXNE  = LOC( VXNE )
C//      MVXND  = LOC( VXND )
C//      MVE3   = LOC( VE3 )
C//      MVLR   = LOC( VLR )
C//      MVLIN  = LOC( VLIN )
C//      MB5    = LOC( B5 )
C//      MPLAN  = LOC( PLAN )
C//      MZONE  = LOC( ZONE )
C//      MD     = LOC( D )
C//      MSA    = LOC( SA )
C//      MSB    = LOC( SB )
C//      MP     = LOC( P )
C//      MPX    = LOC( PX )
C//      MCX    = LOC( CX )
C//      MVY    = LOC( VY )
C//      MVH    = LOC( VH )
C//      MVF    = LOC( VF )
C//      MVG    = LOC( VG )
C//      MVS    = LOC( VS )
C//      MZA    = LOC( ZA )
C//      MZP    = LOC( ZP )
C//      MZQ    = LOC( ZQ )
C//      MZR    = LOC( ZR )
C//      MZM    = LOC( ZM )
C//      MZB    = LOC( ZB )
C//      MZU    = LOC( ZU )
C//      MZV    = LOC( ZV )
C//      MZZ    = LOC( ZZ )
C//      MB     = LOC( B )
C//      MC     = LOC( C )
C//      MH     = LOC( H )
C//      MU1    = LOC( U1 )
C//      MU2    = LOC( U2 )
C//      MU3    = LOC( U3 )
C
      CALL TRACKX
      RETURN
      END
C***********************************************
      SUBROUTINE STATS( STAT, X,n)
C***********************************************
C
C     UNWEIGHTED STATISTICS: MEAN, STADEV, MIN, MAX, HARMONIC MEAN.
C
C     STAT(1)= THE MEAN OF X.
C     STAT(2)= THE STANDARD DEVIATION OF THE MEAN OF X.
C     STAT(3)= THE MINIMUM OF X.
C     STAT(4)= THE MAXIMUM OF X.
C     STAT(5)= THE HARMONIC MEAN
C     X       IS THE ARRAY  OF INPUT VALUES.
C     n       IS THE NUMBER OF INPUT VALUES IN X.
C
C***********************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
      DIMENSION X(n), STAT(9)
cLLL. OPTIMIZE LEVEL G
C
      CALL TRACKS('STATS   ')
C
      DO 10   k= 1,9
   10 STAT(k)= 0.0
C
      IF(n.LE.0)  GO TO 73
C                             CALCULATE MEAN OF X.
      S= 0.0
      DO 1 k= 1,n
    1 S= S + X(k)
      A= S/n
      STAT(1)= A
C                             CALCULATE STANDARD DEVIATION OF X.
      D= 0.0
      DO 2 k= 1,n
    2 D= D + (X(k)-A)**2
      D= D/n
      STAT(2)= SQRT(D)
C                             CALCULATE MINIMUM OF X.
      U= X(1)
      DO 3 k= 2,n
    3 U= MIN(U,X(k))
      STAT(3)= U
C                             CALCULATE MAXIMUM OF X.
      V= X(1)
      DO 4 k= 2,n
    4 V= MAX(V,X(k))
      STAT(4)= V
C                             CALCULATE HARMONIC MEAN OF X.
      H= 0.0
      DO 5 k= 1,n
          IF( X(k).NE.0.0) H= H + 1.0/X(k)
    5 CONTINUE
          IF( H.NE.0.0) H= DBLE(n)/H
      STAT(5)= H
C
   73 CONTINUE
      CALL TRACKX
      RETURN
      END
C***********************************************
      SUBROUTINE STATW( STAT,OX,IX, X,W,n)
C***********************************************
C
C     WEIGHTED STATISTICS: MEAN, STADEV, MIN, MAX, HARMONIC MEAN, MEDIAN.
C
C     STAT( 1)=  THE MEAN OF X.
C     STAT( 2)=  THE STANDARD DEVIATION OF THE MEAN OF X.
C     STAT( 3)=  THE MINIMUM OF X.
C     STAT( 4)=  THE MAXIMUM OF X.
C     STAT( 5)=  THE HARMONIC MEAN
C     STAT( 6)=  THE TOTAL WEIGHT.
C     STAT( 7)=  THE MEDIAN.
C     STAT( 8)=  THE MEDIAN INDEX, ASCENDING.
C     STAT( 9)=  THE ROBUST MEDIAN ABSOLUTE DEVIATION.
C     STAT(10)=  THE GEOMETRIC MEAN
C     STAT(11)=  THE MOMENTAL SKEWNESS
C     STAT(12)=  THE KURTOSIS
C     STAT(13)=  THE LOWER QUARTILE BOUND Q1/Q2 VALUE
C     STAT(14)=  THE UPPER QUARTILE BOUND Q3/Q4 VALUE
C
C     OX      IS THE ARRAY  OF ORDERED (DECENDING) Xs.
C     IX      IS THE ARRAY  OF INDEX LIST MAPS X TO OX.
C
C     X       IS THE ARRAY  OF INPUT VALUES.
C     W       IS THE ARRAY  OF INPUT WEIGHTS.
C     n       IS THE NUMBER OF INPUT VALUES IN X.
C
C***********************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
      DIMENSION STAT(14), OX(n), IX(n), X(n), W(n)
cLLL. OPTIMIZE LEVEL G
C
      CALL TRACKS('STATW   ')
C
      DO 50   k= 1,14
   50 STAT(k)= 0.0
C
      IF( n.LE.0 )  GO TO 73
C
      IF( n.EQ.1 )  THEN
          STAT( 1)= X(1)
          STAT( 3)= X(1)
          STAT( 4)= X(1)
          STAT( 5)= X(1)
          STAT( 6)= W(1)
          STAT( 7)= X(1)
          STAT( 8)= 1.0
          STAT(10)= X(1)
          GO TO 73
      ENDIF
C
C
C                             CALCULATE MEAN OF X.
      A= 0.0
      S= 0.0
      T= 0.0
C
      DO 1 k= 1,n
      S= S + W(k)*X(k)
    1 T= T + W(k)
          IF( T.NE.0.0) A= S/T
      STAT(1)= A
C                             CALCULATE STANDARD DEVIATION OF X.
      D= 0.0
      E= 0.0
      F= 0.0
      Q= 0.0
      U= 0.0
C
      DO 2 k= 1,n
      B= W(k) *( X(k) -A)**2
      D= D + B
      E= E + B*( X(k) -A)
    2 F= F + B*( X(k) -A)**2
          IF( T.NE.0.0) Q= 1.0/T
                        D= D*Q
                        E= E*Q
                        F= F*Q
          IF( D.GE.0.0) U= SQRT(D)
      STAT(2)= U
C                             CALCULATE MINIMUM OF X.
      U= X(1)
      DO 3 k= 2,n
    3 U= MIN(U,X(k))
      STAT(3)= U
C                             CALCULATE MAXIMUM OF X.
      V= X(1)
      DO 4 k= 2,n
    4 V= MAX(V,X(k))
      STAT(4)= V
C                             CALCULATE HARMONIC MEAN OF X.
      H= 0.0
      DO 5 k= 1,n
          IF( X(k).NE.0.0) H= H + W(k)/X(k)
    5 CONTINUE
          IF( H.NE.0.0) H= T/H
      STAT(5)= H
      STAT(6)= T
C                             CALCULATE WEIGHTED MEDIAN
      CALL SORDID( IX, OX, X, n, 1)
C
           ew= 0.0
      DO 7  k= 2,n
           IF( W(1) .NE. W(k))  GO TO 75
    7 continue
           ew= 1.0
   75 continue
C
        qt= 0.500D0
      CALL  TILE( STAT( 7), STAT(8), OX,IX,W,ew,T, qt,n)
C
        qt= 0.250D0
      CALL  TILE( STAT(13),  stin13, OX,IX,W,ew,T, qt,n)
C
        qt= 0.750D0
      CALL  TILE( STAT(14),  stin14, OX,IX,W,ew,T, qt,n)
C
C
C                           CALCULATE ROBUST MEDIAN ABSOLUTE DEVIATION (MAD)
      DO 90 k= 1,n
   90   OX(k)= ABS( X(k) - STAT(7))
C
      CALL SORDID( IX, OX, OX, n, 1)
C
        qt= 0.700D0
      CALL  TILE( STAT( 9),  stin09, OX,IX,W,ew,T, qt,n)
C
C                             CALCULATE GEOMETRIC MEAN
            R= 0.0
      DO 10 k= 1,n
           IF( X(k).LE. 0.)  GO TO 10
            R= R + W(k) *LOG10( X(k))
   10 CONTINUE
             U= R*Q
             G= 10.0
            IF( U.LT. 0.)  G= 0.1
        POWTEN= 50.0
            IF( ABS(U) .GT. POWTEN)  U= SIGN( POWTEN, U)
      STAT(10)=  G** ABS(U)
C
C                             CALCULATE MOMENTAL SKEWNESS
             G= 0.0
           DXD= D*D
            IF( DXD.NE.0.0) G= 1.0/(DXD)
      STAT(11)= 0.5*E*G*STAT(2)
C
C                             CALCULATE KURTOSIS
      STAT(12)= 0.5*( F*G -3.0)
C
C                             CALCULATE DESCENDING ORDERED X.
      CALL SORDID( IX, OX, X, n, 2)
C
   73 CONTINUE
      CALL TRACKX
      RETURN
      END
C
C***********************************************
      FUNCTION SUMO( V,n)
C***********************************************
C
C     CHECK-SUM WITH ORDINAL DEPENDENCY.
C
C     V   - input array,   floating-point numbers
C     n   - input integer, number of elements in V.
C
C***********************************************
      DOUBLE PRECISION  SUMO, V
cIBM  REAL*8            SUMO, V
c
c     Use the following Double Precision declaration to improve Real*4 tests.
c     Use the following Real*16          declaration to improve Real*8 tests.
c
      DOUBLE PRECISION  S
cIBM  REAL*16           S
C
      DIMENSION  V(n)
C
      CALL TRACKS('SUMO    ')
           S= 0.00D0
C
      DO 1 k= 1,n
    1      S= S + DBLE(k)*V(k)
       SUMO = S
      CALL TRACKX
      RETURN
      END
C
C***********************************************
      SUBROUTINE  SUPPLY(i)
C***********************************************
C
C            SUPPLY     initializes common blocks containing type real arrays.
C
C     i    :=  kernel number
C
C****************************************************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
C/      PARAMETER( l1=   1001, l2=   101, l1d= 2*1001 )
C/      PARAMETER( l13= 64, l13h= 64/2, l213= 64+32, l813= 8*64 )
C/      PARAMETER( l14= 2048, l16= 75, l416= 4*75 , l21= 25)
C
C/      PARAMETER( kn= 47, kn2= 95, np= 3, ls= 3*47, krs= 24)
C/C
C/C/      PARAMETER( NN0= 39 )
C/C/      PARAMETER( NNI=  2*l1 +2*l213 +l416 )
C/C/      PARAMETER( NN1= 16*l1 +13*l2 +2*l416 + l14 )
C/C/      PARAMETER( NN2= 4*512 + 3*25*101 +121*101 +3*64*64 )
C
      COMMON /SPACES/ ion,j5,k2,k3,Loop,m,kr,it,n13h,ibuf,
     1                n,n1,n2,n13,n213,n813,n14,n16,n416,n21,nt1,nt2
C
      COMMON /SPACE0/ TIME(47), CSUM(47), WW(47), WT(47), ticks,
     1                FR(9), TERR1(47), SUMW(7), START,
     2              SKALE(47), BIAS(47), WS(95), TOTAL(47), FLOPN(47),
     3                IQ(7), NPF, NPFS1(47)
C
C/      COMMON /SPACE1/ U(NN1)
C/      COMMON /SPACE2/ P(NN2)
C/      COMMON /SPACER/ A11(NN0)
C/C
        COMMON /SPACE1/ U(19977)
        COMMON /SPACE2/ P(34132)
        COMMON /SPACER/ A11(39)
C
C
C***********************************************************************
C           Method 1:  Least space and most cpu time (D.P. SIGNAL arith)
C***********************************************************************
C
Csmall      CALL TRACKS('SUPPLY  ')
Csmall      IP1= i+1
Csmall      nt0= 39
CsmallC
Csmall      CALL SIGNAL(  U, SKALE(IP1), BIAS(IP1), nt1)
Csmall      CALL SIGNAL(  P, SKALE(IP1), BIAS(IP1), nt2)
Csmall      CALL SIGNAL(A11, SKALE(IP1), BIAS(IP1), nt0)
Csmall      CALL TRACKX
Csmall      RETURN
C
C***********************************************************************
C           Method 2:  Double space and least cpu time
C***********************************************************************
C
        COMMON /BASE1/ BUFU(19977)
        COMMON /BASE2/ BUFP(34132)
        COMMON /BASER/ BUFA(39)
C
      CALL TRACKS('SUPPLY  ')
C
      IP1= i+1
      nt0= 39
C               Execute SIGNAL calls only once; re-use generated data.
          ibuf= ibuf+1
      IF( ibuf.EQ. 1) THEN
          CALL SIGNAL(  BUFU, SKALE(IP1), BIAS(IP1), nt1)
          CALL SIGNAL(  BUFP, SKALE(IP1), BIAS(IP1), nt2)
          CALL SIGNAL(  BUFA, SKALE(IP1), BIAS(IP1), nt0)
      ENDIF
C
      DO 1 k= 1,nt0
    1 A11(k)= BUFA(k)
      DO 2 k= 1,nt1
    2   U(k)= BUFU(k)
      DO 3 k= 1,nt2
    3   P(k)= BUFP(k)
C
      CALL TRACKX
      RETURN
      END
C
C***********************************************************************
      SUBROUTINE  TALLY( iou, mode )
C***********************************************************************
C                                                                      *
C    TALLY      computes average and minimum Cpu timings and variances.*
C                                                                      *
C               iou -  i/o unit number                                 *
C                                                                      *
C              mode -  = 1 selects average run time: Preferred mode.   *
C                      = 2 selects minimum run time: Less accurate mode*
C                                                                      *
C***********************************************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
      DOUBLE PRECISION  cs
C
C/      PARAMETER( nk= 47, nl= 3, nr= 8 )
C
      COMMON /ALPHA/ mk,ik,ml,il,Nruns,jr, NPFS(8,3,47)
      COMMON /BETA / tic, TIMES(8,3,47), SEE(5,3,8,3),
     1              TERRS(8,3,47), CSUMS(8,3,47),
     2              FOPN(8,3,47), DOS(8,3,47)
C
      COMMON /SPACE0/ TIME(47), CSUM(47), WW(47), WT(47), ticks,
     1                FR(9), TERR1(47), SUMW(7), START,
     2              SKALE(47), BIAS(47), WS(95), TOTAL(47), FLOPN(47),
     3                IQ(7), NPF, NPFS1(47)
C
      DIMENSION  S1(12), S2(12), S3(12), S4(12)
      DIMENSION  T1(47), T4(47)
C
      CALL TRACKS('TALLY   ')
C
           CALL  SIZES(-1)
C
      m= 1
      IF( mode .EQ. 2 )  m= 3
C
      IF(    jr .LT.  1)     jr= 1
      IF(    jr .GE.  8)     jr= 8-1
      IF( Nruns .LT.  1)  Nruns= 1
      IF( Nruns .GE.  8)  Nruns= 8-1
      IF(    il .LT.  1)     il= 1
      IF(    il .GT.  3)     il= 3
      IF(    mk .LT.  1)     mk= 1
      IF(    mk .GT. 24)     mk= 24
C
      CALL  PAGE(iou)
      WRITE( iou, 99)
      WRITE( iou,100)
C
      DO 2 j= 1,Nruns
      WRITE( iou,102)  j, ( SEE(k,1,j,il), k= 1,4)
      T1(j)= SEE(1,1,j,il)
      i= 0
      IF( (SEE(3,2,j,il).LT. 0.01) .OR. (SEE(4,2,j,il).GT. 1.0))  i= i+1
      IF( (SEE(3,3,j,il).LT. 0.01) .OR. (SEE(4,3,j,il).GT. 1.0))  i= i+1
      IF( i.GT.0 )  THEN
      WRITE( iou,131)  j, il
      ENDIF
      IF( ( j.EQ.Nruns ) .OR. ( i.GT.0 ))  THEN
      WRITE( iou,104)  j, ( SEE(k,2,j,il), k= 1,4)
      WRITE( iou,104)  j, ( SEE(k,3,j,il), k= 1,4)
      ENDIF
    2 continue
C
      CALL STATS( S1, T1, Nruns)
      WRITE( iou,102)  Nruns, ( S1(k), k= 1,4)
C
C
C
      WRITE( iou,120) Nruns
      WRITE( iou,122)
      WRITE( iou,121)
      WRITE( iou,122)
C
      DO 8 k= 1,mk
        npft= 0
          cs= 0.0
C
      DO 4 j= 1,Nruns
        npft= npft +  NPFS(j,il,k)
          cs= cs   + CSUMS(j,il,k)
    4 continue
C
      CALL  STATS( S2, TIMES(1,il,k), Nruns)
      TIME(k)= S2(m)
      CSUM(k)= cs
      TERR1(k)= 100.0*( S2(2)/( S2(1) + 1.0e-9))
      T4(k)= TERR1(k)
C
C
C     If this clock resolution test fails, you must increase Loop (Subr. SIZES)
C
      CALL  STATS( S3, TERRS(1,il,k), Nruns)
         IF( S3(1) .GT. 15.0)  THEN
            WRITE( iou,113) k
         ENDIF
C
      WRITE( iou,123) k, S2(3), S2(1), S2(4), TERR1(k), S3(1), npft
      TERR1(k)= MAX( TERR1(k), S3(1))
      CALL  STATS( S1, DOS(1,il,k), Nruns)
      TOTAL(k)= S1(1)
           IF( (S1(1).LE.0.) .OR. (ABS(S1(3)-S1(4)).GT.1.0E-5)) THEN
           WRITE( iou,131) il, k, ( S1(k4), k4= 1,4)
           ENDIF
      CALL  STATS( S4, FOPN(1,il,k), Nruns)
      FLOPN(k)= S4(1)
           IF( (S4(1).LE.0.) .OR. (ABS(S4(3)-S4(4)).GT.1.0E-5)) THEN
           WRITE( iou,131) il, k, ( S4(k4), k4= 1,4)
           ENDIF
    8 continue
C
      WRITE( iou,122)
      CALL  STATS( S4, T4, mk)
      WRITE( iou,124)
      WRITE( iou,133)
      WRITE( iou,125)  ( S4(k), k= 1,4)
C
      CALL TRACKX
      RETURN
C
   99 FORMAT(//,29H CPU CLOCK OVERHEAD (t err):  )
  100 FORMAT(/,6X,3HRUN,8X,7HAVERAGE,8X,7HSTANDEV,8X,7HMINIMUM,8X,
     1 7HMAXIMUM )
  102 FORMAT(1X,5HTICK ,I3,4E15.6)
  104 FORMAT(1X,5HDATA ,I3,4E15.6)
  113 FORMAT(/,1X,I2,45H POOR CPU CLOCK RESOLUTION; NEED LONGER RUN. )
  120 FORMAT(//,39H THE EXPERIMENTAL TIMING ERRORS FOR ALL,I3,5H RUNS)
  121 FORMAT(55H  k   T min      T avg      T max    T err   tick   P-F)
  122 FORMAT(55H --  ---------  ---------  --------- -----  -----   ---)
  123 FORMAT(1X,I2,3E11.4,F6.2,1H%,F6.2,1H%,1X,I5)
  124 FORMAT(//,50H NET CPU TIMING VARIANCE (T err);  A few % is ok: )
  125 FORMAT(4X,5H Terr,4(F14.2,1H%))
  131 FORMAT(1X,25H** ERROR: INVALID DATA** ,2I6,4E14.6)
  133 FORMAT(/,17X,7HAVERAGE,8X,7HSTANDEV,8X,7HMINIMUM,8X,7HMAXIMUM )
      END
C
C***********************************************
      SUBROUTINE TDIGIT( derr, nzd, s )
C***********************************************************************
C                                                                      *
C     TDIGIT  -  Count Lead Digits Followed By Trailing Zeroes.        *
C                                                                      *
C       derr  -  Result,  Digital Error in percent.                    *
C        nzd  -  Result,  Number Of Lead Digits                        *
C          s  -  Input ,  A Floated Integer                            *
C                                                                      *
C***********************************************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
      frac(z)= (ABS(z) - DBLE(INT(ABS(z))))
C
      CALL TRACKS('TDIGIT  ')
C
            n= 14
            x= ABS(s)
         fuzz= 1.0e-6
         derr= 100.
          nzd= 0
           IF( x.EQ. 0.0)  GO TO 73
C                                  Normalize x
            y= LOG10(x)
            v= DBLE( 10**( ABS( INT(y)) + 1 ))
           IF( (y.GE. 0.0) .AND. (v.NE. 0.0))  v= 10./v
            x= x*v
C                                  Multiply x Until Trailing Digits= Fuzz
       DO 1 k= 1,n
           IF( ((1.-frac(x)).LE.fuzz) .OR. (frac(x).LE.fuzz)) GO TO 2
            x= 10.*x
    1 continue
C
    2      IF( x.NE. 0.0)  derr= 50./x
           IF( x.NE. 0.0)   nzd= INT( LOG10( ABS( 9.99999*x )))
C
   73 CONTINUE
      CALL TRACKX
      RETURN
      END
C
C***********************************************
      SUBROUTINE  TEST(i)
C***********************************************
C
C     TIME, TEST, AND INITIALIZE THE EXECUTION OF KERNEL i.
C
C*******************************************************************************
C
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
      REAL*8 SECOND
C
C
C/      PARAMETER( l1= 1001, l2=  101, l1d= 2*1001 )
C/      PARAMETER( l13=  64, l13h= l13/2, l213= l13+l13h, l813= 8*l13 )
C/      PARAMETER( l14=2048, l16=  75, l416= 4*l16 , l21= 25 )
C
C/      PARAMETER( kn= 47, kn2= 95, np= 3, ls= 3*47, krs= 24)
C
      COMMON /SPACES/ ion,j5,k2,k3,Loop,m,kr,it,n13h,ibuf,
     1                n,n1,n2,n13,n213,n813,n14,n16,n416,n21,nt1,nt2
C
      COMMON /SPACER/ A11,A12,A13,A21,A22,A23,A31,A32,A33,
     2                AR,BR,C0,CR,DI,DK,
     3  DM22,DM23,DM24,DM25,DM26,DM27,DM28,DN,E3,E6,EXPMAX,FLX,
     4  Q,QA,R,RI,S,SCALE,SIG,STB5,T,XNC,XNEI,XNM
C
      COMMON /SPACE0/ TIME(47), CSUM(47), WW(47), WT(47), ticks,
     1                FR(9), TERR1(47), SUMW(7), START,
     2              SKALE(47), BIAS(47), WS(95), TOTAL(47), FLOPN(47),
     3                IQ(7), NPF, NPFS1(47)
C
      COMMON /SPACEI/ WTP(3), MUL(3), ISPAN(47,3), IPASS(47,3)
C
      INTEGER    E,F,ZONE
      COMMON /ISPACE/ E(96), F(96),
     1  IX(1001), IR(1001), ZONE(300)
C
      COMMON /SPACE1/ U(1001), V(1001), W(1001),
     1  X(1001), Y(1001), Z(1001), G(1001),
     2  DU1(101), DU2(101), DU3(101), GRD(1001), DEX(1001),
     3  XI(1001), EX(1001), EX1(1001), DEX1(1001),
     4  VX(1001), XX(1001), RX(1001), RH(2048),
     5  VSP(101), VSTP(101), VXNE(101), VXND(101),
     6  VE3(101), VLR(101), VLIN(101), B5(101),
     7  PLAN(300), D(300), SA(101), SB(101)
C
C
      COMMON /SPACE2/ P(4,512), PX(25,101), CX(25,101),
     1  VY(101,25), VH(101,7), VF(101,7), VG(101,7), VS(101,7),
     2  ZA(101,7)  , ZP(101,7), ZQ(101,7), ZR(101,7), ZM(101,7),
     3  ZB(101,7)  , ZU(101,7), ZV(101,7), ZZ(101,7),
     4  B(64,64), C(64,64), H(64,64),
     5  U1(5,101,2),  U2(5,101,2),  U3(5,101,2)
C
C
C*******************************************************************************
C           t= second(0)  := cumulative cpu time for task in seconds.
C
       TEMPUS= SECOND(0.0D0) - START
C
cLLNL     call ENDPFM
C$C                           5 get number of page faults (optional)
C$      KSTAT= LIB$STAT_TIMER(5,KPF)
C$      NPF  = KPF - IPF
      CALL TRACKS('TEST    ')
C
           NN= n
           NP= Loop
         Loop= 0
           ik= i
           IF( i.LT.0 .OR. i.GT.kr )  GO TO 200
           IF( i.EQ.0 )  GO TO 100
         CALL  SIZES(i)
C     Net Time=  Timing - Overhead Time
      TIME(i)= TEMPUS - ticks
           IF( it.EQ.23456) GO TO 200
C
        GO TO(  1,  2,  3,  4,  5,  6,  7,  8,  9, 10,
     .         11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
     .         21, 22, 23, 24, 25                      ), i
C
C
C
C*******************************************************************************
C
    1 CSUM (1) =  SUMO ( X, n)
      TOTAL(1) =  NP*NN
      GO TO 100
C
C*******************************************************************************
C
    2 CSUM (2) =  SUMO ( X, n)
      TOTAL(2) =  NP*(NN-4)
      GO TO 100
C
C*******************************************************************************
C
    3 CSUM (3) =  Q
      TOTAL(3) =  NP*NN
      GO TO 100
C
C*******************************************************************************
C
    4        MM= (1001-7)/2
      DO 400 k = 7,1001,MM
  400      V(k)= X(k)
      CSUM (4) = SUMO ( V, 3)
      TOTAL(4) =  NP*(((NN-5)/5)+1)*3
      GO TO 100
C
C*******************************************************************************
C
    5 CSUM (5) =  SUMO ( X(2), n-1)
      TOTAL(5) =  NP*(NN-1)
      GO TO 100
C
C*******************************************************************************
C
    6 CSUM (6) =  SUMO ( W, n)
      TOTAL(6) =  NP*NN*((NN-1)/2)
      GO TO 100
C
C*******************************************************************************
C
    7 CSUM (7) =  SUMO ( X, n)
      TOTAL(7) =  NP*NN
      GO TO 100
C
C*******************************************************************************
C
    8 CSUM (8) = SUMO ( U1,5*n*2) + SUMO ( U2,5*n*2) + SUMO ( U3,5*n*2)
      TOTAL(8) =  NP*(NN-1)*2
      GO TO 100
C
C*******************************************************************************
C
    9 CSUM (9) =  SUMO ( PX, 15*n)
      TOTAL(9) =  NP*NN
      GO TO 100
C
C*******************************************************************************
C
   10 CSUM (10) =  SUMO ( PX, 15*n)
      TOTAL(10) =  NP*NN
      GO TO 100
C
C*******************************************************************************
C
   11 CSUM (11) =  SUMO ( X(2), n-1)
      TOTAL(11) =  NP*(NN-1)
      GO TO 100
C
C*******************************************************************************
C
   12 CSUM (12) =  SUMO ( X, n-1)
      TOTAL(12) =  NP*NN
      GO TO 100
C
C*******************************************************************************
C
   13 CSUM (13) =  SUMO ( P, 8*n) + SUMO ( H, 8*n)
      TOTAL(13) =  NP*NN
      GO TO 100
C
C*******************************************************************************
C
   14 CSUM (14) =  SUMO ( VX,n) + SUMO ( XX,n) + SUMO ( RH,67)
      TOTAL(14) =  NP*NN
      GO TO 100
C
C*******************************************************************************
C
   15 CSUM (15) =  SUMO ( VY, n*7) + SUMO ( VS, n*7)
      TOTAL(15) =  NP*(NN-1)*5
      GO TO 100
C
C*******************************************************************************
C
   16 CSUM (16) =  DBLE( k3+k2+j5+m)
      FLOPN(16) =  k2+k2+10*k3
      TOTAL(16) =  1.
      GO TO 100
C
C*******************************************************************************
C
   17 CSUM (17) =  SUMO ( VXNE, n) + SUMO ( VXND, n) + XNM
      TOTAL(17) =  NP*NN
      GO TO 100
C
C*******************************************************************************
C
   18 CSUM (18) =  SUMO ( ZR, n*7) + SUMO ( ZZ, n*7)
      TOTAL(18) =  NP*(NN-1)*5
      GO TO 100
C
C*******************************************************************************
C
   19 CSUM (19) =  SUMO ( B5, n) + STB5
      TOTAL(19) =  NP*NN
      GO TO 100
C
C*******************************************************************************
C
   20 CSUM (20) =  SUMO ( XX(2), n)
      TOTAL(20) =  NP*NN
      GO TO 100
C
C*******************************************************************************
C
   21 CSUM (21) =  SUMO ( PX, 25*n)
      TOTAL(21) =  NP*25*25*NN
      GO TO 100
C
C*******************************************************************************
C
   22 CSUM (22) =  SUMO ( W, n)
      TOTAL(22) =  NP*NN
      GO TO 100
C
C*******************************************************************************
C
   23 CSUM (23) =  SUMO ( ZA, n*7)
      TOTAL(23) =  NP*(NN-1)*5
      GO TO 100
C
C*******************************************************************************
C
   24 CSUM (24) =  DBLE(m)
      TOTAL(24) =  NP*(NN-1)
      GO TO 100
C
C*******************************************************************************
C
   25 CONTINUE
      GO TO 100
C
C*******************************************************************************
C
  100 CONTINUE
C
      CALL SIZES (i+1)
C             it = 23456
               IF( it.EQ.23456) GO TO 120
               IF( i.EQ.0     ) GO TO 120
            TERR0= 100.0
               IF( TEMPUS.NE. 0.0)  TERR0= TERR0*(ticks/TEMPUS)
         TERR1(i)= TERR0
         NPFS1(i)= NPF
               IF( ion .LE. 0 )  GO TO 120
C
C     If this clock resolution test fails, you must increase Loop (Subr. SIZES)
C
               IF( TERR0 .LT. 15.0)  GO TO 114
            WRITE( ion,113) I
  113 FORMAT(/,1X,I2,45H INACCURATE TIMING OR ERROR. NEED LONGER RUN )
C
  114      WRITE ( ion,115) i, TEMPUS, TERR0, NPF
  115      FORMAT( 2X,i2,9H Done  T= ,E11.4,8H  T err= ,F8.2,1H% ,
     1             I8,13H  Page-Faults )
C
  120 CALL VALUES(i)
      CALL SIZES (i+1)
C
C      The following pause can be used for stop-watch timing of each kernel.
C      You may have to increase the iteration count Loop in Subr. SIZES.
C
C/           pause
  200 CONTINUE
C
      IF(    jr .LT.  1)     jr= 1
      IF(    jr .GE.  8)     jr= 8-1
      IF( Nruns .LT.  1)  Nruns= 1
      IF( Nruns .GE.  8)  Nruns= 8-1
      IF(    il .LT.  1)     il= 1
      IF(    il .GT.  3)     il= 3
      IF(    mk .LT.  1)     mk= 1
      IF(    mk .GT. 24)     mk= 24
C
C
C$C                           5 get number of page faults (optional)
C$      NSTAT= LIB$STAT_TIMER(5,IPF)
cLLNL     call PFM( 0, ion)
C
         ik= i+1
      START= SECOND(0.0D0)
      CALL TRACKX
      RETURN
C
C$      DATA  IPF/0/, KPF/0/
      END
C
C***********************************************
      FUNCTION TICK( iou )
C***********************************************
C
C            TICK       measures timing overhead of subroutine test
C
C     iou    -  Logical Output Device Number                           *
C
C***********************************************
C
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
C
C/      PARAMETER( l1= 1001, l2=  101, l1d= 2*1001 )
C/      PARAMETER( l13=  64, l13h= l13/2, l213= l13+l13h, l813= 8*l13 )
C/      PARAMETER( l14=2048, l16=  75, l416= 4*l16 , l21= 25 )
C
C/      PARAMETER( kn= 47, kn2= 95, np= 3, ls= 3*47, krs= 24)
C
C/      PARAMETER( nk= 47, nl= 3, nr= 8 )
C
      COMMON /ALPHA/ mk,ik,ml,il,Nruns,jr, NPFS(8,3,47)
      COMMON /BETA / tic, TIMES(8,3,47), SEE(5,3,8,3),
     1              TERRS(8,3,47), CSUMS(8,3,47),
     2              FOPN(8,3,47), DOS(8,3,47)
C
C
      COMMON /SPACES/ ion,j5,k2,k3,Loop,m,kr,it,n13h,ibuf,
     1                n,n1,n2,n13,n213,n813,n14,n16,n416,n21,nt1,nt2
C
      COMMON /SPACER/ A11,A12,A13,A21,A22,A23,A31,A32,A33,
     2                AR,BR,C0,CR,DI,DK,
     3  DM22,DM23,DM24,DM25,DM26,DM27,DM28,DN,E3,E6,EXPMAX,FLX,
     4  Q,QA,R,RI,S,SCALE,SIG,STB5,T,XNC,XNEI,XNM
C
      COMMON /SPACE0/ TIME(47), CSUM(47), WW(47), WT(47), ticks,
     1                FR(9), TERR1(47), SUMW(7), START,
     2              SKALE(47), BIAS(47), WS(95), TOTAL(47), FLOPN(47),
     3                IQ(7), NPF, NPFS1(47)
C
      COMMON /SPACEI/ WTP(3), MUL(3), ISPAN(47,3), IPASS(47,3)
C
      INTEGER    E,F,ZONE
      COMMON /ISPACE/ E(96), F(96),
     1  IX(1001), IR(1001), ZONE(300)
C
      COMMON /SPACE1/ U(1001), V(1001), W(1001),
     1  X(1001), Y(1001), Z(1001), G(1001),
     2  DU1(101), DU2(101), DU3(101), GRD(1001), DEX(1001),
     3  XI(1001), EX(1001), EX1(1001), DEX1(1001),
     4  VX(1001), XX(1001), RX(1001), RH(2048),
     5  VSP(101), VSTP(101), VXNE(101), VXND(101),
     6  VE3(101), VLR(101), VLIN(101), B5(101),
     7  PLAN(300), D(300), SA(101), SB(101)
C
      COMMON /SPACE2/ P(4,512), PX(25,101), CX(25,101),
     1  VY(101,25), VH(101,7), VF(101,7), VG(101,7), VS(101,7),
     2  ZA(101,7)  , ZP(101,7), ZQ(101,7), ZR(101,7), ZM(101,7),
     3  ZB(101,7)  , ZU(101,7), ZV(101,7), ZZ(101,7),
     4  B(64,64), C(64,64), H(64,64),
     5  U1(5,101,2),  U2(5,101,2),  U3(5,101,2)
C
      DIMENSION TSEC(16),  MAP(47)
C
C
      CALL TRACKS('TICK    ')
C
      ion= iou
      kr = mk
      n  = 0
      Loop = 0
      k2 = 0
      k3 = 0
      it = 23456
      m  = 0
      ticks= 0.0
C
C
      CALL TEST(0)
      CALL TEST(1)
      CALL TEST(2)
      CALL TEST(3)
      CALL TEST(4)
      CALL TEST(5)
      CALL TEST(6)
      CALL TEST(7)
      CALL TEST(8)
      it = 0
C
      DO 10 k= 1,8
10    TSEC(k)= TIME(k)
C
              bl=  0.00D0
              bu= 10.00D0
      CALL  VALID( TIME,MAP,neff, bl, TSEC, bu, 8)
      CALL  STATS( SEE(1,1,jr,il), TIME, neff)
c     ticks= MAX( tic, SEE(1,1,jr,il))
      ticks= SEE(1,1,jr,il)
      TICK= ticks
C
      DO 20 k= 1,47
      TIME(k)= 0.0
      CSUM(k)= 0.0
20    CONTINUE
C
       CALL  STATS( SEE(1,2,jr,il), U, nt1)
       CALL  STATS( SEE(1,3,jr,il), P, nt2)
C
      mmin= 1
      mmax= 1001
      CALL IQRANF( IX, mmin, mmax, 1001)
C
         IF( iou.LE.0)  GO TO 73
      WRITE( iou, 99)
      WRITE( iou,100)
      WRITE( iou,102)  ( SEE(k,1,jr,il), k= 1,4)
      WRITE( iou,104)  ( SEE(k,2,jr,il), k= 1,4)
      WRITE( iou,104)  ( SEE(k,3,jr,il), k= 1,4)
C
   73 CONTINUE
      CALL TRACKX
      RETURN
C
   99 FORMAT(//,17H CLOCK OVERHEAD:  )
  100 FORMAT(/,14X,7HAVERAGE,8X,7HSTANDEV,8X,7HMINIMUM,8X,7HMAXIMUM )
  102 FORMAT(/,1X,5H TICK,4E15.6)
  104 FORMAT(/,1X,5H DATA,4E15.6)
      END
C
C***********************************************
      SUBROUTINE TILE( sm, si, OX,IX,W,ew,T,tiles,n)
C***********************************************
C
C     TILE       computes  m-tile value and corresponding index
C
C     sm      -  RESULT VALUE  IS m-TILE VALUE
C     si      -  RESULT VALUE  IS CORRESPONDING INDEX.r IN W
C
C     OX      -  INPUT  ARRAY  OF ORDERED (DECENDING) Xs.
C     IX      -  INPUT  ARRAY  OF INDEX LIST MAPS X TO OX.
C     W       -  INPUT  ARRAY  OF INPUT  WEIGHTS.
C     ew      -  INPUT  VALUE  FLAGS EQUAL WEIGHTS= 1.0; ELSE 0.0
C     T       -  INPUT  VALUE  IS SUM OF WEIGHTS
C     tiles   -  INPUT  VALUE  IS FRACTION OF RANGE, E.G. 0.25
C     n       -  INPUT  NUMBER OF INPUT  VALUES IN X.
C
C***********************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
      DIMENSION  OX(n), IX(n), W(n)
C
      CALL TRACKS('TILE    ')
C
       thresh= tiles*T + 0.50*ew*W(1)
            R= 0.0
      DO 70 k= 1,n
            S= R
            R= R + W( IX(k))
           IF( R .GT. thresh )  GO TO 7
   70 CONTINUE
            k= n
    7       z= 0.0
            y= 0.0
           IF( k.GT.1 )    y =   OX(k-1)
           IF( R.NE.S )    z = ( thresh - S)/( R - S)
           sm= y         + z * ( OX(k)  - y)
           si= DBLE(k-1) + z
C
      CALL TRACKX
      RETURN
      END
C
C***********************************************
      SUBROUTINE TRACE
C***********************************************
C
C     This routine is a convenient program break-point which is
C     selected by pre-setting:  match in COMMON /ORDER/  or by data
C     loading in BLOCK DATA  to equal the serial index of a
C     particular call to TRACKS, as previously recorded in NSTACK.
C     The call to TRACE is selected in subroutine TRACKS.
C
      RETURN
      END
C
C***********************************************
      SUBROUTINE TRACKS( name )
C***********************************************
C
C      Records /DEBUG/ info: sequence of called subroutine names
C      Enters current subroutine name on top of /DEBUG/ stack
C      NOT NECESSARY FOR PERFORMANCE TEST, MAY BE DUMMIED.
C
C      name   -  Input;  Callers name
C      ISTACK -  Contains names of subroutines in active linkage chain.
C
C                Interupt shows active chain of subr. names in ISTACK:
C      bkp kernel
C      run
C      BREAKPOINT REACHED AT 00417457PB = KERNEL:KERNEL+201PB
C      bkp trap
C      run
C      BREAKPOINT REACHED AT 00450122PB = TRAP:TRAP+45PB
C      sub= tracks  bcd  istack,10  dec  nstack,10
C
C      ISTACK(1) = "IQRANF  VALUES  TEST    KERNEL   MAIN.  "
C      NSTACK(1) =  164  162  157  65  1  0  0  0  0  0
C
C                Setting TRACKS call 5317 causes CALL to TRACE:
C      sub= tracks
C      set match= 5317
C      rel all.
C      bkp trace
C      run
C      BREAKPOINT REACHED AT 00440575PB = TRACE:TRACE+6PB
C      sub= tracks  bcd  istack,10  dec  nstack,10
C
C      ISTACK(1) = "SORDID  STATW   SENSIT  REPORT   MAIN.  "
C      NSTACK(1) =  5317  5316  5308  5282  1  0  0  0  0  0
C
C***********************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
      CHARACTER  name*8, ISTACK*8
      COMMON /DEBUG/     ISTACK(20)
      COMMON /ORDER/ index, match, NSTACK(20)
C
C                             pushdown stack of subroutine names
      DO  1  k = 10,2,-1
      NSTACK(k)= NSTACK(k-1)
      ISTACK(k)= ISTACK(k-1)
    1 continue
C
          index= index + 1
      NSTACK(1)= index
      ISTACK(1)= name
C
      IF( index.EQ.match ) THEN
          CALL TRACE
      ENDIF
C
      RETURN
      END
C
C***********************************************
      SUBROUTINE TRACKX
C***********************************************
C
C      Releases current subroutine name from top of /DEBUG/ stack
C      NOT NECESSARY FOR PERFORMANCE TEST, MAY BE DUMMIED.
C
C***********************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
      CHARACTER          ISTACK*8
      COMMON /DEBUG/     ISTACK(20)
      COMMON /ORDER/ index, match, NSTACK(20)
C
C                             pop stack of subroutine names
      DO  1  k = 1,9
      NSTACK(k)= NSTACK(k+1)
      ISTACK(k)= ISTACK(k+1)
    1 continue
C
      RETURN
      END
C
C***********************************************
      SUBROUTINE TRAP( I, name, mini, maxi,meff)
C***********************************************
C
C      Checks that Index List values are in valid domain
C
C     I     - ARRAY  OF INPUT INDEX-LIST
C     name  -           INPUT CALLERS name
C     mini  - INPUT SMALLEST INDEX VALUE
C     maxi  - INPUT LARGEST  INDEX VALUE
C     meff  - NUMBER OF INPUT VALUES IN I.
C
C***********************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
      COMMON /SPACES/ ion,j5,k2,k3,Loop,m,kr,it,n13h,ibuf,
     1                n,n1,n2,n13,n213,n813,n14,n16,n416,n21,nt1,nt2
      DIMENSION  I(meff)
      CHARACTER  name*8, ISTACK*8
      COMMON /DEBUG/     ISTACK(20)
      COMMON /ORDER/ index, match, NSTACK(20)
C
C
      CALL TRACKS('TRAP    ')
C
           L= 0
      DO 1 k= 1,meff
          IF( I(k).LT.mini .OR. I(k).GT.maxi )  L= k
    1 CONTINUE
C
C
          IF( L.NE.0 )   THEN
              io= ABS( ion)
              IF( io.LE.0 .OR. io.GT.100 ) io=6
              WRITE( io,110)  L, name
  110         FORMAT(////,' ERROR IN INDEX-LIST(',i4,')  IN SUBR:  ',A )
              WRITE( io,111)
  111         FORMAT(/,' ACTIVE SUBROUTINE LINKAGE CHAIN:')
              WRITE( io,112) ( ISTACK(k), NSTACK(k), k= 1,10 )
  112         FORMAT(10X,A  ,4X,I8)
              WRITE( io,113) I
  113         FORMAT(1X,10I6)
              STOP
          ENDIF
C
      CALL TRACKX
      RETURN
      END
C
C***********************************************
      SUBROUTINE VALID( VX,MAP,L,  BL,X,BU,n )
C***********************************************
C
C      Compress valid data sets;  form compression list.
C
C
C     VX    - ARRAY  OF RESULT COMPRESSED Xs.
C     MAP   - ARRAY  OF RESULT COMPRESSION INDICES
C     L     -           RESULT COMPRESSED LENGTH OF VX, MAP
C           -
C     BL    -           INPUT LOWER BOUND FOR VX
C     X     - ARRAY  OF INPUT VALUES.
C     BU    -           INPUT UPPER BOUND FOR VX
C     n     - NUMBER OF INPUT VALUES IN X.
C
C***********************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
      DIMENSION  VX(n), MAP(n), X(n)
CLLL. OPTIMIZE LEVEL G
C
      CALL TRACKS('VALID   ')
C
           m= 0
      DO 1 k= 1,n
              IF( X(k).LE. BL .OR. X(k).GE. BU )  GO TO 1
                     m= m + 1
                MAP(m)= k
                 VX(m)= X(k)
    1 CONTINUE
C
      L= m
      IF( m.GT.0 )  THEN
          CALL TRAP( MAP, 8H VALID   , 1, n,m)
      ENDIF
      CALL TRACKX
      RETURN
      END
C
C***********************************************
      SUBROUTINE VALUES(i)
C***********************************************
C
C            VALUES     initializes special values
C
C     i    :=  kernel number
C
C****************************************************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
      DOUBLE PRECISION  DS, DW
C
C/      PARAMETER( l1=   1001, l2=   101, l1d= 2*1001 )
C/      PARAMETER( l13= 64, l13h= 64/2, l213= 64+32, l813= 8*64 )
C/      PARAMETER( l14= 2048, l16= 75, l416= 4*75 , l21= 25)
C
C/      PARAMETER( kn= 47, kn2= 95, np= 3, ls= 3*47, krs= 24)
C
      COMMON /SPACES/ ion,j5,k2,k3,Loop,m,kr,it,n13h,ibuf,
     1                n,n1,n2,n13,n213,n813,n14,n16,n416,n21,nt1,nt2
C
      COMMON /SPACER/ A11,A12,A13,A21,A22,A23,A31,A32,A33,
     2                AR,BR,C0,CR,DI,DK,
     3  DM22,DM23,DM24,DM25,DM26,DM27,DM28,DN,E3,E6,EXPMAX,FLX,
     4  Q,QA,R,RI,S,SCALE,SIG,STB5,T,XNC,XNEI,XNM
C
      COMMON /SPACE0/ TIME(47), CSUM(47), WW(47), WT(47), ticks,
     1                FR(9), TERR1(47), SUMW(7), START,
     2              SKALE(47), BIAS(47), WS(95), TOTAL(47), FLOPN(47),
     3                IQ(7), NPF, NPFS1(47)
C
      COMMON /SPACEI/ WTP(3), MUL(3), ISPAN(47,3), IPASS(47,3)
C
      INTEGER    E,F,ZONE
      COMMON /ISPACE/ E(96), F(96),
     1  IX(1001), IR(1001), ZONE(300)
C
      COMMON /SPACE1/ U(1001), V(1001), W(1001),
     1  X(1001), Y(1001), Z(1001), G(1001),
     2  DU1(101), DU2(101), DU3(101), GRD(1001), DEX(1001),
     3  XI(1001), EX(1001), EX1(1001), DEX1(1001),
     4  VX(1001), XX(1001), RX(1001), RH(2048),
     5  VSP(101), VSTP(101), VXNE(101), VXND(101),
     6  VE3(101), VLR(101), VLIN(101), B5(101),
     7  PLAN(300), D(300), SA(101), SB(101)
C
      COMMON /SPACE2/ P(4,512), PX(25,101), CX(25,101),
     1  VY(101,25), VH(101,7), VF(101,7), VG(101,7), VS(101,7),
     2  ZA(101,7)  , ZP(101,7), ZQ(101,7), ZR(101,7), ZM(101,7),
     3  ZB(101,7)  , ZU(101,7), ZV(101,7), ZZ(101,7),
     4  B(64,64), C(64,64), H(64,64),
     5  U1(5,101,2),  U2(5,101,2),  U3(5,101,2)
C
      PARAMETER (KSIZE=8192)
      COMMON /SPACE3/ CACHE(KSIZE)
C
C     ******************************************************************
      CALL TRACKS('VALUES  ')
C
      IP1= i+1
C
      CALL  SUPPLY( i)
C
   13 IF( IP1.NE.13 ) GO TO 14
            DS= 1.000D0
            DW= 0.500D0
      DO 205 j= 1,4
      DO 205 k= 1,512
      P(j,k)  = DS
            DS= DS + DW
  205 CONTINUE
C
      DO 210 j= 1,96
      E(j) = 1
      F(j) = 1
  210 CONTINUE
C
   14 IF( IP1.NE.14) GO TO 16
C
      mmin= 1
      mmax= 1001
      CALL IQRANF( IX, mmin, mmax, 1001)
c
            DW= -100.000D0
      DO 215 J= 1,1001
      DEX(J) =  DW*DEX(J)
      GRD(J) = IX(J)
  215 CONTINUE
      FLX= 0.00100D0
C
   16 IF( IP1.NE.16 ) GO TO 50
CONDITIONS:
            MC= 2
            lr= n
            II= lr/3
            LB= II+II
            FW= 1.000D-4
          D(1)= 1.0198048642876400D0
      DO 400 k= 2,300
  400     D(k)= D(k-1) + FW/D(k-1)
             R= D(lr)
            FW= 1.000D0
      DO 403 L= 1,MC
             m= (lr+lr)*(L-1)
      DO 401 j= 1,2
      DO 401 k= 1,lr
             m= m+1
             S= DBLE(k)
       PLAN(m)= R*((S + FW)/S)
  401  ZONE(m)= k+k
  403 CONTINUE
             k= lr+lr+1
       ZONE(k)= lr
             S= D(lr-1)
             T= D(lr-2)
C
   50 CONTINUE
c               Clear the scalar Cache-memory with never used data-set.
c     fw= 1.000D0
c     CALL SIGNAL( CACHE, fw, 0.0, KSIZE)
c
      CACHE(1  )= 0.0
      CACHE(5  )= 0.0
      DO  520  k= 9,KSIZE,8
      CACHE(k  )= CACHE(k-4) + 0.1
      CACHE(k+4)= CACHE(k  ) + 0.1
  520 CONTINUE
C
      CALL TRACKX
      RETURN
      END
C
C***********************************************
      SUBROUTINE VERIFY( iou)
C***********************************************************************
C                                                                      *
C      VERIFY     auxiliary test routine to check-out function SECOND  *
C                 and to verify that sufficiently long Loop sizes are  *
C                 defined in Subr. SIZES for accurate CPU timing.      *
C                                                                      *
C       iou    -  Logical Output Device Number                         *
C                                                                      *
C***********************************************************************
      IMPLICIT  DOUBLE PRECISION (A-H,O-Z)
cIBM  IMPLICIT  REAL*8           (A-H,O-Z)
C
      REAL*8 SECOND
C
C/      PARAMETER( l1=   1001, l2=   101, l1d= 2*1001 )
C/      PARAMETER( l13= 64, l13h= 64/2, l213= 64+32, l813= 8*64 )
C/      PARAMETER( l14= 2048, l16= 75, l416= 4*75 , l21= 25)
C/      PARAMETER( kn= 47, kn2= 95, np= 3, ls= 3*47, krs= 24)
C
C
      COMMON /SPACE1/ U(1001), V(1001), W(1001),
     1  X(1001), Y(1001), Z(1001), G(1001),
     2  DU1(101), DU2(101), DU3(101), GRD(1001), DEX(1001),
     3  XI(1001), EX(1001), EX1(1001), DEX1(1001),
     4  VX(1001), XX(1001), RX(1001), RH(2048),
     5  VSP(101), VSTP(101), VXNE(101), VXND(101),
     6  VE3(101), VLR(101), VLIN(101), B5(101),
     7  PLAN(300), D(300), SA(101), SB(101)
C
      COMMON /SPACE2/ P(4,512), PX(25,101), CX(25,101),
     1  VY(101,25), VH(101,7), VF(101,7), VG(101,7), VS(101,7),
     2  ZA(101,7)  , ZP(101,7), ZQ(101,7), ZR(101,7), ZM(101,7),
     3  ZB(101,7)  , ZU(101,7), ZV(101,7), ZZ(101,7),
     4  B(64,64), C(64,64), H(64,64),
     5  U1(5,101,2),  U2(5,101,2),  U3(5,101,2)
C
      COMMON /ALPHA/ mk,ik,ml,il,Nruns,jr, NPFS(8,3,47)
C
      COMMON /SPACES/ ion,j5,k2,k3,Loop,m,kr,it,n13h,ibuf,
     1                n,n1,n2,n13,n213,n813,n14,n16,n416,n21,nt1,nt2
C
      COMMON /SPACEI/ WTP(3), MUL(3), ISPAN(47,3), IPASS(47,3)
C
C
      DIMENSION  TIM(20), TUM(20), TAV(20), TER(20), TMX(20)
      DIMENSION  LEN(20), SIG(20)
C
C
C
C
C
C
C
C
C
C
C
C
C
C****************************************************************************
C         VERIFY ADEQUATE Loop SIZE VERSUS CPU CLOCK ACCURACY
C****************************************************************************
C
C         VERIFY produced the following output on CRAY-XMP4 in a
C         fully loaded, multi-processing, multi-programming system:
C
C
C         VERIFY ADEQUATE Loop SIZE VERSUS CPU CLOCK ACCURACY
C         -----     -------     -------    -------   --------
C         EXTRA     MAXIMUM     DIGITAL    DYNAMIC   RELATIVE
C         Loop      CPUTIME     CLOCK      CLOCK     TIMING
C         SIZE      SECONDS     ERROR      ERROR     ERROR
C         -----     -------     -------    -------   --------
C             1  5.0000e-06      10.00%     17.63%     14.26%
C             2  7.0000e-06       7.14%      6.93%      4.79%
C             4  1.6000e-05       3.12%      6.56%      7.59%
C             8  2.8000e-05       1.79%      2.90%      2.35%
C            16  6.1000e-05       0.82%      6.72%      4.50%
C            32  1.1700e-04       0.43%      4.21%      4.62%
C            64  2.2700e-04       0.22%      3.13%      2.41%
C           128  4.4900e-04       0.11%      3.14%      0.96%
C           256  8.8900e-04       0.06%      2.06%      2.50%
C           512  1.7740e-03       0.03%      1.92%      1.59%
C          1024  3.4780e-03       0.01%      0.70%      1.63%
C          1360              Current Run:    Loop=   10.000*Loop
C          2048  7.0050e-03       0.01%      0.74%      1.28%
C          4096  1.3823e-02       0.00%      1.35%      0.78%
C         -----     -------     -------    -------   --------
C
C          Approximate Serial Job Time=   2.5e+01 Sec.    ( Nruns= 7 RUNS)
C
C****************************************************************************
c
      CALL TRACKS('VERIFY  ')
c
      DO  1 k = 1,101
          X(k)= 0.0
    1  CX(1,k)= 0.0
C
            t0= SECOND(0.0D0)
            to= SECOND(0.0D0) - t0
C
            il= 2
           CALL SIZES(12)
C
         loops= IPASS(12,il)*MUL(il)
         scale=  DBLE(Loop)/( DBLE(loops) - 1.0e-6)
                WRITE( iou,45)
                WRITE( iou,49)
                WRITE( iou,46)
                WRITE( iou,47)
                WRITE( iou,48)
                WRITE( iou,49)
   45 FORMAT(8X,51HVERIFY ADEQUATE Loop SIZE VERSUS CPU CLOCK ACCURACY)
   46 FORMAT(8X,'EXTRA     MAXIMUM     DIGITAL    DYNAMIC   RELATIVE')
   47 FORMAT(8X,'Loop      CPUTIME     CLOCK      CLOCK     TIMING  ')
   48 FORMAT(8X,'SIZE      SECONDS     ERROR      ERROR     ERROR   ')
   49 FORMAT(8X,'-----     -------     -------    -------   --------')
C
C
C****************************************************************************
C     Test Cpu Clock Timing Errors As A Function Of Loop Size(lo)
C****************************************************************************
C
             m= 0
            lo= 1
      DO 59  i= 1,18
            mj= Nruns+1
      DO 53  j= 1,mj
             n= 101-1
            t0= SECOND(0.0D0)
c                                    Time Kernel 12
      DO 12 L = 1,lo
      DO 12 k = 1,n
   12     X(k)= Y(k+1) - Y(k)
c
        TIM(j)= SECOND(0.0D0) - t0 - to
   53 continue
c                                    Compute Dynamic Clock Error
c
          CALL  STATS( TUM, TIM, mj)
         rterr= 100.0*( TUM(2)/( TUM(1) +1.0E-9))
            IF( TUM(1).LE. 0.0)  rterr= 100.0
c
c                                    Compute Digital Clock Error
c
          CALL  TDIGIT( SIG(i), nzd, TUM(4))
C
        TAV(i)= TUM(1)
        TMX(i)= TUM(4)
        TER(i)= rterr
        LEN(i)= lo
            lo= lo + lo
            IF( (i.GT.13) .AND. (rterr.LT. 4.0))  GO TO 60
            nn= i
   59 continue
C
C
C****************************************************************************
C     Test Timing Error By Comparing Time Of Each Run With Longest Run
C****************************************************************************
C
   60        m= 0
           tnn= ( TAV(nn+1) + 2.0* TAV(nn))* 0.5
          fuzz= 1.0e-9
            IF( tnn.LT.fuzz)  tnn= fuzz
      DO 69  i= 1,nn
         rterr= TER(i)
            lo= LEN(i)
c                                    Compute Relative Clock Error
c
            rt= 0.
            IF( LEN(i).GE. 0)     rt= LEN(nn+1)/LEN(i)
         rperr= 100.
            IF( tnn.GT. fuzz)  rperr= 100.*(ABS( tnn - rt* TAV(i)) /tnn)
         WRITE( iou,64) lo, TMX(i), SIG(i),rterr, rperr
   64   FORMAT(6X,I7,E12.4,F11.2,1H%,F10.2,1H%,F10.2,1H%)
c
c                                    Find Loop Size Used in Subr. SIZES
c
            IF( (Loop.GE.lo) .AND. (Loop.LE.2*lo))  THEN
                     m= lo
                WRITE( iou,66)  Loop, scale
                IF( rterr .GT. 10.0)  THEN
                  WRITE( iou, 67)
                  WRITE( iou, 68)
                ENDIF
   66 FORMAT(7X,i6,14X,21HCurrent Run:    Loop=, f9.3,5H*Loop )
   67 FORMAT(34X,44HINACCURATE TIMING OR ERROR. NEED LONGER RUN )
   68 FORMAT(34X,44HINCREASE:  Loop  OR  Nruns  IN SUBR. SIZES  )
            ENDIF
C
   69 continue
            IF( m.LE.0 )  THEN
                WRITE( iou,66)  Loop, scale
            ENDIF
                WRITE( iou,49)
C
C
C****************************************************************************
C     Estimate Approximate Job Time By Scaling Time For Kernel 11
C****************************************************************************
C
c                 Time Kernel 11.  Calibrate with static timing of machine code.
            lo= LEN(nn)
            t0= SECOND(0.0D0)
      DO 51 L = 1,lo
       PX(1,1)= CX(1,1) + DBLE(L)*1.0e-6
       PX(1,2)= CX(1,2)
CDIR$ NOVECTOR
      DO 51 k = 3,101
   51  PX(1,k)= PX(1,k-1) + PX(1,k-2)
c
       TIM(nn)= SECOND(0.0D0) - t0 - to
      tx=(TIM(nn)/( 101*LEN(nn) +1.0e-9))
      su= tx/3.2E-7
      estime= su*( 2.387 + Nruns*( 0.1616*scale + 1.534))
      WRITE( iou,70) estime, Nruns
   70 FORMAT(/,9X,28HApproximate Serial Job Time= ,E10.1,5H Sec.,
     1 4X,8H( Nruns=,i2,6H RUNS),/)
C
      CALL TRACKX
      RETURN
C
C****************************************************************************
C
C
C                   EXTRA CLOCK CALIBRATION TEST
C
C           Timing For A Very Long Run Versus External Clock Time.
C
C
C     VERIFY is an auxiliary test that is not needed to execute MFLOPS.
C     This optional routine may be called to verify the accuracy of
C     the function SECOND programmed above to time CPU execution.
C
C     In most cases, the calibration timing in the main program MFLOPS
C     will be sufficient, if the total CPU time measured by SECOND agrees
C     well with the CPU time charged by the operating system for several,
C     stand-alone computer runs. Stable timings should be demonstrated.
C
C     The method used below times Kernel 11 using SECOND and then
C     scales this timing up to predict the time for a very long run
C     of Kernel 11.  This long run is also timed using SECOND and
C     should be long enough to time independently using external, wall-clock
C     timings and/or CPU charge time measured by the operating system.
C     Function SECOND is verified by a close agreement of the
C     predicted total CPU time, the total CPU time measured by SECOND,
C     and the CPU time measured by the operating system.
C     To increase CPU run time simply increase the value of MORE.  The
C     total CPU time is then nearly equal to the CPU time for loop DO 99.
C
C
C     Print output from VERIFY on processor:  CRAY-1S/CFT 1.11
C
C          SECOND  time for          1 loops=   4.524000e-07 Sec.
C
C          Predict time for   10000000 loops=   4.524000e+00 Sec.
C
C          SECOND  time for   10000000 loops=   4.527072e+00 Sec.
C
C
C     Print output from VERIFY on processor:  CRAY-1S/CIVIC 130k
C
C          SECOND  time for          1 loops=   3.620000e-07 Sec.
C
C          Predict time for   10000000 loops=   3.620000e+00 Sec.
C
C          SECOND  time for   10000000 loops=   3.622626e+00 Sec.
C
C     The results of this test on both processors demonstrate a
C     consistant error of three in the fourth significant digit.
C     More important these SECOND measurements agree with external timings.
C     Consistant timing accuracy of two significant digits may be adequate.
C
C     The time for one execution pass can also be compared with a
C     timing analysis of the machine code for Kernel 11 for additional
C     verification of the accuracy of function SECOND.
C     Verification of CPU time measurements in Virtual Memory systems may
C     require this degree of circumspection.
C
C****************************************************************************
cC
c          MORE= 1000
c          loop= 100
c             m= loop*MORE
c             n= 101
c         nloop= loop*(n-1)
c        mnloop= m*(n-1)
c             r= 1.0/float(nloop)
c            t0= SECOND(0.0D0)
cC
c            t0= SECOND(0.0D0)
c      DO 11 L = 1,loop
cC      PX(1,1)= CX(1,1) + L*1.0e-25 use only if optimization distributes L-loop
c       PX(1,1)= CX(1,1)
c      DO 11 k = 2,n
c   11  PX(1,k)= PX(1,k-1) + CX(1,k)
c            tl= SECOND(0.0D0) - t0 - to
cC
c             i= 1
c            t1= tl*r
c            tp= tl*float(MORE)
c         WRITE( iou,15)       i,  t1
c   15   FORMAT(/,18H SECOND  time for ,I10,8H loops= ,E14.6,5H Sec.)
c         WRITE( iou,16)  mnloop,  tp
c   16   FORMAT(/,18H Predict time for ,I10,8H loops= ,E14.6,5H Sec.)
cC
cC****************************************************************************
cC
cC     The following loop should be set-up to run long enough for
cC     an accurate, independent CPU job timing which should be nearly
cC     equal to the total time predicted above (in WRITE-16) if the
cC     timing measured by SECOND is valid.
cC****************************************************************************
cC
c            t0= SECOND(0.0D0)
c      DO 99 L = 1,m
cC      PX(1,1)= CX(1,1) + L*1.0e-25 use only if optimization distributes L-loop
c       PX(1,1)= CX(1,1)
c      DO 99 k = 2,n
c   99  PX(1,k)= PX(1,k-1) + CX(1,k)
c            tm= SECOND(0.0D0) - t0 - to
cC
c         WRITE( iou,15)  mnloop,  tm
c      STOP
      END
